#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Objects = {};
var $O = Scada.Objects;

$O.autoUpdate = false;
$O.updateTime = ('<? cat /tmp/gs-groupmonitor.time 2>/dev/null ?>').trim();

$O.storeFields = [
  'id', 'name', 'address', 'datatype', 'comment', 'datadec', 'tagcache', 'disablelog',
  'highpriolog', 'export', 'visparams', 'updatetime', 'units', 'enums', 'filter_tp', 'filter_ip'
];

$O.store = new Ext.data.JsonStore({
  url: Scada.reqUrl('objects'),
  root: 'data',
  fields: $O.storeFields
});

$O.extObjStore = new Ext.data.ArrayStore({
  fields: [ 'id', 'text', 'datatype' ],
  data: []
});

$O.extNumObjStore = new Ext.data.ArrayStore({
  fields: [ 'id', 'text', 'datatype' ],
  data: []
});

$O.gridStore = new Ext.ux.data.PagingJsonStore({
  data: [],
  fields: $O.storeFields,
  sortInfo: {
    field: 'address',
    direction: 'ASC'
  }
});

$O.extStores = {};
$O.createExtStore = function(id, datatype) {
  var store = new Ext.data.ArrayStore({
    fields: [ 'id', 'text', 'datatype' ],
    data: [],
    datatype: datatype
  });


  $O.extStores[ id ] = store;
  return store;
}

$O.objectRenderer = function(id) {
  var obj = $O.store.getById(id), name = obj ? obj.data.name : '';
  return id ? (Scada.decodeGroupAddress(id) + ' ' + name) : '';
}

$O.updateList = {};
$O.updateTimers = {};

$O.dataTypes = new Ext.data.ArrayStore({
  fields: ['value', 'display', 'extraCls'],
  idIndex: 0,
  data: [
    ['',   $tr('objects_datatype_'), 'unspec' ],
    ['1',  $tr('objects_datatype_1'), 'main'],
    ['1001',  $tr('objects_datatype_1001'), 'sub'],
    ['1002',  $tr('objects_datatype_1002'), 'sub'],
    ['1003',  $tr('objects_datatype_1003'), 'sub'],
    ['1004',  $tr('objects_datatype_1004'), 'sub'],
    ['1005',  $tr('objects_datatype_1005'), 'sub'],
    ['1006',  $tr('objects_datatype_1006'), 'sub'],
    ['1007',  $tr('objects_datatype_1007'), 'sub'],
    ['1008',  $tr('objects_datatype_1008'), 'sub'],
    ['1009',  $tr('objects_datatype_1009'), 'sub'],
    ['1010',  $tr('objects_datatype_1010'), 'sub'],
    ['1011',  $tr('objects_datatype_1011'), 'sub'],
    ['1012',  $tr('objects_datatype_1012'), 'sub'],
    ['1013',  $tr('objects_datatype_1013'), 'sub'],
    ['1014',  $tr('objects_datatype_1014'), 'sub'],
    ['2',  $tr('objects_datatype_2'), 'main'],
    ['3',  $tr('objects_datatype_3'), 'main'],
    ['3007',  $tr('objects_datatype_3007'), 'sub'],
    ['4',  $tr('objects_datatype_4'), 'main'],
    ['5',  $tr('objects_datatype_5'), 'main'],
    ['5001',  $tr('objects_datatype_5001'), 'sub'],
    ['5003',  $tr('objects_datatype_5003'), 'sub'],
    ['6',  $tr('objects_datatype_6'), 'main'],
    ['7',  $tr('objects_datatype_7'), 'main'],
    ['8',  $tr('objects_datatype_8'), 'main'],
    ['9',  $tr('objects_datatype_9'), 'main'],
    ['9001',  $tr('objects_datatype_9001'), 'sub'],
    ['232', $tr('objects_datatype_232'), 'main'],
    ['232600', $tr('objects_datatype_232600'), 'sub'],
    ['10', $tr('objects_datatype_10'), 'main'],
    ['11', $tr('objects_datatype_11'), 'main'],
    ['12', $tr('objects_datatype_12'), 'main'],
    ['13', $tr('objects_datatype_13'), 'main'],
    ['14', $tr('objects_datatype_14'), 'main'],
    ['15', $tr('objects_datatype_15'), 'main'],
    ['16', $tr('objects_datatype_16'), 'main'],
    ['16002', $tr('objects_datatype_16002'), 'sub'],
    ['255', $tr('objects_datatype_255'), 'main']
  ]
});

$O.dtSelect = $def.select({
  fieldLabel: $tr('objects_datatype'),
  name: 'datatype',
  emptyText: $tr('objects_datatype_'),
  triggerAction: 'all',
  hiddenName: 'datatype',
  valueField: 'value',
  displayField: 'display',
  mode: 'local',
  tpl: '<tpl for="."><div class="x-combo-list-item scada-combo-item-{extraCls}">{display}</div></tpl>'
});

$O.pagination = new Ext.ux.PagingToolbar({
  pageSize: Config.perpage,
  store: $O.gridStore,
  displayMsg: $tr('objects_pages_hint'),
  emptyMsg: $tr('objects_pages_empty'),
  beforePageText: $tr('page'),
  afterPageText: $tr('page_of'),
  items: [
    { text: $tr('objects_add'), id: 'objects-add', iconCls: 'icon-add' }, '-',
    { text: $tr('objects_auto_update_on'), id: 'objects-auto-update', iconCls: 'icon-accept', enableToggle: true }, '-',
    { text: $tr('utilities_reset_button'), id: 'reset-objects', iconCls: 'icon-clear' }, '-',
    { text: $tr('objects_mass_edit'), id: 'objects-mass-edit', iconCls: 'icon-edit', hidden: true }
  ],
  extraItems: [
    '-', { xtype: 'tbtext', id: 'objects-filter-text', text: '' }
  ]
});

$O.filterReset = function() {
  $O.filters.name.setValue('');
  $O.filters.dt.setValue('');
  $O.filters.tags.setValue('');
  $O.filters.tags_mode.setValue('all');

  $O.filterBy = null;

  $O.gridStore.filterBy($O.filterCb);
};

$O.filterAddressEncode = function(item) {
  return Scada.encodeGroupAddress(item.replace(/\*/g, '1'));
};

$O.filterApply = function() {
  var name = $O.filters.name.getValue().trim(),
      dt = Ext.get('objects-filter-dt-hidden').getValue().trim(),
      tags = $O.filters.tags.getValue().trim(),
      chunks, address, regs;

  if (!name && !dt && !tags) {
    $O.filterBy = null;
  }
  else {
    $O.filterBy = {};

    if (name) {
      chunks = name.split(',');
      address = $O.filterAddressEncode(chunks[ 0 ]);

      if (address) {
        regs = [];

        Ext.each(chunks, function(chunk) {
          address = $O.filterAddressEncode(chunk);
          if (address) {
            regs.push('^' + chunk.replace(/\*/g, '[0-9]+') + '$');
          }
        });

        $O.filterBy.address = new RegExp(regs.join('|'), 'i');
      }
      else {
        $O.filterBy.name = new RegExp(Ext.escapeRe(name), 'i');
      }
    }

    if (dt) {
      $O.filterBy.dt = Scada.normalizeDt(dt);
    }

    if (tags) {
      $O.filterBy.tags = tags.toList();
      $O.filterBy.tagsAny = $O.filters.tags_mode.getValue().inputValue == 'any';
    }
  }

  $O.pagination.moveFirst();
  $O.gridStore.filterBy($O.filterCb);
};

$O.filterListener = {
  specialkey: function(field, event) {
    var key = event.getKey();
    if (key == event.ENTER) {
      $O.filterApply();
    }
    else if (key == event.ESC) {
      $O.filterReset();
    }
  }
};

$O.filter = new Ext.form.FormPanel({
  title: $tr('objects_filter'),
  region: 'west',
  collapsible: true,
  width: 188,
  margins: '10 10 10 10',
  cmargins: '10 10 10 10',
  labelAlign: 'top',
  padding: 10,
  defaults: {
    width: 166
  },
  items: [
    { id: 'objects-filter-name', xtype: 'textfield', fieldLabel: $tr('objects_filter_name_ga'), name: 'address', allowBlank: true, listeners: $O.filterListener },
    $extend($clone($O.dtSelect), { id: 'objects-filter-dt', hiddenId: 'objects-filter-dt-hidden', store: $O.dataTypes }),
    { id: 'objects-filter-tags', xtype: 'tagbox', fieldLabel: $tr('objects_filter_tags'), name: 'tags', allowBlank: true, listeners: $O.filterListener },
    {
      id: 'objects-filter-tags-mode',
      xtype: 'radiogroup',
      fieldLabel: $tr('objects_filter_tags_mode'),
      value: 'all',
      items: [
        { boxLabel: $tr('objects_filter_tags_mode_all'), name: 'tags_mode', inputValue: 'all' },
        { boxLabel: $tr('objects_filter_tags_mode_any'), name: 'tags_mode', inputValue: 'any' }
      ]
    },
    {
      xtype: 'container',
      layout: 'hbox',
      layoutConfig: {
        pack: 'end',
        defaultMargins: '5 0 0 5',
      },
      items: [
        { text: $tr('objects_filter_submit'), xtype: 'button', handler: $O.filterApply },
        { text: $tr('objects_filter_reset'), xtype: 'button', handler: $O.filterReset }
      ]
    }
  ]
});

$O.checkCallback = function(id, field, value) {
  $O.objectId = id;

  Ext.Ajax.request({
    url: Scada.reqUrl('objects', 'check'),
    params: Scada.encodeData({ id: id, field: field, value: value }),
    success: $O.update
  });
}

$O.checkScript = function(record) {
  var addr, found = false;

  addr = Scada.decodeGroupAddress(record.data.address);
  Ext.each($S.eStore.getAll(), function(item) {
    if (item.data.params == addr) {
      found = true;
    }
  });

  return found;
};

$O.grid = new Ext.grid.GridPanel({
  region: 'center',
  border: true,
  enableColumnHide: false,
  enableColumnMove: false,
  store: $O.gridStore,
  id: 'objects-grid',
  ctCls: 'scada-list-view-grid',
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('objects_address'), width: 0.07, dataIndex: 'address', renderer: Scada.decodeGroupAddress },
      { header: $tr('objects_name'), width: 0.18, dataIndex: 'name' },
      { header: $tr('objects_event_script'), width: 0.05, xtype: 'boolclscolumn', trueCls: Scada.editorCls, falseCls: (Scada.editorCls + '-disabled'), dataIndex: 'script', getValue: $O.checkScript, hidden: true },
      { header: $tr('objects_datatype'), width: 0.07, dataIndex: 'datatype', trPrefix: 'objects_datatype', xtype: 'translatecolumn' },
      { header: $tr('objects_value'), width: 0.07, dataIndex: 'datadec', xtype: 'knxdtcolumn' },
      { header: $tr('objects_filter_ip'), width: 0.05, dataIndex: 'filter_ip', xtype: 'checkcolumn', callback: $O.checkCallback },
      { header: $tr('objects_filter_tp'), width: 0.05, dataIndex: 'filter_tp', xtype: 'checkcolumn', callback: $O.checkCallback },
      { header: $tr('objects_disablelog'), width: 0.04, dataIndex: 'disablelog', xtype: 'checkcolumn', invert: true, callback: $O.checkCallback },
      { header: $tr('objects_export'), width: 0.04, dataIndex: 'export', xtype: 'checkcolumn', callback: $O.checkCallback },
      { header: $tr('objects_tags'), width: 0.09, dataIndex: 'tagcache', hidden: true },
      { header: $tr('objects_updatetime'), width: 0.09, dataIndex: 'updatetime', xtype: 'timestampcolumn' },
      { header: $tr('objects_setvalue'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-object-edit', dataIndex: 'setvalue' },
      { header: $tr('objects_vis_params_short'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-object-vis', trueCls: 'scada-cell-object-vis', dataIndex: 'visparams', hidden: true },
      { header: $tr('objects_enums_short'), width: 0.05, xtype: 'boolclscolumn', trueCls: 'scada-cell-customvals', falseCls: 'scada-cell-customvals', dataIndex: 'enums', hidden: true },
      { header: '', width: 0.03, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: $O.pagination,
  margins: '10 10 10 0',
  viewConfig: {
    forceFit: true,
    markDirty: false,
    getRowClass: function(record) {
      var cls, ut, pt, id = record.id;

      ut = record.data.updatetime.toNumber();
      pt = $O.updateList[ id ] ? $O.updateList[ id ] : ut;

      if (pt != ut || $O.updateTimers[ id ]) {
        cls = 'objects-row-updated';

        if (pt != ut) {
          $O.updateTimers[ id ] = 5;
        }
      }
      else if (!record.data.name || !record.data.datatype) {
        cls = 'objects-row-unspec';
      }
      else {
        cls = 'objects-row-normal';
      }

      $O.updateList[ id ] = ut;
      return cls;
    }
  },
  sm: new Ext.grid.RowSelectionModel({ singleSelect: true })
});

$O.addrMenu = new Ext.menu.Menu({ width: 250, items: [] });
$O.fieldMenu = new Ext.menu.Menu({ width: 250, items: [] });

$O.massEditCompare = function(prev, curr) {
  Ext.iterate(prev, function(key, value) {
    if (value != curr[ key ]) {
      prev[ key ] = '';
    }
  });

  return prev;
};

$O.prepareForm = function(single) {
  $O.form.findField('datatype').allowBlank = !single;
  $O.form.findField('data').setVisible(single);
  $O.form.findField('name').setVisible(single).setDisabled(!single);
  $O.form.findField('address').setVisible(single).setDisabled(true);
  $O.form.findField('address_list_menu').setVisible(!single).setDisabled(single);
  $O.form.findField('field_list_menu').setVisible(!single).setDisabled(single);
  $O.form.findField('address_list').setDisabled(single);
  $O.form.findField('field_list').setDisabled(single);

  $O.form.reset();
  $O.formWindow.show().center();
  $O.formWindow.setHeight(single ? 450 : 420);
  $O.formWindow.setTitle($tr(single ? 'objects_form' : 'objects_mass_edit'));

  $O.massEditShowFields(single);
};

$O.massEditFields = fields = [
  'datatype', 'tags', 'units', 'disablelog',
  'highpriolog', 'export', 'pollinterval', 'comment'
];

$O.massEditShowFields = function(single) {
  var value = $O.formFieldList.getValue()
    , checked = value.split(',');
  single = single === true;

  Ext.each($O.massEditFields, function(field) {
    var show = checked.indexOf(field) >= 0;
    $O.form.findField(field).setVisible(single || show);
  });

  $O.formFieldList[ value == '' ? 'markInvalid' : 'clearInvalid' ]();
};

$O.massEditInit = function() {
  Ext.each($O.massEditFields, function(field) {
    $O.fieldMenu.addItem({
      xtype: 'menucheckitem',
      hideOnClick: false,
      text: $tr('objects_' + field),
      value: field
    });
  });

  $O.fieldMenu.on('hide', $O.massEditShowFields);
  $O.addrMenu.on('hide', function() {
    var fn = $O.formAddressList.getValue() == '' ? 'markInvalid' : 'clearInvalid';
    $O.formAddressList[ fn ]();
  });
};

$O.massEdit = function() {
  var items = $O.gridStore.allData.items, cmp;

  $O.prepareForm(false);

  $O.addrMenu.removeAll();
  Ext.each(items, function(item) {
    cmp = cmp ? $O.massEditCompare(cmp, item.data) : $clone(item.data);

    $O.addrMenu.addItem({
      xtype: 'menucheckitem',
      hideOnClick: false,
      checked: true,
      text: Scada.decodeGroupAddress(item.id) + ' ' + (item.data.name || ''),
      value: item.id
    });
  });

  if (cmp) {
    cmp.disablelog = !cmp.disablelog;

    if ($O.rawTags) {
      Ext.getCmp('objects-form-tags').updateTags($O.rawTags);
    }

    cmp.tags = cmp.tagcache || '';
    $O.form.setValues(cmp);
  }

  $O.formAddressList.onMenuHide();
  $O.formFieldList.onMenuHide();
};

$O.formWindow = new Ext.Window(
  $def.window({
    title: $tr('objects_form'), width: 490, height: 420,
    items: $def.form({
      id: 'objects-form',
      url: Scada.reqUrl('objects', 'save'),
      labelWidth: 180,
      buttons: $def.formButtons('objects-form'),
      items: [
        { fieldLabel: $tr('objects_name'), name: 'name', allowBlank: false, invalidText: $tr('objects_error_name') },
        { fieldLabel: $tr('objects_address'), name: 'address', id: 'objects-form-address', invalidText: $tr('objects_error_address'), allowBlank: false, vtype: 'knxga' },
        { fieldLabel: $tr('objects_field_list'), name: 'field_list_menu', id: 'objects-form-field-list', xtype: 'menutriggerfield', allowBlank: false, editable: false,
          menu: $O.fieldMenu, countText: $tr('objects_field_list_count') },
        { fieldLabel: $tr('objects_address_list'), name: 'address_list_menu', id: 'objects-form-address-list', xtype: 'menutriggerfield', allowBlank: false, editable: false,
          menu: $O.addrMenu, countText: $tr('objects_address_list_count') },
        $extend($O.dtSelect, { store: $O.dataTypes, allowBlank: false }),
        { fieldLabel: $tr('objects_value'), name: 'data', xtype: 'displayfield', id: 'objects-form-data' },
        { fieldLabel: $tr('objects_tags'), name: 'tags', xtype: 'tagbox', id: 'objects-form-tags', editable: false },
        { fieldLabel: $tr('objects_units'), name: 'units' },
        { fieldLabel: $tr('objects_disablelog'), name: 'disablelog', xtype: 'checkbox' },
        { fieldLabel: $tr('objects_highpriolog'), name: 'highpriolog', xtype: 'checkbox' },
        { fieldLabel: $tr('objects_export'), name: 'export', xtype: 'checkbox' },
        { fieldLabel: $tr('objects_pollinterval'), name: 'pollinterval', xtype: 'spinnerfield', allowBlank: true, minValue: 0, maxValue: 3600, width: 60, editable: false },
        { fieldLabel: $tr('objects_comment'), name: 'comment', xtype: 'textarea', height: 90 },
        { name: 'address_list', xtype: 'hidden' },
        { name: 'field_list', xtype: 'hidden' },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$O.container = new Ext.Container({
  id: 'Objects',
  title: $tr('objects_title'),
  layout: 'border',
  items: [ $O.filter, $O.grid ]
});

$O.submitForm = function() {
  $O.form.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$O.showForm = function(id) {
  $O.prepareForm(true);
  $O.form.objectId = id;

  if (id) {
    $O.form.load({
      url: Scada.reqUrl('objects', 'form'),
      params: Scada.encodeData({ id: id }),
      waitMsg: $tr('loading')
    });
  }
  else {
    $O.form.findField('disablelog').setValue(Config.logpolicy == 'accept');

    if ($O.rawTags) {
      Ext.getCmp('objects-form-tags').updateTags($O.rawTags);
    }
  }
};

$O.hideForm = function() {
  $O.formWindow.hide();
};

$O.update = function(full) {
  var params = {};

  if (full === true) {
    $O.store.removeAll();
    $O.gridStore.removeAll();
  }
  else {
    if ($O.updateTime) {
      params.updatetime = $O.updateTime;
    }

    if ($O.objectId) {
      params.objectid = $O.objectId;
      $O.objectId = null;
    }
  }

  $O.store.load({ add: true, params: params });
};

$O.deleteItem = function(id) {
  var record;

  record = $O.store.getById(id);
  record && $O.store.remove(record);

  record = $O.gridStore.getById(id);
  record && $O.gridStore.remove(record);
}

$O.onConfirm = function(res) {
  if (res == 'yes') {
    $O.deleteItem(this.id);

    Ext.Ajax.request({
      url: Scada.reqUrl('objects', 'delete'),
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$O.showSetValueFormRow = function(row) {
  Scada.showSetValueForm(row.data);
};

$O.checkUpdate = function() {
  Ext.Ajax.request({
    url: Scada.genUrl('objects-update'),
    method: 'GET',
    disableCaching: false,
    params: { upd: $O.updateTime },
    success: $O.checkUpdateCb,
    failure: $O.checkUpdateCb,
    timeout: 60 * 1000
  });
};

$O.listeners = {};

$O.addListener = function(id, fn) {
  $O.listeners[ id ] = fn;
};

$O.removeListener = function(id) {
  delete $O.listeners[ id ];
};

$O.notify = function() {
  var id;
  for (id in $O.listeners) {
    $O.listeners[ id ]();
  }
};

$O.checkUpdateCb = function(result) {
  if (result && result.responseText) {
    var updateTime = result.responseText.trim();

    if (updateTime != $O.updateTime) {
      if ($O.autoUpdate) {
        $O.update();
      }

      $O.updateTime = updateTime;
      $O.notify();
    }
  }

  $O.checkUpdate.defer(1000);
};

$O.filterCb = function(r) {
  if (!$O.filterBy) {
    return true;
  }

  var data = r.data, dt = $O.filterBy.dt;

  if (dt) {
    var odt = Scada.normalizeDt(data.datatype || 0);

    if (odt.major != dt.major) {
      return false;
    }

    if (dt.minor && odt.minor != dt.minor) {
      return false;
    }
  }

  if ($O.filterBy.name && !$O.filterBy.name.test(data.name)) {
    return false;
  }

  if ($O.filterBy.address) {
    var addr = Scada.decodeGroupAddress(data.id);
    if (!addr.match($O.filterBy.address)) {
      return false;
    }
  }

  if ($O.filterBy.tags) {
    var matches = 0, search = $O.filterBy.tags, tags = data.tagcache.toList();

    Ext.each(search, function(tag) {
      if (tags.indexOf(tag) >= 0) {
        matches++;
      }
    });

    if (matches == 0) {
      return false;
    }
    else if (!$O.filterBy.tagsAny && matches != search.length) {
      return false;
    }
  }

  return true;
};

$O.submitVisParams = function(fw, action, form) {
  $O.objectId = form.findField('id').getValue();
  $O.update();
}

$O.visParamsFW_bool = new ExtFormWindow({
  id: 'objects-vis-params-bool',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 150
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 160,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      $def.select({
        name: 'control',
        hiddenName: 'control',
        fieldLabel: $tr('objects_vis_params_control'),
        store: [
          [ 'toggle', $tr('objects_vis_params_control_toggle') ],
          [ 'checkbox', $tr('objects_vis_params_control_checkbox') ]
        ]
      })
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.visParamsFW_bit4 = new ExtFormWindow({
  id: 'objects-vis-params-bit4',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 150
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 160,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      $def.select({
        name: 'step',
        hiddenName: 'step',
        fieldLabel: $tr('objects_vis_params_bit4_step'),
        store: [ [ '7', '1%' ], [ '6', '3%' ], [ '5', '6%' ], [ '4', '12%' ], [ '3', '25%' ], [ '2', '50%' ], [ '1', '100%' ] ]
      })
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.visParamsFW_numeric = new ExtFormWindow({
  id: 'objects-vis-params-numeric',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 430
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 160,
    defaults: { width: 180 },
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      { xtype: 'spinnerfield', name: 'decimals', width: 50, fieldLabel: $tr('objects_vis_params_decimals'), minValue: 0, maxValue: 10, allowDecimals: false, hidden: true },
      $def.select({
        id: 'objects-vis-params-numeric-control',
        name: 'control',
        hiddenName: 'control',
        fieldLabel: $tr('objects_vis_params_control'),
        store: [
          [ 'spinner', $tr('objects_vis_params_control_step') ],
          [ 'slider', $tr('objects_vis_params_control_slider') ],
          [ 'circularslider', $tr('objects_vis_params_control_circular_slider') ],
          [ 'vselect', $tr('objects_vis_params_control_vselect') ]
        ]
      }),
      { xtype: 'checkbox', name: 'icons', fieldLabel: $tr('objects_vis_params_icons'), inputValue: 1, hidden: true },
      { xtype: 'spinnerfield', name: 'min', width: 100, fieldLabel: $tr('objects_vis_params_min') },
      { xtype: 'spinnerfield', name: 'max', width: 100, fieldLabel: $tr('objects_vis_params_max') },
      { xtype: 'spinnerfield', name: 'step', width: 100, fieldLabel: $tr('objects_vis_params_step') },
      { xtype: 'checkbox', name: 'vertical', fieldLabel: $tr('objects_vis_params_vertical'), boxLabel: $tr('objects_vis_params_vertical_hint'), inputValue: 1, hidden: true },
      { xtype: 'checkbox', name: 'invert', fieldLabel: $tr('objects_vis_params_invert'), inputValue: 1, hidden: true },
      { xtype: 'colorfield', name: 'fgcolor', width: 100, fieldLabel: $tr('objects_vis_params_fgcolor'), hidden: true },
      { xtype: 'colorfield', name: 'bgcolor', width: 100, fieldLabel: $tr('objects_vis_params_bgcolor'), hidden: true  },
      { xtype: 'checkbox', name: 'roundcap', fieldLabel: $tr('objects_vis_params_roundcap'), inputValue: 1, hidden: true },
      { xtype: 'checkbox', name: 'hidetitle', fieldLabel: $tr('objects_vis_params_hidetitle'), inputValue: 1, hidden: true },
      { xtype: 'checkbox', name: 'hideminmax', fieldLabel: $tr('objects_vis_params_hideminmax'), inputValue: 1, hidden: true },
      { xtype: 'compositefield', fieldLabel: $tr('objects_vis_params_linewidth'), items: [
          { xtype: 'spinnerfield', name: 'linewidth', minValue: 1, maxValue: 50, value: 15, width: 55, allowDecimals: false },
          { xtype: 'label', text: '%', cls: 'x-form-cb-label' }
        ]
      },
      { xtype: 'compositefield', fieldLabel: $tr('objects_vis_params_size'), items: [
          { xtype: 'spinnerfield', name: 'size', minValue: 150, maxValue: 500, value: 200, width: 55, allowDecimals: false },
          { xtype: 'label', text: $tr('objects_vis_params_size_hint'), cls: 'x-form-cb-label' }
        ]
      }
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.visParamsFW_rgb = new ExtFormWindow({
  id: 'objects-vis-params-rgb',
  window: {
    title: $tr('objects_vis_params'),
    width: 400,
    height: 325
  },
  form: {
    url: Scada.reqUrl('objects', 'setvisparams'),
    labelWidth: 160,
    items: [
      { xtype: 'hidden', name: 'id' },
      { xtype: 'displayfield', name: 'name', fieldLabel: $tr('objects_obj_title') },
      { xtype: 'checkbox', name: 'change', fieldLabel: $tr('objects_vis_params_change'), inputValue: 1 },
      $def.select({
        id: 'objects-vis-params-rgb-presets',
        name: 'presets',
        hiddenName: 'presets',
        width: 80,
        fieldLabel: $tr('objects_vis_params_presets'),
        store: [ [0, 0], [1, 1], [2, 2], [3, 3], [4, 4], [5, 5], [6, 6] ]
      }),
      { xtype: 'colorfield', name: 'preset1', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 1' },
      { xtype: 'colorfield', name: 'preset2', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 2' },
      { xtype: 'colorfield', name: 'preset3', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 3' },
      { xtype: 'colorfield', name: 'preset4', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 4' },
      { xtype: 'colorfield', name: 'preset5', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 5' },
      { xtype: 'colorfield', name: 'preset6', width: 80, fieldLabel: $tr('objects_vis_params_preset') + ' 6' }
    ]
  },
  events: { onFormSubmitComplete: $O.submitVisParams, onFormSubmitFailed: $O.submitVisParams }
});

$O.showVisParams = function(obj) {
  var dt = Scada.normalizeDt(obj.datatype), win, form, show, i, key
    , values = { id: obj.id }, params, bounds
    , num = Scada.dtIsNumeric(dt.major)
    , rgb = Scada.isRgb(dt);

  values.name = Scada.decodeGroupAddress(obj.address);
  if (obj.name) {
    values.name = obj.name + ' (' + values.name + ')';
  }

  params = Ext.decode(obj.visparams) || {};

  if (dt.major == Scada.dt.bool) {
    win = $O.visParamsFW_bool;
    values.control = params.control || 'toggle';
  }
  else if (dt.major == Scada.dt.bit4) {
    win = $O.visParamsFW_bit4;
    values.step = params.step || 3;
  }
  else if (rgb) {
    win = $O.visParamsFW_rgb;
    values.change = !!params.change;
    values.presets = params.presets || 0;

    for (i = 0; i <= 6; i++) {
      key = 'preset' + i;
      values[ key ] = params[ key ] || '#FFFFFF';
    }

    num = false;
  }
  else if (num) {
    win = $O.visParamsFW_numeric;

    bounds = Scada.dtGetBounds(obj.datatype);
    values.min = params.min || bounds[ 0 ];
    values.max = params.max || bounds[ 1 ];
    values.step = params.step || '';
    values.decimals = params.decimals || '';
    values.control = params.control || (dt.major == Scada.dt.uint8 && dt.minor == Scada.minorDt.scale ? 'slider' : 'spinner');
    values.fgcolor = params.fgcolor || '#51A3CC';
    values.bgcolor = params.bgcolor || '#EEEEEE';
    values.vertical = !!params.vertical;
    values.invert = !!params.invert;
    values.roundcap = !!params.roundcap;
    values.hidetitle = !!params.hidetitle;
    values.hideminmax = !!params.hideminmax;
    values.linewidth = params.linewidth || 15;
    values.size = params.size || 200;
    values.icons = !!params.icons;
  }

  if (win) {
    form = win.formPanel.getForm();

    form.reset();

    if (num) {
      form.findField('min').setMinValue(bounds[ 0 ]);
      form.findField('min').setMaxValue(bounds[ 1 ]);

      form.findField('max').setMinValue(bounds[ 0 ]);
      form.findField('max').setMaxValue(bounds[ 1 ]);

      show = Scada.isFloat(dt);
      form.findField('decimals').setVisible(show).setDisabled(!show);
    }

    win.show();
    form.setValues(values);

    if (num) {
      $O.setNumericControl();
    }
    else if (rgb) {
      $O.setRgbControl();
    }
  }
  else {
    Ext.Msg.alert(
      $tr('objects_vis_params'),
      $tr('objects_vis_params_alert')
    );
  }
}

$O.enumsId = 0;

$O.enumsDeleteRow = function(cmp) {
  var fw = $O.enumsWindow;

  if (cmp.items) {
    cmp.items.each(function(item) {
      fw.form.remove(item, true);
    });
  }

  fw.form.remove(cmp, true);
  fw.formPanel.remove(cmp, true);
}

$O.enumsDeleteItem = function() {
  var id = this.id.replace('_delete', ''), cmp = Ext.getCmp(id);
  $O.enumsDeleteRow(cmp);

  if ($O.enumsButton) {
    $O.enumsButton.show();
  }
}

$O.enumsSubmit = function(fw) {
  var values, store = {}, enums = {}, count = 0;

  if (fw.form.isValid()) {
    values = fw.form.getValues();

    Ext.iterate(values, function(key, value) {
      var chunks, name, id;

      if (key == 'default') {
        enums['default'] = value;
      }
      else {
        chunks = key.split('_');
        name = chunks[0];
        id = chunks[1];

        if (!store[ id ]) {
          store[ id ] = {};
        }

        store[ id ][ name ] = name == 'text' ? value : value.toNumber();
      }
    });

    Ext.iterate(store, function(id, values) {
      count++;
      enums[ values.value ] = values.text;
    });

    fw.window.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('objects', 'setenums'),
      params: Scada.encodeData({
        id: $O.enumsObj.id,
        enums: (count ? Ext.encode(enums) : '')
      }),
      success: function() {
        $O.objectId = $O.enumsObj.id;
        $O.update();

        fw.window.setDisabled(false);
        fw.hide();
      }
    });
  }
}

$O.enumsAddItem = function(value, text) {
  var id = $O.enumsId, fp = $O.enumsWindow.formPanel, items = fp.items, index = items.length - 1, bounds, prev, max;
  bounds = Scada.dtGetBounds($O.enumsObjDt);

  if (value == 'default') {
    $O.enumsWindow.form.setValues({ 'default': text });
    return;
  }
  else if (typeof value != 'string' && typeof value != 'number') {
    value = 0;
    text = '';

    if (index > 0) {
      try {
        items = items.items[ index - 1 ].items.items;
        prev = items[ 1 ].getValue().toNumber();

        if (prev !== null) {
          value = prev + 1;
        }
      }
      catch (e) {}
    }
  }

  value = value.toNumber() || 0;
  value = value.constrain(bounds[0], bounds[1]);

  fp.insert(index, {
    id: 'objects_enums_row_' + id,
    xtype: 'compositefield',
    hideLabel: true,
    width: 440,
    items: [
      { xtype: 'displayfield', value: $tr('objects_enums_value'), width: 100 },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: false, name: 'value_' + id, width: 80, fieldLabel: $tr('vis_min_value'), value: value },
      { xtype: 'displayfield', value: $tr('objects_enums_text') },
      { xtype: 'textfield', name: 'text_' + id, value: text },
      { xtype: 'button', iconCls: 'icon-delete', width: 16, id: 'objects_enums_row_' + id + '_delete', handler: $O.enumsDeleteItem }
    ]
  });

  $O.enumsId++;
  max = bounds[ 1 ] - bounds[ 0 ];
  if (max < index) {
    $O.enumsButton.hide();
  }
}

$O.enumsShow = function(obj) {
  var fp = $O.enumsWindow.formPanel, dt = Scada.normalizeDt(obj.datatype), items;

  if ((dt.major == Scada.dt.bool || Scada.dtIsNumeric(dt.major)) && !Scada.isFloat(dt) && !Scada.isRgb(dt)) {
    $O.enumsObj = obj;
    $O.enumsObjDt = dt;
    $O.enumsWindow.show();

    fp.items.each($O.enumsDeleteRow);
    fp.add({ xtype: 'textfield', name: 'default', fieldLabel: $tr('objects_enums_default'), width: 145 });

    $O.enumsButton = fp.add({
      xtype: 'button',
      text: $tr('objects_enums_add'),
      handler: function() {
        $O.enumsAddItem();
        fp.doLayout();
      },
      iconCls: 'icon-add',
      width: 'auto',
      cls: 'btn-bottom'
    });

    items = Ext.decode(obj.enums);
    if (typeof items == 'object') {
      Ext.iterate(items, $O.enumsAddItem);
    }

    fp.doLayout();
  }
  else {
    Ext.Msg.alert(
      $tr('objects_enums'),
      $tr('objects_enums_error')
    );
  }
}

$O.enumsWindow = new ExtFormWindow({
  id: 'scada-objects-enums-window',
  window: {
    title: $tr('objects_enums'),
    width: 500,
    height: 400
  },
  form: {
    autoScroll: true,
    labelWidth: 100,
    items: []
  },
  events: { onSubmit: $O.enumsSubmit }
});

$O.onBlur = function() {
  var items, objData = [], numObjData = [], extStoreData = {};
  items = $O.store.getRange().sort(function(a, b) { return a.id - b.id; });

  $O.extObjStore.removeAll();
  $O.extNumObjStore.removeAll();

  Ext.iterate($O.extStores, function(id, store) {
    extStoreData[ id ] = [];
    store.removeAll();
  });

  Ext.each(items, function(item) {
    var data = item.data, dt, row, text;

    text = Scada.decodeGroupAddress(data.id);
    if (data.name) {
      text += ' ' + data.name;
    }

    row = [ data.id, text, data.datatype ];
    objData.push(row);

    if (data.datatype) {
      dt = Scada.normalizeDt(data.datatype);

      if (Scada.dtIsNumeric(dt.major)) {
        numObjData.push(row);
      }

      Ext.iterate($O.extStores, function(id, store) {
        if (store.datatype && store.datatype == dt.major) {
          extStoreData[ id ].push(row);
        }
      });
    }
  });

  $O.extObjStore.loadData(objData);
  $O.extNumObjStore.loadData(numObjData);

  Ext.iterate($O.extStores, function(id, store) {
    store.loadData(extStoreData[ id ]);
  });
}

$O.updateRowHighlight = function() {
  Ext.iterate($O.updateTimers, function(id) {
    $O.updateTimers[ id ]--;
    if (!$O.updateTimers[ id ]) {
      delete $O.updateTimers[ id ];
    }
  });

  if ($O.grid.rendered) {
    $O.grid.getView().refresh();
  }
}

$O.setNumericControl = function() {
  var form = $O.visParamsFW_numeric.form, show
    , type = form.findField('control').getValue()

  show = type != 'vselect';
  form.findField('icons').setVisible(!show).setDisabled(show);

  Ext.each(form.items.items.slice(-12, -7), function(item) {
    item.setVisible(show).setDisabled(!show);
  });

  show = type == 'circularslider';
  Ext.each(form.items.items.slice(-7), function(item) {
    item.setVisible(show).setDisabled(!show);
  });

  show = type != 'slider';
  form.findField('vertical').setVisible(!show).setDisabled(show);
  form.findField('invert').setVisible(!show).setDisabled(show);
};

$O.setRgbControl = function() {
  var form = $O.visParamsFW_rgb.form, i
    , count = form.findField('presets').getValue();

  for (i = 1; i <= 6; i++) {
    form.findField('preset' + i).setVisible(count >= i);
  }
};

$O.init = function() {
  $O.checkUpdate();

  $O.formPanel = Ext.getCmp('objects-form');
  $O.form = $O.formPanel.getForm();
  $O.formData = Ext.getCmp('objects-form-data');
  $O.formAddress = Ext.getCmp('objects-form-address');
  $O.formAddressList = Ext.getCmp('objects-form-address-list');
  $O.formFieldList = Ext.getCmp('objects-form-field-list');

  Ext.getCmp('objects-form-submit').setHandler($O.submitForm);
  Ext.getCmp('objects-form-cancel').setHandler($O.hideForm);
  Ext.getCmp('objects-mass-edit').setHandler($O.massEdit);

  Ext.getCmp('objects-add').setHandler(function() {
    var addr = Scada.findFreeAddress();
    $O.showForm();
    $O.formData.setValue('&mdash;');
    if (addr) {
      addr = Scada.decodeGroupAddress(addr);
      $O.formAddress.setValue(addr);
    }
    $O.formAddress.enable();
  });

  Ext.getCmp('objects-auto-update').on('toggle', function() {
    $O.autoUpdate = !this.pressed;
    this.setIconClass($O.autoUpdate ? 'icon-accept' : 'icon-delete');
    this.setText($tr('objects_auto_update_' + ($O.autoUpdate ? 'on' : 'off')));
  });

  Ext.getCmp('objects-vis-params-numeric-control').on('select', $O.setNumericControl);
  Ext.getCmp('objects-vis-params-rgb-presets').on('select', $O.setRgbControl);

  $O.form.on('beforeaction', function(form, action) {
    var visible = $O.formAddressList.isVisible(), value, res = true;

    if (action.type == 'submit' && form.isValid() && visible) {
      value = $O.formAddressList.getValue();

      if (value) {
        form.findField('address_list').setValue(value);
      }
      else {
        $O.formAddressList.markInvalid();
        res = false;
      }

      value = $O.formFieldList.getValue();

      if (value) {
        form.findField('field_list').setValue(value);
      }
      else {
        $O.formFieldList.markInvalid();
        res = false;
      }
    }

    return res;
  });

  $O.form.on('actioncomplete', function(form, action) {
    var address, value, data, full

    if (action.type == 'load') {
      value = '&mdash;';
      address = '&mdash;';
      data = action.result.data;

      if (data) {
        if (data.datatype) {
          value = Scada.decodeValue(data.datadec, data.datatype.toNumber(), data.units, data.enums, true);
        }

        if (data.address) {
          address = Scada.decodeGroupAddress(data.address);
        }
      }

      $O.formData.setValue(value);
      $O.formAddress.setValue(address);
    }
    else if (action.type == 'submit') {
      full = $O.form.findField('address_list').getValue() != '';
      address = $O.formAddress.getValue();

      $O.hideForm.apply(this);
      $O.objectId = Scada.encodeGroupAddress(address);
      $O.update(full);
    }
  });

  $O.grid.on('cellclick', function(grid, r, c, event) {
    var row = grid.getStore().getAt(r);
    var id = row.get('id').toNumber();
    var field = grid.getColumnModel().getDataIndex(c);

    if (field == 'delete') {
      Ext.Msg.confirm(
        $tr('objects_delete_title'),
        $tr('objects_delete_confirm'),
        $O.onConfirm,
        { id: id }
      );
    }
    else if (field == 'setvalue') {
      $O.showSetValueFormRow(row);
    }
    else if (field == 'visparams') {
      $O.showVisParams(row.data);
    }
    else if (field == 'enums') {
      $O.enumsShow(row.data);
    }
    else if (field == 'script') {
      $S.showEditor(id, true, event);
    }
    else {
      $O.showForm.apply(this, [ id ]);
    }
  });

  $O.store.on('beforeload', function() {
    $O.pagination.refresh.disable();
    $O.pagination.disableAll();
  });

  $O.filters = {
    name: Ext.getCmp('objects-filter-name'),
    dt: Ext.getCmp('objects-filter-dt'),
    tags: Ext.getCmp('objects-filter-tags'),
    tags_mode: Ext.getCmp('objects-filter-tags-mode')
  };

  $O.store.on('load', function(store) {
    var data = store.reader.jsonData.data, sort = $O.gridStore.getSortState();

    $O.rawTags = {
      data: [],
      tags: store.reader.jsonData.tags
    };
    $O.filters.tags.updateTags($O.rawTags);

    $O.gridStore.loadData(data, true);
    $O.gridStore.sort(sort.field, sort.direction);
    $O.gridStore.filterBy($O.filterCb);

    if (!$O.pagination.init) {
      $O.pagination.init = true;
      $O.pagination.moveFirst();
    }

    $O.pagination.refresh.enable();

    if (!$O.extStoreReady || $O.updateExt) {
      $O.onBlur();
      $O.autoUpdate = true;
      $O.extStoreReady = true;
      $O.updateExt = false;
    }
  });

  $O.pagination.refresh.handler = $O.update;
  $O.store.load();
  setInterval($O.updateRowHighlight, 1000);

  $O.massEditInit();

  Scada.setValueInit();
};

Scada.Modules.push($O);
