#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Buildings = {};
var $B = Scada.Buildings;

$B.fields = [
  'id', 'floorid', 'name', 'description', 'building', 'number', 'background',
  'parent', 'offset', 'duplicate', 'delete', 'usermode_param', 'touch_param',
  'type', 'background_add'
];

$B.lpStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings'),
  root: 'data',
  baseParams: { filter: 'lp' },
  fields: $B.fields
});

$B.lwStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings'),
  root: 'data',
  baseParams: { filter: 'lw' },
  fields: $B.fields
});

$B.layoutStore = new Ext.data.JsonStore({
  url: Scada.reqUrl('buildings', 'layouts'),
  root: 'data',
  fields: [ 'id', 'name' ]
});

$B.layoutSelect = $def.select({
  fieldLabel: $tr('buildings_layout'),
  name: 'layout',
  mode: 'local',
  allowBlank: true,
  hiddenName: 'layout',
  store: $B.layoutStore,
  valueField: 'id',
  displayField: 'name'
});

$B.paramSelect = function(name) {
  name = name + '_param';

  return $def.select({
    fieldLabel: $tr('buildings_' + name),
    id: name,
    name: name,
    mode: 'local',
    allowBlank: true,
    hiddenName: name,
    store: [
      ['', $tr('buildings_vis_show') ],
      ['D', $tr('buildings_vis_default') ],
      ['H', $tr('buildings_vis_hide') ]
    ],
    valueField: 'id',
    displayField: 'name'
  });
};

$B.renderName = function(name, cell, record) {
  var data = record.data, level = 0;

  if (data.offset) {
    level = 2;
  }
  else if (data.floorid || data.parent) {
    level = 1;
  }

  cell.css += data.floorid ? 'scada-cell-plan ' : 'scada-cell-folder ';
  cell.css += 'scada-cell-icon scada-level-' + level;

  return name;
};

$B.renderVisibility = function(name, cell, record) {
  var data = record.data, uParam = data.usermode_param, tParam = data.touch_param, result = '';

  if (data.floorid && data.building) {
    if (uParam == 'D') {
      result = '<b>' + $tr('buildings_usermode') + '</b>';
    }
    else if (uParam == '') {
      result = $tr('buildings_usermode');
    }

    if (tParam != 'H') {
      if (result) {
        result += ', ';
      }

      if (tParam == 'D') {
        result += '<b>' + $tr('buildings_touch') + '</b>';
      }
      else {
        result += $tr('buildings_touch');
      }
    }
  }

  return result;
};

$B.viewConfig = {
  forceFit: true,
  getRowClass: function(record) {
    var data = record.data;
    return (data.floorid || data.parent) ? 'objects-row-normal' : 'objects-row-hl';
  }
};

$B.lpGrid = new Ext.grid.GridPanel($def.grid({
  store: $B.lpStore,
  border: false,
  loadMask: true,
  title: $tr('buildings_levels_plans'),
  colModel: new Ext.grid.ColumnModel({
    defaults: {
      sortable: false,
      menuDisabled: true
    },
    columns: [
      { header: $tr('buildings_name'), width: 0.20, dataIndex: 'name', renderer: $B.renderName },
      { header: $tr('buildings_visibility'), width: 0.15, dataIndex: 'usermode_param', renderer: $B.renderVisibility },
      { header: $tr('buildings_description'), width: 0.30, dataIndex: 'description' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'scada-cell-duplicate', dataIndex: 'duplicate' ,  hidden:true},
      { header: $tr('buildings_move_up'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-up', dataIndex: 'up' },
      { header: $tr('buildings_move_down'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-down', dataIndex: 'down' },
      { header: $tr('buildings_add_import'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-add', trueCls: '', dataIndex: 'floorid', hidden:true },
      { header: $tr('buildings_export'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-export', trueCls: 'scada-cell-export', action: 'export', dataIndex: 'floorid' ,hidden:true},
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  viewConfig: $B.viewConfig,
  bbar: [
//    { text: $tr('buildings_add'), iconCls: 'icon-add', handler: function() { $B.showBuildingForm(); } },
    '-',
//    { text: $tr('buildings_import'), iconCls: 'icon-import', handler: function() { $B.showImportFW(0); } }
  ]
}));

$B.lwGrid = new Ext.grid.GridPanel($def.grid({
  store: $B.lwStore,
  border: false,
  loadMask: true,
  title: $tr('buildings_layouts_widgets'),
  colModel: new Ext.grid.ColumnModel({
    defaults: {
      sortable: false,
      menuDisabled: true
    },
    columns: [
      { header: $tr('buildings_name'), width: 0.20, dataIndex: 'name', renderer: $B.renderName },
      { header: $tr('buildings_description'), width: 0.55, dataIndex: 'description' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'scada-cell-duplicate', dataIndex: 'duplicate' },
      { header: $tr('buildings_add_import'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-add', trueCls: '', dataIndex: 'floorid' },
      { header: $tr('buildings_export'), width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'scada-cell-export', action: 'export', dataIndex: 'floorid' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: '', trueCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  viewConfig: $B.viewConfig,
  bbar: [
    { text: $tr('buildings_add_layout'), iconCls: 'icon-add', handler: function() { $B.showFloorForm({ bid: 'layout', bname: $tr('buildings_layout') }); } },
    '-',
    { text: $tr('buildings_add_widget'), iconCls: 'icon-add', handler: function() { $B.showFloorForm({ bid: 'widget', bname: $tr('buildings_widget') }); } }
  ]
}));

$B.bFormWindow = new Ext.Window(
  $def.window({
    title: $tr('buildings_bform_title'), width: 490, height: 240,
    items: $def.form({
      id: 'buildings-bform',
      url: Scada.reqUrl('buildings', 'building-save'),
      labelWidth: 180,
      buttons: $def.formButtons('buildings-bform'),
      items: [
        { fieldLabel: $tr('buildings_pname'), name: 'pname', xtype: 'displayfield' },
        { fieldLabel: $tr('buildings_bname'), name: 'name', allowBlank: false },
        { fieldLabel: $tr('vis_pincode'), name: 'pincode', xtype: 'textfield', width: 80, minLength: 3, maxLength: 8, allowBlank: true, maskRe: /[0-9]/ },
        { fieldLabel: $tr('buildings_bdescription'), name: 'description', xtype: 'textarea', height: 90 },
        { name: 'parent', xtype: 'hidden' },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$B.sizePresets = [
  { value: [ '1024', '748' ], text: 'iPad landscape, fullscreen (XGA) 1024 x 748' },
  { value: [ '1024', '672' ], text: 'iPad landscape, browser (XGA) 1024 x 672' },
  { value: [ '768', '1004' ], text: 'iPad portrait, fullscreen (XGA) 768 x 1004' },
  { value: [ '768', '928' ], text: 'iPad portrait, browser (XGA) 768 x 928' },
  { value: [ '1024', '600' ], text: 'Tablet landscape (WSVGA) 1024 x 600' },
  { value: [ '600', '1024' ], text: 'Tablet portrait (WSVGA) 600 x 1024' },
  { value: [ '1280', '800' ], text: 'Laptop / Tablet landscape (WXGA) 1280 x 800' },
  { value: [ '800', '1280' ], text: 'Laptop / Tablet portrait (WXGA) 800 x 1280' },
  { value: [ '1360', '768' ], text: 'Laptop / Tablet landscape (HD) 1360 x 768' },
  { value: [ '768', '1360' ], text: 'Laptop / Tablet portrait (HD) 768 x 1360' },
  { value: [ '1920', '1080' ], text: 'Big screen (Full HD) 1920 x 1080' }
];

$B.fFormWindow = new Ext.Window(
  $def.window({
    width: 490, height: 430,
    items: $def.form({
      id: 'buildings-fform',
      url: Scada.reqUrl('buildings', 'floor-save'),
      labelWidth: 180,
      buttons: $def.formButtons('buildings-fform'),
      items: [
        { fieldLabel: $tr('buildings_parent'), name: 'bname', xtype: 'displayfield' },
        { fieldLabel: $tr('buildings_name'), name: 'name', allowBlank: false },
        {
          xtype: 'compositefield',
          fieldLabel: $tr('vis_plan_size'),
          msgTarget: 'title',
          items: [
            { name: 'width', xtype: 'spinnerfield', minValue: 16, maxValue: 5120, width: 55, value: 1024, allowDecimals: false },
            { name: 'height', xtype: 'spinnerfield', minValue: 16, maxValue: 5120, width: 55, value: 768, allowDecimals: false },
            { xtype: 'button', iconCls: 'icon-list-view', menu: {
                id: 'buildings-plan-menu',
                items: $B.sizePresets
              }
            }
          ]
        },
        {
          id: 'buildings-widget-loc',
          xtype: 'compositefield',
          fieldLabel: $tr('vis_widget_loc'),
          msgTarget: 'title',
          items: [
            { name: 'locx', xtype: 'spinnerfield', minValue: 0, maxValue: 5120, width: 55, value: '', allowDecimals: false, allowBlank: true },
            { name: 'locy', xtype: 'spinnerfield', minValue: 0, maxValue: 5120, width: 55, value: '', allowDecimals: false, allowBlank: true }
          ]
        },
        $B.layoutSelect,
        $B.paramSelect('usermode'),
        $B.paramSelect('touch'),
        { fieldLabel: $tr('vis_pincode'), name: 'pincode', xtype: 'textfield', width: 80, minLength: 3, maxLength: 8, allowBlank: true, maskRe: /[0-9]/ },
        { fieldLabel: $tr('buildings_background'), id: 'buildings-bg-select', xtype: 'imgtrigger', name: 'background', width: 250 },
        { fieldLabel: $tr('buildings_background_add'), id: 'buildings-bg-select-add', xtype: 'imgtrigger', name: 'background_add', width: 250 },
        { fieldLabel: $tr('buildings_bgcolor'), xtype: 'colorfield', name: 'bgcolor', width: 100 },
        { fieldLabel: $tr('buildings_touch_bgcolor'), xtype: 'colorfield', name: 'touch_bgcolor', width: 100 },
        { fieldLabel: $tr('buildings_bgrepeat'), name: 'bgrepeat', xtype: 'checkbox' },
        { fieldLabel: $tr('buildings_bgfixed'), name: 'bgfixed', xtype: 'checkbox' },
        { fieldLabel: $tr('buildings_fadminonly'), name: 'adminonly', xtype: 'checkbox' },
        { name: 'bid', xtype: 'hidden' },
        { name: 'id', xtype: 'hidden' }
      ]
    })
  })
);

$B.MainPanel = new Ext.TabPanel({
  border: false,
  region: 'center',
  margins: '5 0 0 0',
  items: [ $B.lpGrid, $B.lwGrid ]
});

$B.container = new Ext.Container({
  id: 'Buildings',
  title: $tr('buildings_title'),
  layout: 'border',
  items: $B.MainPanel
});

$B.update = function() {
  $B.lpStore.reload();
  $B.lwStore.reload();
  $B.layoutStore.reload();
};

$B.importError = function(form, resp) {
  var err = (resp.result ? resp.result.error : '') || 'general';
  Ext.Msg.alert($tr('error'), $tr('buildings_import_error_' + err));
};

$B.importFW = new ExtFormWindow({
  id: 'buildings-import',
  window: { title: $tr('buildings_import'), width: 460, height: 145 },
  form: {
    url: Scada.reqUrl('buildings', 'import'),
    fileUpload: true,
    items: [
      { fieldLabel: $tr('buildings_import_mode'), name: 'mode', xtype: 'radiogroup', columns: 2,
        items: [
          { xtype: 'radio', boxLabel: $tr('buildings_import_clear'), checked: true, name: 'mode', inputValue: 'clear' },
          { xtype: 'radio', boxLabel: $tr('buildings_import_keep'), name: 'mode', inputValue: 'keep' }
        ]
      },
      $def.fileUpload({ name: 'file', fieldLabel: $tr('buildings_file') }),
      { name: 'id', xtype: 'hidden' },
      { name: 'ptype', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $B.update, onFormSubmitFailed: $B.importError }
});

$B.showImportFW = function(bid) {
  var id = bid.toNumber();

  $B.importFW.show();
  $B.importFW.form.findField('id').setValue(id || 0);
  $B.importFW.form.findField('ptype').setValue(id === null ? bid : '');
};

$B.showBuildingForm = function(id, parent) {
  var p = !!parent;

  $B.bForm.reset();
  $B.bFormWindow.show().setHeight(p ? 265 : 240).center();

  $B.bForm.findField('pname').setVisible(p);

  if (id) {
    $B.bForm.load({
      url: Scada.reqUrl('buildings', 'building-form'),
      params: Scada.encodeData({ id: id }),
      waitMsg: $tr('loading')
    });
  }
  else if (parent) {
    $B.bForm.findField('parent').setValue(parent.bid);
    $B.bForm.findField('pname').setValue(parent.bname);
  }
};

$B.hideBuildingForm = function() {
  $B.bFormWindow.hide();
};

$B.submitBuildingForm = function() {
  $B.bForm.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$B.showFloorForm = function(data) {
  var layout = $B.fForm.findField('layout'),
    uParam = $B.fForm.findField('usermode_param'),
    tParam = $B.fForm.findField('touch_param'),
    type, bid = data.bid.toNumber();

  if (data.type) {
    type = data.type;
  }
  else {
    type = (data.bid == 'layout') ? 'layout' : (bid ? 'plan' : 'widget');
  }

  $B.fFormWindow.setTitle($tr('buildings_' + type));

  $B.fForm.reset();
  $B.fFormWindow.show().setHeight(type == 'plan' ? 455 : 325).center();

  layout.setVisible(bid);
  uParam.setVisible(bid);
  tParam.setVisible(bid);
  $B.fForm.findField('pincode').setVisible(bid);
  $B.fForm.findField('background_add').setVisible(type != 'widget');
  Ext.getCmp('buildings-widget-loc').setVisible(type == 'widget');

  if (bid) {
    layout.setValue(0);
    uParam.setValue('');
    tParam.setValue('');
  }

  if (data.id) {
    $B.fForm.load({
      url: Scada.reqUrl('buildings', 'floor-form'),
      params: Scada.encodeData({ id: data.id }),
      waitMsg: $tr('loading')
    });
  }
  else {
    $B.fForm.setValues(data);
  }

  $B.fForm.findField('adminonly').setVisible(bid);
};

$B.hideFloorForm = function() {
  $B.fFormWindow.hide();
};

$B.submitFloorForm = function() {
  $B.fForm.submit({ mode: 'json', waitMsg: $tr('loading') });
};

$B.onDeleteConfirm = function(confirmResult) {
  if (confirmResult == 'yes') {
    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'delete'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData(this)
    });
  }
};

$B.lvl2Window = new Ext.Window($def.window({
  title: $tr('buildings_lvl2_title'),
  border: false,
  layout: 'vbox',
  layoutConfig: {
    pack: 'center',
    padding: 5
  },
  width: 222,
  height: 140,
  items: [
    {
      xtype: 'button',
      text: $tr('buildings_add_lvl2'),
      margins: '0 0 10 0',
      width: 200,
      handler: function() {
        $B.showBuildingForm(0, $B.lvl2Window.params);
        $B.lvl2Window.hide();
      }
    },
    {
      xtype: 'button',
      text: $tr('buildings_add_lvl3'),
      margins: '0 0 10 0',
      width: 200,
      handler: function() {
        $B.showFloorForm($B.lvl2Window.params);
        $B.lvl2Window.hide();
      }
    },
    {
      xtype: 'button',
      text: $tr('buildings_import'),
      width: 200,
      handler: function() {
        $B.showImportFW($B.lvl2Window.params.bid);
        $B.lvl2Window.hide();
      }
    }
  ]
}));

$B.addImportWindow = new Ext.Window($def.window({
  title: $tr('buildings_lvl2_title'),
  border: false,
  layout: 'vbox',
  layoutConfig: {
    pack: 'center',
    padding: 5
  },
  width: 222,
  height: 100,
  items: [
    {
      xtype: 'button',
      text: $tr('add'),
      margins: '0 0 10 0',
      width: 200,
      handler: function() {
        $B.showFloorForm($B.addImportWindow.params);
        $B.addImportWindow.hide();
      }
    },
    {
      xtype: 'button',
      text: $tr('buildings_import'),
      width: 200,
      handler: function() {
        $B.showImportFW($B.addImportWindow.params.bid);
        $B.addImportWindow.hide();
      }
    }
  ]
}));

$B.gridCellClick = function(grid, r, c) {
  var row, field, mode, id, editable, action;

  row = grid.getStore().getAt(r);
  field = grid.getColumnModel().getDataIndex(c);

  mode = row.get('floorid') ? 'floor' : 'building';
  editable = row.get('delete');
  id = row.get(mode == 'building' ? 'id' : 'floorid');

  if (field == 'floorid' || !editable) {
    action = grid.getColumnModel().getColumnAt(c).action;

    if (action == 'export') {
      mode = mode == 'building' ? '&mode=level' : '';
      window.open('/cgi-bin/scada/general/visstruct-export.cgi?id=' + id + mode);
    }
    else if (mode == 'building') {
      var parent = row.get('parent'), params = { bid: id, bname: row.get('name') };

      if (parent || !editable) {
        $B.addImportWindow.params = params;
        $B.addImportWindow.show();
      }
      else {
        $B.lvl2Window.params = params;
        $B.lvl2Window.show();
      }
    }
  }
  else if (field == 'duplicate') {
    grid.loadMask.show();

    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'duplicate'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData({ id: id, mode: mode })
    });
  }
  else if (field == 'delete') {
    Ext.Msg.confirm(
      $tr('buildings_delete_title_' + mode),
      $tr('buildings_delete_confirm_' + mode),
      $B.onDeleteConfirm,
      { id: id, mode: mode }
    );
  }
  else if (field == 'up' || field == 'down') {
    Ext.Ajax.request({
      url: Scada.reqUrl('buildings', 'updown'),
      success: $B.update,
      failure: $B.update,
      params: Scada.encodeData({ id: id, up: field == 'up', mode: mode })
    });
  }
  else if (mode == 'building') {
    $B.showBuildingForm(id);
  }
  else {
    $B.showFloorForm({ id: id, bid: row.get('id'), type: row.get('type') });
  }
};

$B.init = function() {
  $B.lpGrid.on('cellclick', $B.gridCellClick);
  $B.lwGrid.on('cellclick', $B.gridCellClick);

  $B.bForm = Ext.getCmp('buildings-bform').getForm();
  $B.bForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $B.hideBuildingForm();
      $B.update();
    }
  });

  Ext.getCmp('buildings-plan-menu').on('itemclick', function(item) {
    var size = item.value;

    $B.fForm.findField('width').setValue(size[ 0 ]);
    $B.fForm.findField('height').setValue(size[ 1 ]);
  });

  Ext.getCmp('buildings-bform-submit').setHandler($B.submitBuildingForm);
  Ext.getCmp('buildings-bform-cancel').setHandler($B.hideBuildingForm);

  $B.fForm = Ext.getCmp('buildings-fform').getForm();

  $B.fForm.on('actioncomplete', function(form, action) {
    if (action.type == 'submit') {
      $B.hideFloorForm();
      $B.update();
    }
  });

  Scada.initImgSelect($B);
  Scada.bindImgSelect($B, 'buildings-bg-select');
  Scada.bindImgSelect($B, 'buildings-bg-select-add');

  Ext.getCmp('buildings-fform-submit').setHandler($B.submitFloorForm);
  Ext.getCmp('buildings-fform-cancel').setHandler($B.hideFloorForm);

  $B.MainPanel.setActiveTab(0);
};

$B.onFocus = $B.update;
Scada.Modules.push($B);
