#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Scripting = {};

var $S = Scada.Scripting;
$S.pList = [ 'event', 'resident', 'scheduled', 'user' ];
$S.topBar = [];

$S.helpersRestore = function() {
  Ext.Msg.confirm(
    $tr('scripting_tools_helpers_restore'),
    $tr('scripting_tools_helpers_restore_confirm'),
    function(res) {
      if (res == 'yes') {
        Ext.Ajax.request({
          url: Scada.genUrl('scripting-helpers-restore')
        });
      }
    }
  );
};

$S.backup = function() {
  Ext.Msg.confirm(
    $tr('scripting_tools_backup'),
    $tr('scripting_tools_backup_confirm'),
    function(res) {
      window.open(Scada.genUrl('scripting-backup') + '?full=' + res);
    }
  );
};

$S.showRestoreWindow = function(btn) {
  $S.restoreWindow.show().setHeight(btn.full ? 170 : 120).center();
  $S.restoreForm.findField('mode').setValue(btn.full ? 'overwrite' : 'append').setVisible(btn.full);
};

$S.update = function(res) {
  var resp = Ext.decode(res.responseText), data = {};

  Ext.each($S.pList, function(type) {
    data[ type ] = { data: [] };
  });

  Ext.each(resp.data, function(row) {
    if (row.type && data[ row.type ]) {
      data[ row.type ].data.push(row);
    }
  });


  Ext.each($S.pList, function(type) {
    var name = type[ 0 ];
    $S[ name + 'Store' ].loadData(data[ type ]);
  });

  $S.loadCategories();
};

$S.reload = function() {
  Ext.Ajax.request({
    url: Scada.reqUrl('scripting'),
    success: $S.update
  });
};

Ext.each($S.pList, function(item) {
  var name = item[ 0 ], sorter = (item == 'event' ? 'params' : 'name'), items;

  $S[ name + 'Store' ] = new Ext.data.JsonStore({
    baseParams: Scada.encodeData({ type: item }),
    root: 'data',
    data: { data:[] },
    sortInfo: { field: sorter, direction: 'ASC' },
    fields: [ 'id', 'name', 'type', 'description', 'params', 'subparams', 'active', 'category', 'source' ]
  });

  $S.topBar.push({
    id: 'scripting-btn-' + item,
    text: $tr('scripting_' + item),
    iconCls: 'scada-scripting-btn-' + item,
    pressed: (item == 'event'),
    xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
  });
});

$S.topBar.push({
  text: $tr('scripting_tools_userlib'),
  iconCls: 'scada-scripting-btn-userlib',
  handler: function() {
    $S.showEditor('userlib');
  },
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$S.topBar.push({
  text: $tr('scripting_tools_initscript'),
  iconCls: 'scada-scripting-btn-initscript',
  handler: function() {
    $S.showEditor('initscript');
  },
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$S.topBar.push({
  text: $tr('scripting_tools_blockly'),
  iconCls: 'scada-scripting-btn-blockly',
  handler: function() {
    $S.showEditor('blockly');
  },
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
});

$S.topBar.push({
  id: 'scripting-btn-tools',
  text: $tr('scripting_tools'),
  iconCls: 'scada-scripting-btn-tools',
  xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120,
  menu: {
    items: [
      { text: $tr('scripting_tools_helpers_export'), iconCls: 'icon-export', handler: function() { window.open('/scada/resources/helpers.js'); } },
      { text: $tr('scripting_tools_helpers_import'), iconCls: 'icon-import', handler: function() { $S.helpersImportWindow.show(); } },
      { text: $tr('scripting_tools_helpers_restore'), iconCls: 'icon-restore', handler: $S.helpersRestore },
      { text: $tr('scripting_tools_backup'), iconCls: 'icon-backup', handler: $S.backup },
      { text: $tr('scripting_tools_restore'), iconCls: 'icon-restore', handler: $S.showRestoreWindow, full: true },
      { text: $tr('scripting_tools_print'), iconCls: 'icon-print', handler: function() { window.open(Scada.genUrl('scripting-print')); } },
      { text: $tr('logs_window_show'), iconCls: 'icon-list-view', handler: function() { $L.logWindow.show(); } }
    ]
  }
});

$S.categoryStore = new Ext.data.ArrayStore({
  fields: ['value', 'display'],
  idIndex: 0,
  data: [
    [ '', $tr('scripting_filter_all') ]
  ]
});

$S.topBar.push({
  cls: 'column-form',
  xtype: 'panel',
  layout: 'form',
  labelAlign: 'top',
  border: false,
  items: $def.select({
    id: 'scripting-category-filter',
    fieldLabel: $tr('scripting_category_filter'),
    name: 'category',
    mode: 'local',
    valueField: 'value',
    displayField: 'display',
    value: '',
    store: $S.categoryStore
  })
});

$S.rGrid = new Ext.grid.GridPanel($def.grid({
  id: 'scripting-grid-resident',
  store: $S.rStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('scripting_name'), width: 0.25, dataIndex: 'name' },
      { header: $tr('scripting_sleep'), width: 0.15, dataIndex: 'params' },
      { header: $tr('scripting_description'), width: 0.25, dataIndex: 'description' },
      { header: $tr('scripting_category'), width: 0.15, dataIndex: 'category' },
      { header: $tr('scripting_editor'), width: 0.05, xtype: 'boolclscolumn', falseCls: Scada.editorCls, dataIndex: 'editor' },
      { header: $tr('scripting_active'), width: 0.05, xtype: 'boolclscolumn', dataIndex: 'active' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-duplicate', dataIndex: 'duplicate' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: [
    { text: $tr('scripting_btn_add'), iconCls: 'icon-add', id: 'scripting-resident-add' }
  ]
}));

$S.eGrid = new Ext.grid.GridPanel($def.grid({
  id: 'scripting-grid-event',
  store: $S.eStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('scripting_name'), width: 0.25, dataIndex: 'name' },
      { header: $tr('scripting_address'), width: 0.15, dataIndex: 'params' },
      { header: $tr('scripting_description'), width: 0.25, dataIndex: 'description' },
      { header: $tr('scripting_category'), width: 0.15, dataIndex: 'category' },
      { header: $tr('scripting_editor'), width: 0.05, xtype: 'boolclscolumn', falseCls: Scada.editorCls, dataIndex: 'editor' },
      { header: $tr('scripting_active'), width: 0.05, xtype: 'boolclscolumn', dataIndex: 'active' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-duplicate', dataIndex: 'duplicate' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: [
    { text: $tr('scripting_btn_add'), iconCls: 'icon-add', id: 'scripting-event-add' }
  ]
}));

$S.sGrid = new Ext.grid.GridPanel($def.grid({
  id: 'scripting-grid-scheduled',
  store: $S.sStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('scripting_name'), width: 0.25, dataIndex: 'name' },
      { header: $tr('scripting_datetime'), width: 0.15, dataIndex: 'params' },
      { header: $tr('scripting_description'), width: 0.25, dataIndex: 'description' },
      { header: $tr('scripting_category'), width: 0.15, dataIndex: 'category' },
      { header: $tr('scripting_editor'), width: 0.05, xtype: 'boolclscolumn', falseCls: Scada.editorCls, dataIndex: 'editor' },
      { header: $tr('scripting_active'), width: 0.05, xtype: 'boolclscolumn', dataIndex: 'active' },
      { header: $tr('scripting_duplicate'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-duplicate', dataIndex: 'duplicate' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: [
    { text: $tr('scripting_btn_add'), iconCls: 'icon-add', id: 'scripting-scheduled-add' }
  ]
}));

$S.uGrid = new Ext.grid.GridPanel($def.grid({
  id: 'scripting-grid-user',
  store: $S.uStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('scripting_name'), width: 0.85, dataIndex: 'name' },
      { header: $tr('scripting_editor'), width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-editor', dataIndex: 'editor' },
      { header: $tr('scripting_source'), width: 0.05, xtype: 'boolclscolumn', dataIndex: 'source' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: [
    { text: $tr('scripting_btn_library_add'), iconCls: 'icon-add', id: 'scripting-user-add' }, '-',
    { text: $tr('scripting_tools_library_export'), iconCls: 'icon-export', handler: function() { window.open(Scada.genUrl('scripting-backup') + '?user=1'); } }, '-',
    { text: $tr('scripting_tools_library_restore'), iconCls: 'icon-restore', handler: $S.showRestoreWindow, full: false }
  ]
}));

$S.eFormWindow = new Ext.Window($def.window({
  title: $tr('scripting_event_form'),
  width: 490,
  height: 320,
  items: $def.form({
    id: 'scripting-event-form',
    url: Scada.reqUrl('scripting', 'save'),
    labelWidth: 180,
    items: [
      { fieldLabel: $tr('scripting_name'), name: 'name', allowBlank: false },
      { fieldLabel: $tr('scripting_address'), name: 'params', allowBlank: false, xtype: 'combo', store: [], mode: 'local', triggerAction: 'all' },
      { fieldLabel: $tr('scripting_active'), name: 'active', xtype: 'checkbox' },
      { fieldLabel: $tr('scripting_attachread'), name: 'subparams', xtype: 'checkbox', value: 1 },
      { fieldLabel: $tr('scripting_category'), name: 'category', xtype: 'combo', store: [], mode: 'local', triggerAction: 'all' },
      { fieldLabel: $tr('scripting_description'), name: 'description', xtype: 'textarea', height: 90 },
      { name: 'id', xtype: 'hidden' },
      { name: 'type', xtype: 'hidden', id: 'scripting-event-form-type' }
    ],
    buttons: $def.formButtons('scripting-event-form')
  })
}));

$S.rFormWindow = new Ext.Window($def.window({
  title: $tr('scripting_resident_form'),
  width: 490,
  height: 300,
  items: $def.form({
    id: 'scripting-resident-form',
    url: Scada.reqUrl('scripting', 'save'),
    labelWidth: 180,
    items: [
      { fieldLabel: $tr('scripting_name'), name: 'name', allowBlank: false },
      { fieldLabel: $tr('scripting_sleep'), name: 'params', xtype: 'spinnerfield', allowBlank: false, minValue: 0, maxValue: 60, allowDecimals: false },
      { fieldLabel: $tr('scripting_active'), name: 'active', xtype: 'checkbox' },
      { fieldLabel: $tr('scripting_category'), name: 'category', xtype: 'combo', store: [], mode: 'local', triggerAction: 'all' },
      { fieldLabel: $tr('scripting_description'), name: 'description', xtype: 'textarea', height: 90 },
      { name: 'id', xtype: 'hidden' },
      { name: 'type', xtype: 'hidden', id: 'scripting-resident-form-type' }
    ],
    buttons: $def.formButtons('scripting-resident-form')
  })
}));

$S.sFormWindow = new Ext.Window($def.window({
  title: $tr('scripting_scheduled_form'),
  width: 490,
  height: 400,
  items: $def.form({
    id: 'scripting-scheduled-form',
    url: Scada.reqUrl('scripting', 'save'),
    labelWidth: 180,
    items: [
      { fieldLabel: $tr('scripting_name'), name: 'name', allowBlank: false },
      { name: 'params', id: 'scripting-scheduled-form-params', xtype: 'hidden' },
      { fieldLabel: $tr('scripting_dt_minute'), id: 'scripting-scheduled-form-minute', allowBlank: false, vtype: 'cronMinute' },
      { fieldLabel: $tr('scripting_dt_hour'), id: 'scripting-scheduled-form-hour', allowBlank: false, vtype: 'cronHour' },
      { fieldLabel: $tr('scripting_dt_monthday'), id: 'scripting-scheduled-form-monthday', allowBlank: false, vtype: 'cronDay' },
      { fieldLabel: $tr('scripting_dt_month'), name: 'dt_month', xtype: 'menutriggerfield', allowBlank: false, editable: false,
        id: 'scripting-scheduled-form-month', allText: $tr('scripting_dt_month_all'), width: 250,
        menu: {
          id: 'scripting-scheduled-form-month-menu',
          width: 250,
          items: [
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_jan'), value: '1', fieldText: $tr('scripting_dt_month_1') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_feb'), value: '2', fieldText: $tr('scripting_dt_month_2') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_mar'), value: '3', fieldText: $tr('scripting_dt_month_3') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_apr'), value: '4', fieldText: $tr('scripting_dt_month_4') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_may'), value: '5', fieldText: $tr('scripting_dt_month_5') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_jun'), value: '6', fieldText: $tr('scripting_dt_month_6') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_jul'), value: '7', fieldText: $tr('scripting_dt_month_7') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_aug'), value: '8', fieldText: $tr('scripting_dt_month_8') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_sep'), value: '9', fieldText: $tr('scripting_dt_month_9') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_oct'), value: '10', fieldText: $tr('scripting_dt_month_10') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_nov'), value: '11', fieldText: $tr('scripting_dt_month_11') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_month_dec'), value: '12', fieldText: $tr('scripting_dt_month_12') }
          ]
        }
      },
      { fieldLabel: $tr('scripting_dt_weekday'), name: 'dt_weekday', xtype: 'menutriggerfield', allowBlank: false, editable: false,
        id: 'scripting-scheduled-form-weekday', allText: $tr('scripting_dt_weekday_all'), width: 250,
        menu: {
          id: 'scripting-scheduled-form-weekday-menu',
          width: 250,
          items: [
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_mon'), value: '1', fieldText: $tr('scripting_dt_weekday_1') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_tue'), value: '2', fieldText: $tr('scripting_dt_weekday_2') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_wed'), value: '3', fieldText: $tr('scripting_dt_weekday_3') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_thu'), value: '4', fieldText: $tr('scripting_dt_weekday_4') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_fri'), value: '5', fieldText: $tr('scripting_dt_weekday_5') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_sat'), value: '6', fieldText: $tr('scripting_dt_weekday_6') },
            { xtype: 'menucheckitem', hideOnClick: false, text: $tr('scripting_dt_weekday_sun'), value: '0', fieldText: $tr('scripting_dt_weekday_0') }
          ]
        }
      },
      { fieldLabel: $tr('scripting_active'), name: 'active', xtype: 'checkbox' },
      { fieldLabel: $tr('scripting_category'), name: 'category', xtype: 'combo', store: [], mode: 'local', triggerAction: 'all' },
      { fieldLabel: $tr('scripting_description'), name: 'description', xtype: 'textarea', height: 90 },
      { name: 'id', xtype: 'hidden' },
      { name: 'type', xtype: 'hidden', id: 'scripting-scheduled-form-type' }
    ],
    buttons: $def.formButtons('scripting-scheduled-form')
  })
}));

$S.uFormWindow = new Ext.Window($def.window({
  title: $tr('scripting_user_form'),
  width: 490,
  height: 270,
  items: $def.form({
    id: 'scripting-user-form',
    url: Scada.reqUrl('scripting', 'save'),
    labelWidth: 180,
    items: [
      { fieldLabel: $tr('scripting_name'), name: 'name', allowBlank: false, regex: /^[-\w]{1,30}$/, invalidText: $tr('objects_error_name') },
      { fieldLabel: $tr('scripting_source'), name: 'source', xtype: 'checkbox' },
      { fieldLabel: $tr('scripting_load'), name: 'load', xtype: 'checkbox', boxLabel: $tr('scripting_load_hint'), },
      { fieldLabel: $tr('scripting_description'), name: 'description', xtype: 'textarea', height: 90 },
      { name: 'id', xtype: 'hidden' },
      { name: 'type', xtype: 'hidden', id: 'scripting-user-form-type' }
    ],
    buttons: $def.formButtons('scripting-user-form')
  })
}));

$S.restoreWindow = new Ext.Window($def.window({
  title: $tr('scripting_restore_form'),
  width: 460,
  height: 170,
  items: $def.form({
    id: 'scripting-restore-form',
    url: Scada.genUrl('scripting-restore'),
    fileUpload: true,
    labelWidth: 100,
    defaults: { width: 300 },
    items: [
      { fieldLabel: $tr('scripting_restore_mode'), name: 'mode', xtype: 'radiogroup', columns: 1,
        items: [
          { xtype: 'radio', boxLabel: $tr('scripting_restore_overwrite'), checked: true, name: 'mode', inputValue: 'overwrite' },
          { xtype: 'radio', boxLabel: $tr('scripting_restore_append'), name: 'mode', inputValue: 'append' }
        ]
      },
      $def.fileUpload({ id: 'scripting-restore-form-file', fieldLabel: $tr('scripting_restore_file') })
    ],
    buttons: $def.formButtons('scripting-restore-form')
  })
}));

$S.mainPanel = new Ext.Panel({
  id: 'scripting-main-container',
  layout: 'card',
  margins: '10 10 10 10',
  border: false,
  activeItem: 0,
  flex: 1,
  items: [ $S.eGrid, $S.rGrid, $S.sGrid, $S.uGrid ]
});

$S.container = new Ext.Container({
  id: 'Scripting',
  title: $tr('scripting_title'),
  layout: 'vbox',
  layoutConfig: { align : 'stretch' },
  items: [
    {
      region: 'center',
      layout: 'column',
      cls: 'x-column-menu',
      border: false,
      height: 56,
      margins: '10 10 0 10',
      items: $S.topBar
    },
    $S.mainPanel
  ]
});

$S.onConfirm = function(confirmResult) {
  var grid;

  if (confirmResult == 'yes') {
    grid = $S[ this.name + 'Grid' ];

    if (grid.rendered) {
      grid.loadMask.show();
    }

    Ext.Ajax.request({
      url: Scada.reqUrl('scripting', this.mode),
      success: $S.reload,
      failure: $S.reload,
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$S.showPanel = function() {
  var mode = this;

  Ext.each($S.pList, function(type) {
    Ext.getCmp('scripting-btn-' + type).toggle(mode == type);
  });

  $S.mainPanel.layout.setActiveItem('scripting-grid-' + mode);
};

$S.showForm = function(itemId) {
  var form = $S[ this.name + 'Form' ];
  var formWindow = $S[ this.name + 'FormWindow' ];

  form.reset();
  formWindow.show().center();

  if (typeof itemId != 'number' && typeof itemId != 'string') {
    itemId = 0;
  }

  if (itemId || this.type == 'event') {
    form.load({
      url: Scada.reqUrl('scripting', 'form'),
      params: Scada.encodeData({ id: itemId }),
      waitMsg: $tr('loading')
    });
  }

  if (!itemId && this.type == 'user') {
    form.findField('load').setValue(1);
    form.findField('source').setValue(1);
  }
};

$S.hideForm = function() {
  $S[ this + 'FormWindow' ].hide();
};

$S.submitForm = function() {
  var form = $S[ this.name + 'Form' ];
  Ext.get('scripting-' + this.type + '-form-type').dom.value = this.type;

  form.submit({
    mode: 'json',
    exclude: 'script',
    waitMsg: $tr('loading')
  });
};

$S.showEditor = function(id, object, event) {
  var param = object ? 'object' : 'id', win, mode = 'editor', el, x, w, k;

  if (event && Scada.blockly && id.toNumber()) {
    el = Ext.fly(event.getTarget());
    x = el.getX();
    w = el.getWidth();

    k = (event.getXY()[0] - x) / w;
    if (k > 0.5) {
      mode = 'blockly';
    }
  }

  win = window.open('/cgi-bin/scada/' + mode + '.cgi?' + param + '=' + id);
  win.onload = function() {
    if (!win.parentWin) {
      win.parentWin = window;
    }

    if (object) {
      $S.reload();
    }
  };
};

$S.helpersImportWindow = new Ext.Window($def.window({
  title: $tr('scripting_tools_helpers_import'),
  width: 460,
  height: 130,

  items: $def.form({
    id: 'scripting-helpers-import-form',
    url: Scada.genUrl('scripting-helpers-import'),
    fileUpload: true,
    labelWidth: 100,
    defaults: { width: 300 },
    items: [
      $def.fileUpload({ id: 'helpers-import-form-file', fieldLabel: $tr('scripting_tools_helpers_file') })
    ],
    buttons: $def.formButtons('scripting-helpers-import-form')
  })
}));

$S.loadCategories = function() {
  var categories = [], store = [];

  Ext.each([ 'e', 'r', 's' ], function(type) {
    var name = type[ 0 ], items = $S[ name + 'Store' ].getRange();

    Ext.each(items, function(item) {
      var category = (item.data.category || '').trim();

      if (category && categories.indexOf(category) == -1) {
        categories.push(category);
      }
    });
  });

  categories.sort();
  store.push([ '', $tr('scripting_filter_all') ]);

  Ext.each(categories, function(category) {
    store.push([ category, category ]);
  });

  $S.categoryStore.loadData(store);
};

$S.init = function() {
  Ext.each($S.pList, function(item) {
    var name = item[ 0 ];
    var bindParams = { name: name, type: item };

    var formPanel = Ext.getCmp('scripting-' + item + '-form');
    var form = formPanel.getForm();

    $S[ name + 'FormPanel' ] = formPanel;
    $S[ name + 'Form' ] = form;

    form.on('actioncomplete', function(form, action) {
      if (action.type == 'load') {
        if (form.findField('category')) {
          form.findField('category').store.loadData(action.result.data.categories || []);
        }

        if (item == 'event') {
          form.findField('params').store.loadData(action.result.data.tags || []);
        }
      }
      else if (action.type == 'submit') {
        $S.hideForm.apply(this);
        $S.reload();
      }
    }, name);

    Ext.getCmp('scripting-btn-' + item).setHandler($S.showPanel, item);
    Ext.getCmp('scripting-' + item + '-add').setHandler($S.showForm, bindParams);

    Ext.getCmp('scripting-' + item + '-form-submit').setHandler($S.submitForm, bindParams);
    Ext.getCmp('scripting-' + item + '-form-cancel').setHandler($S.hideForm, name);

    $S[ name + 'Grid' ].on('cellclick', function(grid, r, c, event) {
      var id = grid.getStore().getAt(r).get('id');
      var fieldName = grid.getColumnModel().getDataIndex(c);

      if (fieldName == 'duplicate') {
        $S[ this.name + 'Grid' ].loadMask.show();

        Ext.Ajax.request({
          url: Scada.reqUrl('scripting', 'duplicate'),
          success: $S.reload,
          failure: $S.reload,
          params: Scada.encodeData({ id: id })
        });
      }
      else if (fieldName == 'delete' || fieldName == 'active' || fieldName == 'source') {
        var mode = fieldName == 'active' ? 'status' : fieldName;
        Ext.Msg.confirm(
          $tr('scripting_' + mode + '_title'),
          $tr('scripting_' + mode + '_confirm'),
          $S.onConfirm,
          { name: this.name, id: id, mode: mode }
        );
      }
      else if (fieldName == 'editor') {
        $S.showEditor(id, 0, this.name == 'user' ? null : event);
      }
      else {
        $S.showForm.apply(this, [ id ]);
      }
    }, bindParams);
  });

  $S.restoreForm = Ext.getCmp('scripting-restore-form').getForm();
  $S.helpersImportForm = Ext.getCmp('scripting-helpers-import-form').getForm();
  $S.toolsButton = Ext.getCmp('scripting-btn-tools');

  $S.helpersImportForm.on('actioncomplete', function() {
    $S.helpersImportWindow.hide();
    Ext.Msg.alert($tr('scripting_tools_helpers_import'), $tr('scripting_tools_helpers_import_ok'));
  });

  $S.helpersImportForm.on('actionfailed', function(form, action) {
    if (action.result && action.result.error) {
      Ext.Msg.alert(
        $tr('scripting_tools_helpers_import'),
        $tr('scripting_tools_helpers_import_error')
      );
    }
  });

  Ext.getCmp('scripting-restore-form-cancel').setHandler(function() {
    $S.restoreWindow.hide();
  });

  Ext.getCmp('scripting-restore-form-submit').setHandler(function() {
    $S.restoreForm.submit({ waitMsg: $tr('loading') });
  });

  $S.restoreForm.on('actioncomplete', function() {
    $S.restoreWindow.hide();
    Ext.Msg.alert($tr('utilities_restore'), $tr('utilities_restore_ok'));
  });

  $S.restoreForm.on('actionfailed', function(form, action) {
    if (action.result && action.result.error) {
      Ext.Msg.alert(
        $tr('scripting_restore_error_title'),
        $tr('scripting_restore_error_' + action.result.error)
      );
    }
  });

  Ext.getCmp('scripting-helpers-import-form-cancel').setHandler(function() {
    $S.helpersImportWindow.hide();
  });

  Ext.getCmp('scripting-helpers-import-form-submit').setHandler(function() {
    $S.helpersImportForm.submit({ waitMsg: $tr('loading') });
  });

  $S.dtParams = Ext.getCmp('scripting-scheduled-form-params');
  $S.dtMinute = Ext.getCmp('scripting-scheduled-form-minute');
  $S.dtHour = Ext.getCmp('scripting-scheduled-form-hour');
  $S.dtMonthday = Ext.getCmp('scripting-scheduled-form-monthday');
  $S.dtMonth = Ext.getCmp('scripting-scheduled-form-month');
  $S.dtWeekday = Ext.getCmp('scripting-scheduled-form-weekday');

  Ext.Ajax.request({
    url: '/scada/help/en/scripting-scheduled.html',
    success: function(r) {
      $S.cronHelp = r.responseText;
    }
  });

  Ext.each([ $S.dtMinute, $S.dtHour, $S.dtMonthday ], function(f) {
    f.on('afterrender', function(f) {
      f.label.addClass('x-label-hint');
      Ext.QuickTips.register({
        target: f.label,
        cls: 'x-html-tip',
        text: $S.cronHelp
      });
    });
  });

  $S.sForm.on('actioncomplete', function(form, action) {
    if (action.type == 'load') {
      var data = action.result.data;
      if (data.params) {
        var runParams = data.params.split(' ');

        if (runParams.length == 5) {
          $S.dtMinute.setValue(runParams[0]);
          $S.dtHour.setValue(runParams[1]);
          $S.dtMonthday.setValue(runParams[2]);

          var rMonth = runParams[ 3 ] == '*' ? '1,2,3,4,5,6,7,8,9,10,11,12' : runParams[ 3 ];
          $S.dtMonth.setValue(rMonth);

          var rWeekday = runParams[ 4 ] == '*' ? '0,1,2,3,4,5,6' : runParams[ 4 ];
          $S.dtWeekday.setValue(rWeekday);
        }
      }
    }
  });

  $S.sForm.on('beforeaction', function(form, action) {
    if (action.type == 'submit' && form.isValid()) {
      var values = [], v;

      values.push($S.dtMinute.getValue());
      values.push($S.dtHour.getValue());
      values.push($S.dtMonthday.getValue());

      v = $S.dtMonth.getValue();
      values.push((v == '1,2,3,4,5,6,7,8,9,10,11,12') ? '*' : v);

      v = $S.dtWeekday.getValue();
      values.push((v == '0,1,2,3,4,5,6' || v == '1,2,3,4,5,6,0') ? '*' : v);

      $S.dtParams.setValue(values.join(' '));
    }
  });

  Ext.getCmp('scripting-category-filter').on('select', function(field) {
    var value = field.getValue();

    Ext.each([ 'e', 'r', 's' ], function(name) {
      var store = $S[ name + 'Store' ];

      if (value) {
        store.filter('category', value, false, false, true);
      }
      else {
        store.clearFilter();
      }
    });
  });

  if (Scada.Utilities && Scada.Utilities.index == 0) {
    $S.reload();
  }
  $S.reloaderTask = { run: $S.reload, interval: 6000 };
};

Scada.Modules.push($S);
