#!/usr/bin/haserl
Content-Type: text/javascript

Scada.ObjectLogs = {};
var $OL = Scada.ObjectLogs;

$OL.store = new Ext.data.JsonStore({
  baseParams: {},
  url: Scada.reqUrl('objectlogs'),
  root: 'data',
  fields: [ 'logtime', 'objectname', 'units', 'datatype', 'datahex', 'datadec', 'address', 'src', 'eventtype' ]
});

$OL.pagination = new Ext.PagingToolbar({
  pageSize: Config.perpage,
  store: $OL.store,
  displayMsg: $tr('objectlogs_pages_hint'),
  emptyMsg: $tr('objectlogs_pages_empty'),
  beforePageText: $tr('page'),
  afterPageText: $tr('page_of'),
  items: [
    { text: $tr('utilities_reset_button'), id: 'reset-objectlogs', iconCls: 'icon-clear' }, '-',
    { text: $tr('objectlogs_export'), iconCls: 'icon-export', handler: function() { window.open('/cgi-bin/scada/general/objectlogs-csv.cgi') } }, '-'
  ]
});

$OL.filterListener = {
  specialkey: function(field, event) {
    var key = event.getKey();
    if (key == event.ENTER) {
      $OL.doFilter();
    }
    else if (key == event.ESC) {
      $OL.filterReset();
    }
  }
};

$OL.filterCallback = function() {
  $OL.filter.setDisabled(false);
};

$OL.doFilter = function() {
  var form = $OL.filter.getForm();
  if (!form.isValid()) {
    return;
  }

  var fv = form.getValues()
    , sd = Date.parseDate(fv.sd + ' ' + fv.st, 'd.m.Y H:i')
    , et = Date.parseDate(fv.ed + ' ' + fv.et, 'd.m.Y H:i');

  $OL.filter.setDisabled(true);

  $OL.store.baseParams = {
    timestamp_from: sd ? sd.format('U') : null,
    timestamp_to: et ? et.format('U') : null,
    address: fv.address,
    data: fv.data,
    tags: fv.tags,
    source: Scada.encodeIndAddress(fv.source)
  };

  $OL.store.load({
    callback: $OL.filterCallback
  });
};

$OL.filterReset = function() {
  $OL.filter.getForm().reset();
  $OL.doFilter();
};

$OL.filter = new Ext.form.FormPanel({
  title: $tr('objectlogs_filter'),
  region: 'west',
  collapsible: true,
  width: 188,
  margins: '10 10 10 10',
  cmargins: '10 10 10 10',
  labelAlign: 'top',
  padding: 10,
  defaults: {
    width: 166
  },
  items: [
    {
      xtype: 'compositefield', fieldLabel: $tr('objectlogs_filter_start_date'), msgTarget: 'under',
      items: [
        { xtype: 'datefield', name: 'sd', allowBlank: true, startDay: 1, format: 'd.m.Y', value: '', width: 96 },
        { xtype: 'timefield', name: 'st', allowBlank: true, format: 'H:i', increment: 30, value: '', width: 65 }
      ]
    },
    {
      xtype: 'compositefield', fieldLabel: $tr('objectlogs_filter_end_date'), msgTarget: 'under',
      items: [
        { xtype: 'datefield', name: 'ed', allowBlank: true, startDay: 1, format: 'd.m.Y', value: '', width: 96 },
        { xtype: 'timefield', name: 'et', allowBlank: true, format: 'H:i', increment: 30, value: '', width: 65 }
      ]
    },
    { xtype: 'textfield', fieldLabel: $tr('objects_filter_name_ga'), name: 'address', allowBlank: true, listeners: $OL.filterListener },
    { xtype: 'tagbox', fieldLabel: $tr('objects_filter_tags'), name: 'tags', id: 'objectlogs-filter-tags', allowBlank: true, listeners: $OL.filterListener },
    { xtype: 'textfield', fieldLabel: $tr('objectlogs_datafilter'), name: 'data', allowBlank: true, listeners: $OL.filterListener },
    { xtype: 'textfield', fieldLabel: $tr('objectlogs_src'), name: 'source', allowBlank: true, vtype: 'knxia', listeners: $OL.filterListener, msgTarget: 'title' },
    {
      xtype: 'container',
      layout: 'hbox',
      layoutConfig: {
        pack: 'end',
        defaultMargins: '5 0 0 5',
      },
      items: [
        { text: $tr('objects_filter_submit'), xtype: 'button', handler: $OL.doFilter },
        { text: $tr('objects_filter_reset'), xtype: 'button', handler: $OL.filterReset }
      ]
    }
  ]
});

$OL.grid = new Ext.grid.GridPanel({
  region: 'center',
  border: true,
  enableColumnHide: false,
  enableColumnMove: false,
  store: $OL.store,
  id: 'objectlog-grid',
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('objectlogs_time'), width: 0.15, dataIndex: 'logtime', xtype: 'timestampcolumn' },
      { header: $tr('objectlogs_address'), width: 0.10, dataIndex: 'address', renderer: Scada.decodeGroupAddress },
      { header: $tr('objectlogs_eventtype'), width: 0.10, dataIndex: 'eventtype' },
      { header: $tr('objectlogs_src'), width: 0.10, dataIndex: 'src', renderer: Scada.decodeIndAddress },
      { header: $tr('objectlogs_name'), width: 0.10, dataIndex: 'objectname' },
      { header: $tr('objectlogs_datadec'), width: 0.15, dataIndex: 'datadec', xtype: 'knxdtcolumn' },
      { header: $tr('objects_datatype'), width: 0.15, dataIndex: 'datatype', trPrefix: 'objects_datatype', xtype: 'translatecolumn' },
      { header: $tr('objectlogs_datahex'), width: 0.15, dataIndex: 'datahex' }
    ]
  }),
  bbar: $OL.pagination,
  margins: '10 10 10 0',
  viewConfig: { forceFit: true },
  sm: new Ext.grid.RowSelectionModel({ singleSelect: true })
});

$OL.container = new Ext.Container({
  id: 'ObjectLogs',
  title: $tr('objectlogs_title'),
  layout: 'border',
  items: [ $OL.filter, $OL.grid ]
});

$OL.update = function() {
  $OL.pagination.doRefresh();
};

$OL.onFocus = function() {
  $OL.update();

  if ($O.rawTags) {
    Ext.getCmp('objectlogs-filter-tags').updateTags($O.rawTags);
  }

  Scada.Objects.addListener('objectlogs', $OL.update);
};

$OL.onBlur = function() {
  Scada.Objects.removeListener('objectlogs');
};

Scada.Modules.push($OL);
