#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Vis = {};
var $V = Scada.Vis;

$V.center = Config.viscenter.toNumber() > 0;
$V.centerWidth = Config.viscenter.toNumber() == 3;
$V.editorTypes = [];

$V.iconStore = new Ext.data.JsonStore({
  fields: ['icon', 'mtime'],
  root: 'images',
  url: Scada.reqUrl('vis', 'graphics')
});

$V.imgStore = new Ext.data.JsonStore({
  fields: ['image', 'mtime'],
  root: 'images',
  url: Scada.reqUrl('vis', 'graphics'),
  baseParams: { mode: 'img' }
});

$V.visIconStore = new Ext.data.ArrayStore({
  fields: ['icon', 'name']
});

$V.graphIconStore = new Ext.data.ArrayStore({
  fields: ['icon', 'name']
});

$V.fontStore = new Ext.data.ArrayStore({
  fields: ['font']
});

$V.iconSelects = [];

$V.tryDecode = function(val, dflt) {
  var res;

  if (typeof dflt === 'undefined') {
    dflt = {};
  }

  try {
    res = Ext.decode(val);
  }
  catch (e) {}

  return res || dflt;
};

$V.createIconSelect = function(mode, type, override) {
  var suffix = type ? ('_' + type) : '', id, config;

  id = 'scada-vis-icon-select-' + mode + suffix.replace(/_/g, '-');
  config = $extend({
    id: id,
    name: 'icon' + suffix,
    xtype: 'trigger',
    width: 160,
    hidden: true,
    editable: false,
    allowBlank: false,
    fieldLabel: $tr('vis_icon' + suffix)
  }, override || {}, true);

  $V.iconSelects.push(id);
  return config;
};

$V.fontSelect = {
  xtype: 'combo',
  width: 160,
  mode: 'local',
  triggerAction: 'all',
  fieldLabel: $tr('vis_font'),
  name: 'font',
  valueField: 'font',
  displayField: 'font',
  store: $V.fontStore
};

$V.iconSelectUpdateCount = function(count) {
  var text = count ? (' (' + count + ')') : '';
  Ext.getCmp('scada-vis-icons-additional').setText($tr('vis_icons_additional') + text);
};

$V.iconSelectSubmit = function(fw) {
  var values = fw.form.clearInvalid().getValues(), result = [], store = {}, valid = true;

  Ext.iterate(values, function(key, value) {
    var chunks = key.split('_'), name = chunks[0], id = chunks[1];

    if (!store[ id ]) {
      store[ id ] = {};
    }

    store[ id ][ name ] = name == 'icon' ? value : value.toNumber();
  });

  Ext.iterate(store, function(id, values) {
    if (isNaN(values.min) || isNaN(values.max) || values.max < values.min) {
      valid = false;
      fw.form.findField('max_' + id).markInvalid($tr('vis_icon_max_error'));
    }
    else {
      result.push(values);
    }
  });

  if (valid) {
    Ext.getCmp('scada-vis-icons-additional-store').setValue(Ext.encode(result));
    $V.iconSelectUpdateCount(result.length);
    fw.hide();
  }
};

$V.iconSelectRestore = function() {
  var value = Ext.getCmp('scada-vis-icons-additional-store').getValue(), items;
  try {
    items = Ext.decode(value);
    if (typeof items == 'object') {
      Ext.each(items, $V.addAdditionalIcon);
    }
  }
  catch (e) {}
};

$V.iconSelectWindow = new ExtFormWindow({
  id: 'scada-vis-icon-select-window',
  window: {
    title: $tr('vis_icons_additional'),
    width: 600,
    height: 400,
  },
  form: {
    autoScroll: true,
    items: []
  },
  events: { onSubmit: $V.iconSelectSubmit }
});

$V.deleteAdditionalIcon = function() {
  var id = this.id.replace('_delete', ''), cmp = Ext.getCmp(id);
  $V.iconSelectWindow.formPanel.remove(cmp, true);
};

$V.additionalIconId = 1;

$V.addAdditionalIcon = function(values) {
  var id = $V.additionalIconId, items = $V.iconSelectWindow.formPanel.items, index = items.length - 1, bounds, dec, dt, min, max, sel;

  bounds = Scada.dtGetBounds($V.iconSelectWindowObj.datatype);

  if (values && !values.id) {
    values = values;
  }
  else {
    values = {};
    if (index > 0) {
      try {
        items = items.items[ index - 1 ].items.items;
        min = items[ 1 ].getValue().toNumber();
        max = items[ 3 ].getValue().toNumber();

        if (min !== null && max !== null) {
          values.min = (max).constrain(bounds[0], bounds[1]);
          values.max = (max + (max - min)).constrain(bounds[0], bounds[1]);
        }
      }
      catch (e) {}
    }
  }

  dt = $V.iconSelectWindowObj.datatype.major;
  dec = dt == Scada.dt.float16 || dt == Scada.dt.float32;

  sel = $V.createIconSelect('additional', id, { hidden: false, value: values.icon });

  $V.iconSelectWindow.formPanel.insert(index, {
    id: 'vis_icons_additional_row_' + id,
    xtype: 'compositefield',
    hideLabel: true,
    width: 540,
    items: [
      { xtype: 'displayfield', value: $tr('vis_min_value') },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: dec, name: 'min_' + id, width: 80, value: values.min },
      { xtype: 'displayfield', value: $tr('vis_max_value') },
      { xtype: 'spinnerfield', minValue: bounds[0], maxValue: bounds[1], allowBlank: false, allowDecimals: dec, name: 'max_' + id, width: 80, value: values.max },
      { xtype: 'displayfield', value: $tr('vis_icon') },
      sel,
      { xtype: 'button', iconCls: 'icon-delete', width: 16, id: 'vis_icons_additional_row_' + id + '_delete', handler: $V.deleteAdditionalIcon }
    ]
  });

  Scada.bindImgSelect($V, sel.id, 'icon');
  $V.iconSelectWindow.formPanel.doLayout();

  $V.additionalIconId++;
};

$V.showIconSelect = function() {
  var o, so, obj;

  o = $V.comboBoxes.object.getValue();
  so = $V.comboBoxes.statusobject.getValue();

  obj = $V.objectStore.getObject(so || o);
  if (!obj || !Scada.dtIsNumeric(obj.datatype.major)) {
    return;
  }

  $V.iconSelectWindowObj = obj;
  $V.iconSelectWindow.formPanel.removeAll(true);

  $V.iconSelectWindow.formPanel.add({
    id: 'scada-vis-icon-add',
    xtype: 'button',
    text: $tr('vis_icon_add'),
    handler: $V.addAdditionalIcon,
    iconCls: 'icon-add',
    width: 'auto'
  });

  $V.iconSelectRestore();

  $V.iconSelectWindow.show();
  $V.iconSelectWindow.formPanel.doLayout();
};

$V.prepareForm = {};
$V.prepareAllForms = function() {
  Ext.iterate($V.prepareForm, function(_, fn) {
    fn();
  });
};

$V.prepareForm.floorlink = function() {
  var form = $V.editorForms.floorlink
    , show = form.findField('displaymode').getValue() == 'value'
    , ext = form.findField('floorid').getValue() == 'url';

  form.findField('url').setVisible(ext).setDisabled(!ext);

  Ext.getCmp('scada-vis-icon-select-floorlink').setVisible(!show).setDisabled(show);
  Ext.getCmp('scada-vis-icon-select-floorlink-active').setVisible(!show).setDisabled(show);

  if (show) {
    form.findField('width').setValue('');
    form.findField('height').setValue('');
  }

  Ext.each(form.items.items.slice(-5, -1), function(item) {
    item.setVisible(show).setDisabled(!show);
  });

  $V.manualResizeSetVisible(show);
};

$V.prepareForm.object = function() {
  var o, so, obj, mode, form, bool, numeric, rgb, hide, show, count, icons, widget, showcontrol;

  form = $V.editorForms.object;

  o = $V.comboBoxes.object.getValue();
  so = $V.comboBoxes.statusobject.getValue();
  mode = form.findField('displaymode').getValue();
  obj = $V.objectStore.getObject(so || o);
  widget = ($V.comboBoxes.objectwidget.getValue() || 0).toNumber();
  showcontrol = form.findField('showcontrol').checked;

  bool = obj ? obj.boolean : false;
  numeric = obj ? Scada.dtIsNumeric(obj.datatype.major) : false;
  rgb = obj ? Scada.isRgb(obj.datatype) : false;

  hide = mode == 'value';
  show = obj && !hide;
  Ext.getCmp('scada-vis-icon-select-object-default')
    .setVisible(show && !bool).setDisabled(!show || bool).allowBlank = showcontrol;
  Ext.getCmp('scada-vis-icon-select-object-on')
    .setVisible(show && bool).setDisabled(!show || !bool).allowBlank = showcontrol;
  Ext.getCmp('scada-vis-icon-select-object-off')
    .setVisible(show && bool).setDisabled(!show || !bool).allowBlank = showcontrol;
  Ext.getCmp('scada-vis-icons-additional').setVisible(show && numeric);

  show = mode != 'icon' && !rgb;
  Ext.each([ 'size', 'color', 'font', 'backdrop' ], function(name) {
    form.findField(name).setVisible(show).setDisabled(!show);
  });

  Ext.getCmp('scada-vis-object-text-styles').setVisible(mode != 'icon' && !rgb);

  form.findField('readonly').setVisible(!widget);
  form.findField('fixedvalue').setVisible(!widget);
  form.findField('update').setVisible(!widget);
  form.findField('showcontrol').setVisible(!widget);

  try {
    icons = Ext.getCmp('scada-vis-icons-additional-store').getValue();
    icons = Ext.decode(icons);
    count = icons ? icons.length : 0;
    $V.iconSelectUpdateCount(count);
  }
  catch(e) {}

  if (hide) {
    form.findField('width').setValue('');
    form.findField('height').setValue('');
  }

  $V.manualResizeSetVisible(hide);
  $V.EditorPanel.doLayout();
};

$V.prepareForm.image = function() {
  var source = Ext.getCmp('scada-vis-image-source').getValue();
  Ext.getCmp('vis-src-local').setVisible(source == 'local').setDisabled(source != 'local');
  Ext.getCmp('vis-src-remote').setVisible(source == 'remote').setDisabled(source != 'remote');
};

$V.prepareForm.iframe = function() {
  var source = Ext.getCmp('scada-vis-iframe-source').getValue();
  Ext.getCmp('vis-iframe-url').setVisible(source == 'url').setDisabled(source != 'url');
};

$V.showVisParams = function(btn) {
  var id = $V.comboBoxes[ btn.mode ].getValue(), item;
  if (id) {
    item = $O.store.getById(id);

    if (item) {
      $O.showVisParams(item.data);
    }
  }
};

$V.validateFixedValue = function(value) {
  var visible, id, obj, res = true;

  visible = $V.editorForms.object.findField('fixedvalue').isVisible();
  id = $V.comboBoxes.object.getValue();
  obj = $V.objectStore.getObject(id);

  if (value && obj && visible) {
    res = Scada.validateValue(value, obj.rawdatatype);
  }

  return res;
};

$V.touchOrderShow = function() {
  var root, floor = $V.floor, res = [];

  if (!floor) {
    return;
  }
  else if (floor.type == 'layout') {
    Ext.Msg.alert($tr('error'), $tr('vis_touch_order_layout_error'));
    return;
  }

  Ext.each($V.floorObjects, function(item) {
    var type = $V.floorController.mapping[ item.type ], obj, name = '';

    if (item.notouch) {
      return;
    }

    if (type == 'object') {
      obj = $O.extObjStore.getExact('id', item.object.toNumber());
      if (obj) {
        name = obj.text;
      }
    }
    else if (type == 'floorlink') {
      obj = item.object.toNumber();

      if (obj) {
        obj = $V.comboBoxes.floor.store.getExact('id', obj);
        if (obj) {
          name = obj.text;
        }
      }
      else {
        name = $tr(item.object + '_title');
      }
    }
    else if (type != 'camera' && type != 'iframe') {
      return;
    }

    if (item.name) {
      if (name) {
        name = item.name + ' (' + name + ')';
      }
      else {
        name = item.name;
      }
    }

    if (name) {
      name = ': ' + name;
    }

    res.push({
      id: item.id,
      text: $tr('vis_widget_' + type) + name,
      cls: 'icon-' + type,
      leaf: true,
      draggable: true,
      sortorder: item.sortorder ? item.sortorder.toNumber() : 0
    });
  });

  if (!res.length) {
    return Ext.Msg.alert(
      $tr('error'),
      $tr('vis_touch_order_error')
    );
  }

  res.sort(function(a, b) {
    return a.sortorder.toNumber() - b.sortorder.toNumber();
  });

  root = new Ext.tree.AsyncTreeNode({
    draggable: false,
    children: res
  });

  $V.TouchOrderWin.show();
  $V.TouchOrderTree.setRootNode(root);
};

$V.touchOrderHide = function() {
  $V.TouchOrderWin.hide();
};

$V.touchOrderSave = function() {
  var nodes = $V.TouchOrderTree.root.childNodes, data = {};

  Ext.each(nodes, function(node, order) {
    data[ node.attributes.id ] = order + 1;
  });

  $V.touchOrderHide();

  Ext.Ajax.request({
    url: Scada.reqUrl('vis', 'order-save'),
    success: $V.onSaveFloor,
    failure: $V.onSaveFloor,
    params: Scada.encodeData(data)
  });
};

$V.TouchOrderWin = new Ext.Window($def.window({
  title: $tr('vis_touch_order'),
  width: 320,
  height: 480,
  draggable: true,
  resizable: true,
  bbar: [
    '->',
    { text: $tr('save'), handler: $V.touchOrderSave }, ' ',
    { text: $tr('cancel'), handler: $V.touchOrderHide }
  ],
  items: $def.tree({
    id: 'scada-vis-touch-tree',
    root: [],
    border: false,
    enableDD: true
  })
}));

$V.Preview = function(btn) {
  var floor = $V.floor, layout = $V.layoutCfg, url = '/cgi-bin/scada-vis/', specs, w, h;

  if (!floor) {
    return;
  }
  else if (floor.type) {
    Ext.Msg.alert($tr('error'), $tr('vis_preview_error'));
  }
  else {
    w = floor.width;
    h = floor.height;

    if (layout) {
      w = Math.max(w, layout.width);
      h = Math.max(h, layout.height);
    }

    specs = 'menubar=no,status=no,toolbar=no,width=' + w + ',height=' + h;
    url += btn.ptype == 'usermode' ? 'index.cgi' : 'touch.html';
    window.open(url + '#' + $V.floorId, '_blank', specs);
  }
};

$V.createEditorPanel = function(id, items) {
  $V.editorTypes.push(id);

  return {
    id: 'scada-vis-editor-' + id + '-panel',
    type: id,
    title: $tr('vis_widget_' + id.replace('-', '_')),
    border: false,
    xtype: 'form',
    layout: 'form',
    autoScroll: true,
    labelWidth: 120,
    padding: 10,
    defaults: { width: 160 },
    items: items,
    buttons: [
      { text: $tr('vis_button_add'), id: 'scada-vis-editor-' + id + '-add', ctCls: 'x-btn-hl' },
      { text: $tr('vis_button_reset'), id: 'scada-vis-editor-' + id + '-reset' }
    ],
    listeners: {
      expand: function(item) {
        var hide = item.type == 'gauge';

        $V.editorResets[ item.type ].fireEvent('click');
        $V.floorController.stopEdit();
        $V.manualResizeSetVisible(hide);
      }
    }
  };
};

$V.createObjectSelect = function(type, store, label, name, width) {
  var cfg;

  name = name || 'object';
  if (type) {
    type += '-';
  }

  cfg = $def.select({
    fieldLabel: $tr('vis_' + label + 'object'),
    id: 'scada-vis-editor-' + type + name,
    name: name,
    allowBlank: false,
    hiddenName: name,
    editable: true,
    matchAny: true,
    mode: 'local',
    valueField: 'id',
    displayField: 'text',
    store: store,
    listWidth: 275
  });

  if (width) {
    cfg.width = width;
  }

  return cfg;
};

$V.statObjStore = new Ext.data.ArrayStore({
  fields: [ 'id', 'text' ],
  data: []
});

$V.floorEditorConfig = [
  $V.createEditorPanel('object', [
    { xtype: 'compositefield', items: [
      $V.createObjectSelect('', $O.extObjStore, 'main', null, 133),
      { xtype: 'button', iconCls: 'icon-visparams', width: 16, handler: $V.showVisParams, tooltip: $tr('objects_vis_params'), tooltipType: 'title', mode: 'object' }
    ]},
    $V.createObjectSelect('', $V.statObjStore, 'status', 'statusobject'),
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    { xtype: 'checkbox', fieldLabel: $tr('vis_readonly'), name: 'readonly', value: 1 },
    { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
    { xtype: 'hidden', name: 'sortorder', value: 0 },
    { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
    { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
    { xtype: 'textfield', fieldLabel: $tr('vis_fixedvalue'), name: 'fixedvalue', allowBlank: true, validator: $V.validateFixedValue },
    { xtype: 'checkbox', fieldLabel: $tr('vis_update'), name: 'update', value: 1, boxLabel: $tr('vis_update_hint') },
    { xtype: 'textfield', fieldLabel: $tr('vis_pincode'), name: 'pincode', minLength: 3, maxLength: 8, allowBlank: true, maskRe: /[0-9]/ },
    $def.select({ fieldLabel: $tr('vis_object_widget'), id: 'scada-vis-editor-object-widget', name: 'floorid', allowBlank: false, hiddenName: 'widget',
      mode: 'local', valueField: 'id', displayField: 'text', store: new Ext.data.ArrayStore({ fields: [ 'id', 'text' ], data: [] }), value: '', listWidth: 275
    }),
    $def.select({ name: 'displaymode', hiddenName: 'displaymode', fieldLabel: $tr('vis_displaymode'), value: 'icon-value', id: 'scada-vis-object-display-mode',
      store: [['icon-value',  $tr('vis_displaymode_icon_value')], ['icon',  $tr('vis_displaymode_icon')], ['value',  $tr('vis_displaymode_value')]]
    }),
    $V.createIconSelect('object', 'default'),
    $V.createIconSelect('object', 'touch', { hidden: false, allowBlank: true, xtype: 'imgtrigger' }),
    $V.createIconSelect('object', 'on'),
    $V.createIconSelect('object', 'off'),
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true },
    { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 12, minValue: 10, maxValue: 100, allowDecimals: false },
    { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', id: 'scada-vis-object-text-styles', items: [
      { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
      { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
      { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
    ]},
    $V.fontSelect,
    { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '' },
    { xtype: 'checkbox', fieldLabel: $tr('vis_backdrop'), name: 'backdrop', value: 1 },
    { xtype: 'checkbox', fieldLabel: $tr('vis_showcontrol'), boxLabel: $tr('vis_showcontrol_hint'), name: 'showcontrol', value: 1, id: 'scada-vis-editor-object-showcontrol' },
    { xtype: 'button', text: $tr('vis_icons_additional'), id: 'scada-vis-icons-additional', handler: $V.showIconSelect, hidden: true },
    { xtype: 'hidden', id: 'scada-vis-icons-additional-store', name: 'icons_add' }
  ]),
  $V.createEditorPanel('floorlink', [
    $def.select({ fieldLabel: $tr('vis_link_to'), id: 'scada-vis-editor-floor', name: 'floorid', allowBlank: false, hiddenName: 'floorid',
      mode: 'local', valueField: 'id', displayField: 'text', store: new Ext.data.ArrayStore({ fields: [ 'id', 'text' ], data: [] }), listWidth: 275
    }),
    { xtype: 'textfield', fieldLabel: $tr('vis_link'), name: 'url', hidden: true },
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
    { xtype: 'hidden', name: 'sortorder', value: 0 },
    { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
    { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
    $def.select({ name: 'displaymode', hiddenName: 'displaymode', fieldLabel: $tr('vis_displaymode'), value: 'icon', id: 'scada-vis-link-display-mode',
      store: [['icon', $tr('vis_displaymode_icon')], ['value', $tr('vis_displaymode_value')]]
    }),
    $V.createIconSelect('floorlink', '', { hidden: false }),
    $V.createIconSelect('floorlink', 'active', { hidden: false, allowBlank: true, xtype: 'imgtrigger' }),
    { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 14, minValue: 10, maxValue: 100, allowDecimals: false },
    { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', items: [
      { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
      { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
      { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
    ]},
    $V.fontSelect,
    { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '' },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true }
  ]),
  $V.createEditorPanel('label', [
    { xtype: 'textfield', fieldLabel: $tr('vis_label_text'), name: 'name', allowBlank: false },
    { xtype: 'spinnerfield', fieldLabel: $tr('vis_label_font_size'), name: 'size', allowBlank: false, value: 14, minValue: 10, maxValue: 100, allowDecimals: false },
    { xtype: 'checkboxgroup', fieldLabel: $tr('vis_text_styles'), columns: 3, itemCls: 'x-check-group-alt', items: [
      { boxLabel: '<b>B</b>', name: 'bold', value: 1 },
      { boxLabel: '<i>I</i>', name: 'italic', value: 1 },
      { boxLabel: '<u>U</u>', name: 'underline', value: 1 },
    ]},
    $V.fontSelect,
    { xtype: 'colorfield', fieldLabel: $tr('vis_color'), name: 'color', value: '' },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true }
  ]),
  $V.createEditorPanel('image', [
    $def.select({ name: 'source', hiddenName: 'source', fieldLabel: $tr('vis_img_source'), value: 'local', id: 'scada-vis-image-source',
      store: [['local',  $tr('vis_img_source_local')], ['remote',  $tr('vis_img_source_remote')]]
    }),
    { xtype: 'textfield', fieldLabel: $tr('vis_src_remote'), name: 'src_remote', allowBlank: false, hidden: true, id: 'vis-src-remote' },
    { xtype: 'trigger', fieldLabel: $tr('vis_select_image'), name: 'src_local', editable: false, hidden: true, id: 'vis-src-local' },
    { xtype: 'compositefield', fieldLabel: $tr('vis_img_size'), items: [
      { xtype: 'spinnerfield', width: 78, name: 'width', allowBlank: true, minValue: 16, maxValue: 2560, allowDecimals: false },
      { xtype: 'spinnerfield', width: 77, name: 'height', allowBlank: true, minValue: 16, maxValue: 2560, allowDecimals: false }
    ]},
    { xtype: 'textfield', fieldLabel: $tr('vis_link'), name: 'link', allowBlank: true },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true }
  ]),
  $V.createEditorPanel('iframe', [
    $def.select({ name: 'source', hiddenName: 'source', fieldLabel: $tr('vis_iframe_source'), value: 'url', id: 'scada-vis-iframe-source',
      store: [['url',  $tr('vis_iframe_source_url')], ['schedulers',  $tr('vis_iframe_source_schedulers')], ['trends',  $tr('vis_iframe_source_trends')]]
    }),
    { xtype: 'textfield', fieldLabel: $tr('vis_iframe_source_url'), name: 'url', allowBlank: false, id: 'vis-iframe-url' },
    { xtype: 'compositefield', fieldLabel: $tr('vis_iframe_size'), items: [
      { xtype: 'spinnerfield', width: 78, name: 'width', allowBlank: false, minValue: 16, maxValue: 2560, value: 480, allowDecimals: false },
      { xtype: 'spinnerfield', width: 77, name: 'height', allowBlank: false, minValue: 16, maxValue: 2560, value: 320, allowDecimals: false }
    ]},
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    { xtype: 'checkbox', fieldLabel: $tr('vis_notouch'), name: 'notouch', value: 1 },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true },
    { xtype: 'hidden', name: 'sortorder', value: 0 }
  ]),
  $V.createEditorPanel('gauge', [
    { xtype: 'compositefield', items: [
      $V.createObjectSelect('gauge', $O.extNumObjStore, 'data', null, 133),
      { xtype: 'button', iconCls: 'icon-visparams', width: 16, handler: $V.showVisParams, tooltip: $tr('objects_vis_params'), tooltipType: 'title', mode: 'gauge' }
    ]},
    { xtype: 'spinnerfield', fieldLabel: $tr('vis_gauge_size'), name: 'size', allowBlank: true, minValue: 100, maxValue: 500, value: 200, allowDecimals: false },
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    { xtype: 'checkbox', fieldLabel: $tr('vis_readonly'), name: 'readonly', value: 1 },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true }
  ]),
  $V.createEditorPanel('camera', [
    { xtype: 'textfield', fieldLabel: $tr('vis_src'), name: 'src', allowBlank: false },
    { xtype: 'compositefield', fieldLabel: $tr('vis_win_size'), items: [
      { xtype: 'spinnerfield', width: 78, name: 'win_width', allowBlank: false, minValue: 160, maxValue: 1024, value: 640, allowDecimals: false },
      { xtype: 'spinnerfield', width: 77, name: 'win_height', allowBlank: false, minValue: 120, maxValue: 1024, value: 480, allowDecimals: false }
    ]},
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    $V.createIconSelect('camera', '', { hidden: false, value: 'camera.png' }),
    { xtype: 'checkbox', fieldLabel: $tr('vis_autoopen'), name: 'autoopen', value: 1 },
    { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true },
    { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' },
    { xtype: 'hidden', name: 'sortorder', value: 0 }
  ]),
  $V.createEditorPanel('graph', [
    $V.createObjectSelect('graph', $O.extNumObjStore, 'data'),
    { xtype: 'textfield', fieldLabel: $tr('vis_name'), name: 'name', allowBlank: true },
    $V.createIconSelect('graph', '', { hidden: false }),
    { xtype: 'compositefield', fieldLabel: $tr('vis_win_size'), items: [
      { xtype: 'spinnerfield', width: 78, name: 'win_width', allowBlank: false, minValue: 320, maxValue: 1024, value: 640, allowDecimals: false },
      { xtype: 'spinnerfield', width: 77, name: 'win_height', allowBlank: false, minValue: 240, maxValue: 1024, value: 480, allowDecimals: false }
    ]},
    { xtype: 'spinnerfield', fieldLabel: $tr('vis_limit'), name: 'limit', allowBlank: false, minValue: 5, maxValue: 200, value: 10, allowDecimals: false },
    { xtype: 'checkbox', fieldLabel: $tr('vis_autoopen'), name: 'autoopen', value: 1 },
    { xtype: 'checkbox', fieldLabel: $tr('vis_nobg'), name: 'nobg', checked: true, value: 1 },
    { xtype: 'textfield', fieldLabel: $tr('vis_cls'), name: 'cls', allowBlank: true },
    { xtype: 'hidden', name: 'width' }, { xtype: 'hidden', name: 'height' }
  ])
];

$V.container = new Ext.Container({
  id: 'Vis',
  layout: 'border',
  title: $tr('vis_title'),
  hideMode: 'offsets',
  items: [
    { xtype: 'panel', margins: '10 10 10 10', cmargins: '10 10 10 10', title: $tr('vis_buildings'), region: 'west', collapsible: true, floatable: false, animCollapse: false, layout: 'vbox',
      width: 250, id: 'scada-vis-sidebar', layoutConfig: { align: 'stretch' },
      items: [
        $def.tree({ id: 'scada-vis-tree', dataUrl: Scada.reqUrl('vis', 'buildings'), border: false, flex: 1, margins: 0 }),
        { xtype: 'form', id: 'scada-vis-plan-size', border: false, hidden: true, padding: 10, labelWidth: 70, items:
          {
            xtype: 'compositefield',
            fieldLabel: $tr('vis_plan_size'),
            msgTarget: 'title',
            items: [
              { id: 'scada-vis-plan-width', xtype: 'spinnerfield', minValue: 16, maxValue: 5120, width: 55, enableKeyEvents: true, value: 1024, allowDecimals: false },
              { id: 'scada-vis-plan-height', xtype: 'spinnerfield', minValue: 16, maxValue: 5120, width: 55, enableKeyEvents: true, value: 768, allowDecimals: false },
              { xtype: 'button', iconCls: 'icon-list-view', menu: {
                  id: 'scada-vis-plan-menu',
                  items: $B.sizePresets
                }
              }
            ]
          }
        },
      ],
      bbar: [
        { xtype: 'button', text: $tr('utilities_config_vis'), handler: function() { Scada.Utilities.configVisFW.show(); } }, ' ',
        { xtype: 'button', iconCls: 'icon-vis-order', tooltip: $tr('vis_touch_order_edit'), handler: $V.touchOrderShow }, ' ',
        { xtype: 'button', iconCls: 'icon-vis-usermode', ptype: 'usermode', tooltip: $tr('vis_view_usermode'), handler: $V.Preview }, ' ',
        { xtype: 'button', iconCls: 'icon-vis-touch', ptype: 'touch', tooltip: $tr('vis_view_touch'), handler: $V.Preview }
      ]
    },
    { xtype: 'panel', margins: '10 10 10 0', id: 'scada-vis-wrap', region: 'center', autoScroll: true },
    { xtype: 'panel', id: 'scada-vis-editor', region: 'east', title: $tr('vis_editor'),
      margins: '10 10 10 0', cmargins: '10 10 10 0', collapsible: true, animCollapse: false, width: 340, collapsed: false,
      layout: 'vbox',
      items: [
        { xtype: 'tabpanel', id: 'scada-vis-editor-panel', border: false, flex: 1, enableTabScroll: true, scrollIncrement: 200, items: $V.floorEditorConfig, activeTab: 0 },
        {
          xtype: 'form', id: 'scada-vis-widget-common-props', border: false, labelWidth: 120, width: 340, padding: 10, items:
          [
            {
              xtype: 'compositefield',
              fieldLabel: $tr('vis_loc'),
              msgTarget: 'title',
              items: [
                { id: 'scada-vis-loc-x', xtype: 'spinnerfield', value: 10, minValue: 0, maxValue: 2560, width: 60, enableKeyEvents: true, allowBlank: false, allowDecimals: false },
                { id: 'scada-vis-loc-y', xtype: 'spinnerfield', value: 10, minValue: 0, maxValue: 2560, width: 60, enableKeyEvents: true, allowBlank: false, allowDecimals: false }
              ]
            },
            {
              id: 'scada-vis-size',
              xtype: 'compositefield',
              fieldLabel: $tr('vis_size'),
              msgTarget: 'title',
              items: [
                { id: 'scada-vis-size-w', xtype: 'spinnerfield', minValue: 16, maxValue: 2560, width: 60, enableKeyEvents: true, allowBlank: true, allowDecimals: false },
                { id: 'scada-vis-size-h', xtype: 'spinnerfield', minValue: 16, maxValue: 2560, width: 60, enableKeyEvents: true, allowBlank: true, allowDecimals: false },
                { id: 'scada-vis-size-reset', xtype: 'button', iconCls: 'icon-delete' },
                { id: 'scada-vis-size-lock-ar', xtype: 'button', iconCls: 'icon-lock', enableToggle: true }
              ]
            }
          ]
        },
        { xtype: 'panel', layout: 'column', border: false, cls: 'x-column-menu', margins: '10 10 10 10', items:
          [
            { xtype: 'button', text: $tr('vis_unlock'), id: 'scada-vis-editor-control', height: 30, width: 238, disabled: true },
            { xtype: 'button', text: $tr('cancel'), id: 'scada-vis-editor-cancel', height: 30, width: 70, disabled: true }
          ]
        }
      ]
    }
  ]
});

$V.toBoolean = function(v) {
  var n = (v && v.toNumber) ? v.toNumber() : 0;
  return v === true || (!isNaN(n) && n);
};

$V.objectStore = {
  objects: {},
  update: function(data) {
    for (var i = data.length - 1; i >= 0; i--) {
      var item = data[ i ];
      this.register(item.data);
    }
  },
  empty: function() {
    this.objects = {};
  },
  register: function(data) {
    if (!data.id || data.datadec === undefined) {
      return;
    }

    var id = data.id.toNumber(), o = this.objects[ id ], params;
    params = $V.tryDecode(data.visparams || '');

    if (o) {
      if (o.name != data.name) {
        o.name = data.name;
        this.notify(null, o, 'name');
      }

      if (!$compare(o.value, data.datadec)) {
        o.value = data.datadec;
        this.notify(null, o, 'value');
      }

      if (o.rawdatatype != data.datatype) {
        o.rawdatatype = data.datatype;
        o.datatype = Scada.normalizeDt(data.datatype);
      }

      o.enums = data.enums;
      o.params = params;
    }
    else {
      var dt = Scada.normalizeDt(data.datatype);
      this.objects[ id ] = {
        rawaddress: data.address.toNumber(),
        address: Scada.decodeGroupAddress(data.address),
        name: data.name,
        units: data.units,
        datatype: dt,
        rawdatatype: data.datatype,
        boolean: (dt.major == 1),
        value: data.datadec,
        params: params,
        enums: data.enums,
        listeners: {}
      };
    }
  },
  notify: function(listenerId, o, type) {
    var listener;

    if (listenerId) {
      listener = o.listeners[ listenerId ];
      this.callListener(listener, o, type);
    }
    else {
      for (listenerId in o.listeners) {
        if (o.listeners.hasOwnProperty(listenerId)) {
          listener = o.listeners[ listenerId ];
          this.callListener(listener, o, type);
        }
      }
    }
  },
  callListener: function(listener, o, type) {
    if (listener && typeof listener.fn == 'function') {
      listener.fn.call(listener.bind, o, type);
    }
  },
  addListener: function(listenerId, objectId, v, bind) {
    var o = this.objects[ objectId ];
    if (o) {
      if (typeof v == 'function') {
        o.listeners[ listenerId ] = { fn: v, bind: bind };
        this.notify(listenerId, o, 'init');
      }
      else {
        delete o.listeners[ listenerId ];
      }
    }
  },
  removeListener: function(listenerId, objectId) {
    this.addListener(listenerId, objectId, null);
  },
  getObject: function(id) {
    return this.objects[ id.toNumber() ];
  }
};

$V.getNumericValue = function(cmp) {
  var value = parseInt(cmp.getValue(), 10);
  return isNaN(value) ? 10 : value;
};

$V.widgetPrio = [
  8, // object
  8, // object
  7, // planlink
  0, // skip
  6, // camera
  5, // graph
  3, // label
  2, // image
  4, // gauge
  1, // iframe
];

$V.floorController = {
  widgetId: 1,
  widgets: {},
  mapping: {
    0: 'object',
    1: 'object',
    2: 'floorlink',
    4: 'camera',
    5: 'graph',
    6: 'label',
    7: 'image',
    8: 'gauge',
    9: 'iframe'
  },
  panels: {
    'object': 0,
    'floorlink': 1,
    'camera': 6,
    'graph': 7,
    'label': 2,
    'image': 3,
    'gauge': 5,
    'iframe': 4
  },
  load: function(data) {
    this.setBackground($V.floor, $V.layoutCfg);

    $V.floorObjects = data.main;

    this.registerItems(data.main, $V.Content);
    this.registerItems(data.layout, $V.Layout);

    $V.ContentGrid.hide();
    $V.PlanSizes.hide();

    $V.ContentBg.removeClass('scada-vis-panel-edit');
    $V.Sidebar.getBottomToolbar().show();
    $V.Sidebar.expand();
    $V.Sidebar.doLayout();
    $V.Content.doLayout();
  },
  registerItems: function(data, container) {
    var i;
    for (i = data.length - 1; i >= 0; i--) {
      this.register(data[ i ], container);
    }
  },
  unlock: function() {
    var widget;
    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        widget = this.widgets[ w ];

        if (widget.container == $V.Content) {
          widget.unlock();
        }
        else {
          widget.disable();
        }
      }
    }

    $V.Tree.disable();
    $V.ContentGrid.show();
    $V.PlanSizes.show();
    $V.Sidebar.getBottomToolbar().hide();
    $V.Sidebar.doLayout();

    $V.ContentBg.addClass('scada-vis-panel-edit');
    $V.Layout.addClass('scada-vis-fade');

    if ($V.planSize && $V.planSize.width && $V.planSize.height) {
      $V.PlanWidthInput.setValue($V.planSize.width);
      $V.PlanHeightInput.setValue($V.planSize.height);
    }
    else {
      $V.PlanWidthInput.setValue(1024);
      $V.PlanHeightInput.setValue(768);
    }

    $V.setPlanGuides();
  },
  lock: function() {
    this.stopEdit();

    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        this.widgets[ w ].lock();
      }
    }

    $V.Tree.enable();
    $V.resetAllForms();
  },
  register: function(config, container, unlock) {
    var t = this.mapping[ config.type.toNumber() ], pos;
    if (!t) {
      return;
    }

    if (unlock) {
      config.locx = $V.getNumericValue($V.WidgetLocX);
      config.locy = $V.getNumericValue($V.WidgetLocY);

      if (this.dupe) {
        config.locx += $V.getNumericValue($V.WidgetDupeX);
        config.locy += $V.getNumericValue($V.WidgetDupeY);
      }
    }

    if (this.edit) {
      if (!this.dupe && t == this.edit.type) {
        var e = this.edit.elem, c = e.config;

        config.locx = c.locx;
        config.locy = c.locy;

        e.markDeleted();
      }

      this.stopEdit();
    }

    var v = new $V[ t + 'Widget' ](config, container);
    if (v.config) {
      this.widgets[ this.widgetId ] = v;
      this.widgetId++;
    }

    if (unlock) {
      v.unlock();
    }

    return v;
  },
  clean: function() {
    for (var w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        if (typeof this.widgets[ w ].dispose == 'function') {
          this.widgets[ w ].dispose();
        }
        delete this.widgets[ w ];
      }
    }
  },
  setBackground: function(contentCfg, layoutCfg) {
    /*
    if (Config.visbgcolor) {
      $V.wrap.body.setStyle('background-color', Config.visbgcolor);
    }
    */

    this.doSetBackground($V.ContentBg.el, contentCfg || {});
    this.doSetBackground($V.ContentBgAdd.el, contentCfg || {}, true);
    this.doSetBackground($V.LayoutBg.el, layoutCfg || {});
    this.doSetBackground($V.LayoutBgAdd.el, layoutCfg || {}, true);
  },
  doSetBackground: function(el, config, add) {
    var bg = config[ add ? 'background_add' : 'background' ], svg = bg && bg.match(/\.svgz?$/);

    bg = bg ? ('url("/scada/resources/img/' + bg + '")') : '';
    el.setStyle('background-image', bg);

    el[ svg ? 'addClass' : 'removeClass' ]('scada-vis-layer-svg');

    if (!add) {
      el.setStyle('background-color', config.bgcolor ? config.bgcolor : 'transparent');
    }

    el.setStyle('background-repeat', !svg && config.bgrepeat ? 'repeat' : 'no-repeat');
  },
  serialize: function() {
    var data = { 'update': [], 'delete': [], 'insert': [] }, w, d, key, widget;

    for (w in this.widgets) {
      if (this.widgets.hasOwnProperty(w)) {
        widget = this.widgets[ w ];

        if (widget && widget.container == $V.Content) {
          d = widget.config;

          if (d && !d.disposed) {
            if (d.deleted) {
              key = 'delete';
            }
            else if (d.insert) {
              key = 'insert';
            }
            else {
              key = 'update';
            }

            data[ key ].push(d);
          }
        }
      }
    }

    data.id = $V.floorId;
    data.size = $V.getPlanSize();
    return data;
  },
  startEdit: function(elem) {
    var type = this.mapping[ elem.config.type.toNumber() ], fn;

    if (!type) {
      return;
    }

    if (this.edit && this.edit.elem == elem) {
      return;
    }

    this.stopEdit();
    $V.EditorPanel.setActiveTab(this.panels[ type ]);

    $V.editorSubmits[ type ].setText($tr('vis_button_edit'));

    var values = elem.getFormData();
    $V.editorForms[ type ].setValues(values);

    this.edit = { type: type, elem: elem };
    elem.form = $V.editorForms[ type ];
    $V.setWidgetLocForm(elem);

    fn = $V.prepareForm[ type ];
    if (fn) {
      fn();
    }

    if (elem.buttonEl) {
      elem.buttonEl.addClass('x-resizable-over').addClass('scada-widget-active');
    }
  },
  stopEdit: function(type) {
    var elem;

    if (!this.edit) {
      return;
    }

    if (type && this.edit.type != type) {
      return;
    }

    elem = this.edit.elem;
    elem.form = false;
    if (elem.buttonEl) {
      elem.buttonEl.removeClass('x-resizable-over').removeClass('scada-widget-active');
    }

    $V.editorSubmits[ this.edit.type ].setText($tr('vis_button_add'));
    $V.editorForms[ this.edit.type ].reset();

    this.edit = null;
    this.dupe = false;

    $V.activeWidget = false;

    $V.prepareAllForms();
    $V.setWidgetLocForm();
  },
  save: function() {
    this.stopEdit();

    var data = this.serialize();
    if (data.size) {
      Ext.Ajax.request({
        url: Scada.reqUrl('buildings', 'floor-plan-save'),
        success: $V.onSaveFloor,
        failure: $V.onSaveFloor,
        params: Scada.encodeData(data)
      });
    }
    else {
      Ext.Msg.alert(
        $tr('vis_plan_size_error_title'),
        $tr('vis_plan_size_error_text')
      );
    }
  }
};

$V.widget = {
  createButton: function(noIcon, clean) {
    var cls = 'scada-widget ' + (noIcon ? 'scada-widget-no-icon' : 'scada-widget-icon'), el, tr, prio;
    if ($V.toBoolean(this.config.nobg)) {
      cls = cls + ' x-scada-widget-nobg';
      clean = true;
    }

    this.buttonEl = new Ext.Button({
      border: false,
      iconAlign: 'top',
      cls: cls + ($V.toBoolean(this.config.readonly) ? ' scada-widget-icon-ro' : ''),
      x: this.config.locx,
      y: this.config.locy
    });

    this.buttonEl.on('click', this.clickHandlerWrap, this);

    this.buttonEl.on('afterrender', function(btn) {
      btn.el.removeClass('x-btn-noicon');
    });

    this.container.add(this.buttonEl);
    this.container.doLayout();

    prio = this.config.displaymode == 'value' ? 1 : 0;
    this.buttonEl.el.setStyle('z-index', $V.widgetPrio[ this.config.type ] * 10 + prio);

    if (clean) {
      el = this.buttonEl.el;
      tr = el.query('tr');
      Ext.get(tr[ 2 ]).remove();
      Ext.get(tr[ 0 ]).remove();
      Ext.get(el.query('.x-btn-ml')).remove();
      Ext.get(el.query('.x-btn-mr')).remove();
    }
  },
  markInvalid: function() {
    if (this.buttonEl) {
      this.buttonEl.addClass('scada-widget-invalid');
    }
  },
  setIcon: function(icon, override, width, height) {
    var el, sized;
    if (this.buttonEl && icon) {
      icon = override ? icon : ('/scada/resources/icons/' + icon);

      if (this.svg) {
        this.buttonEl.btnEl.update('<object type="image/svg+xml"></object>');
        el = this.buttonEl.btnEl.child('object');
        el.dom.setAttribute('data', icon);
      }
      else {
        this.buttonEl.btnEl.update('<img>');
        el = this.buttonEl.btnEl.child('img');
        el.dom.setAttribute('src', icon);
      }

      if (height && height.toNumber()) {
        el.setStyle('height', height + 'px');
        sized = true;
      }

      if (width && width.toNumber()) {
        el.setStyle('width', width + 'px');
        sized = true;
      }

      if (!sized) {
        el.addClass('has-no-size');
      }
    }
  },
  addListener: function(object) {
    $V.objectStore.addListener(this.config.id, (object || this.config.object), this.listener, this);
  },
  removeListener: function() {
    $V.objectStore.removeListener(this.config.id, this.config.object);
  },
  destroy: function() {
    var self = this;

    Ext.each([ 'buttonEl', 'windowEl' ], function(el) {
      if (self[ el ]) {
        self[ el ].destroy();
        delete self[ el ];
      }
    });
  },
  dispose: function() {
    this.destroy();
  },
  unlock: function() {
    if (this.onLock) {
      this.onLock();
    }

    if (this.buttonEl) {
      if (!this.dd) {
        var self = this;

        this.dd = new Ext.dd.DD(this.buttonEl.id, 'scada-vis');
        this.dd.startDrag = function() {
          var fc = $V.floorController;

          if (fc.edit && fc.edit.elem != self) {
            fc.stopEdit();
          }

          this.constrainTo($V.Content.el);

          $V.setWidgetLocForm(self);
          $V.floorController.startEdit(self);
        };
        this.dd.onDrag = function() {
          var aw = $V.activeWidget, pos;
          if (aw) {
            pos = aw.buttonEl.getPosition(true);
            $V.posEl.setPosition(pos);
            $V.posEl.update(pos.join('x'));
          }
        };
        this.dd.endDrag = function() {
          self.dragged = true;
          self.updateLocation();

          $V.setWidgetLocForm(self);
          $V.posEl.setPosition(-9999, -9999);
        };
      }
      else {
        this.dd.unlock();
      }
    }

    this.unlocked = true;
  },
  attachResize: function(params, el) {
    var btn = this.buttonEl.btnEl, resizeEl = btn.child(el || 'img'), rs, self = this;

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.on('resize', function(el, w, h, e) {
      if (e) {
        $V.onResize(self, w, h);
        $V.floorController.startEdit(self);
      }
    });

    self.resizeEl = resizeEl;
    self.params = params;
    self.rs = rs;

    if (resizeEl) {
      resizeEl.on('load', function() {
        var dom = resizeEl.dom, w, h;

        w = (params.width ? params.width.toNumber() : 0) || dom.naturalWidth || dom.width;
        h = (params.height ? params.height.toNumber() : 0) || dom.naturalHeight || dom.height;

        rs.resizeTo(w, h);
      });
    }
  },
  lock: function() {
    if (this.dd) {
      this.dd.lock();
    }

    this.unlocked = false;
  },
  markDeleted: function() {
    this.config.deleted = true;

    if (this.config.insert) {
      this.destroy();
      this.config.disposed = true;
    }
    else if (this.buttonEl) {
      this.buttonEl.hide();
    }
  },
  disable: function() {
    this.buttonEl && this.buttonEl.disable();
    this.windowEl && this.windowEl.hide();
  },
  clickHandlerWrap: function(button, event) {
    if (this.unlocked) {
      if (this.dragged) {
        this.dragged = false;
      }
      else {
        $V.floorController.startEdit(this);
      }
    }
    else if (this.clickHandler) {
      this.clickHandler();
    }

    event.stopPropagation();
  },
  updateLocation: function() {
    if (this.buttonEl) {
      var pos = this.buttonEl.getPosition(true);
      this.config.locx = pos[ 0 ];
      this.config.locy = pos[ 1 ];
    }
  },
  getFormData: function() {
    return this.config;
  },
  applyTextStyles: function(text, params, cls) {
    var btn = this.buttonEl, btnEl = this.valueLabel || btn.btnEl;

    if (text) {
      btn.setText(text);
    }

    if (cls) {
      btn.addClass(cls);
    }

    btnEl.setStyle('font-size', (params.size || 12).toString() + 'px');
    if (params.color) {
      btnEl.setStyle('color', params.color);
    }

    if (params.bold) {
      btnEl.setStyle('font-weight', 'bold');
    }
    if (params.italic) {
      btnEl.setStyle('font-style', 'italic');
    }
    if (params.underline) {
      btnEl.setStyle('text-decoration', 'underline');
    }
    if (params.font) {
      btnEl.setStyle('font-family', params.font);
    }
    if (params.backdrop) {
      btn.addClass('scada-widget-backdrop');
      btnEl.addClass('value');
    }
  }
};

$V.onResize = function(self, w, h) {
  if (self.resizeEl) {
    self.resizeEl.setSize(w, h);
  }

  self.config.width = w;
  self.config.height = h;

  self.params.width = w;
  self.params.height = h;

  $V.WidgetSizeW.setValue(w);
  $V.WidgetSizeH.setValue(h);

  if (self.form) {
    self.form.setValues({ width: w, height: h });
  }

  self.config.params = Ext.encode(self.params);
};

$V.manualResize = function(event) {
  var aw = $V.activeWidget, t = event.xtype || '', dom = aw && aw.resizeEl ? aw.resizeEl.dom : null, w, h, rw, wv, hv;

  if (dom) {
    if (t == 'button') {
      w = dom.naturalWidth;
      h = dom.naturalHeight;
    }
    else {
      if (event.el) {
        rw = event.el.id.indexOf('w') > 0;
      }

      wv = $V.WidgetSizeW.isValid();
      hv = $V.WidgetSizeH.isValid();

      if ($V.resizeAR) {
        if ((rw && !wv) || (!rw && !hv)) {
          return;
        }
      }
      else if (!wv && !hv) {
        return;
      }

      w = $V.WidgetSizeW.getValue().toNumber();
      h = $V.WidgetSizeH.getValue().toNumber();

      if ($V.resizeAR) {
        if (rw) {
          h = Math.round(w / $V.resizeAR);
          $V.WidgetSizeH.clearInvalid();
          $V.WidgetSizeH.setValue(h);
        }
        else {
          w = Math.round(h * $V.resizeAR);
          $V.WidgetSizeW.clearInvalid();
          $V.WidgetSizeW.setValue(w);
        }
      }
    }

    if (w && h) {
      $V.onResize(aw, w, h);
      aw.rs.resizeTo(w, h);
    }
  }
  else if (t == 'button') {
    $V.manualResizeReset();
  }
};

$V.manualResizeSet = function() {
  var aw = $V.activeWidget, el = aw && aw.resizeEl ? aw.resizeEl.dom : null, w, h;

  if (el) {
    w = el.width || el.style.width.toNumber();
    h = el.height || el.style.height.toNumber();
  }

  $V.WidgetSizeW.setValue(w || '');
  $V.WidgetSizeH.setValue(h || '');
};

$V.manualResizeReset = function() {
  $V.WidgetSizeW.setValue('');
  $V.WidgetSizeH.setValue('');
};

$V.manualResizeSetVisible = function(hide) {
  $V.WidgetSizeW.setDisabled(hide);
  $V.WidgetSizeH.setDisabled(hide);
  $V.WidgetSizeR.setVisible(!hide);
  $V.WidgetSizeL.setVisible(!hide);
};

$V.getControlType = function(rawdt, params) {
  var dt = typeof rawdt == 'object' ? rawdt : Scada.normalizeDt(rawdt)
    , control, scale, mdt = dt.major;

  if (Scada.isRgb(dt)) {
    control = false;
  }
  else if (mdt == Scada.dt.bool) {
    control = params.control == 'checkbox' ? 'checkbox' : 'toggler';
  }
  else if (mdt == Scada.dt.bit4) {
    control = 'dimmer';
  }
  else if (Scada.dtIsNumeric(mdt)) {
    scale = mdt == Scada.dt.uint8 && dt.minor == Scada.minorDt.scale;
    control = params.control || (scale ? 'slider' : 'spinner');
  }
  else if (mdt == Scada.dt.char) {
    control = 'charinput';
  }
  else if (mdt == Scada.dt.string) {
    control = 'stringinput';
  }

  if (control == 'inline-slider') {
    control = 'slider';
  }

  if (control == 'slider' && $V.toBoolean(params.vertical)) {
    control += '-v';
  }

  if (control == 'vselect') {
    control = false;
  }

  return control;
}

$V.checkObjectForm = function(config, oid) {
  var err, obj, control;
  if (!config.showcontrol) {
    return true;
  }

  err = config.widget.toNumber() || config.readonly || config.fixedvalue != '';
  obj = $V.objectStore.getObject(oid);

  if (obj) {
    err = err || !$V.getControlType(obj.datatype, obj.params || {});
  }

  if (err) {
    Ext.Msg.alert(
      $tr('error'),
      $tr('vis_showcontrol_error')
    );
  }

  return !err;
}

$V.objectWidget = function(config, container) {
  var o = $V.objectStore.getObject(config.object), s, displaymode, valid, isrgb;

  if (config.statusobject) {
    s = $V.objectStore.getObject(config.statusobject);

    if (config.statusobject != config.object) {
      this.mainobject = o;
    }
  }
  else {
    config.statusobject = config.object;
    s = o;
  }

  valid = o && s;

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;

  if (valid) {
    config.boolean = s.boolean;
    config.mainbool = o.boolean;
  }
  else {
    config.boolean = config.type == 0;
    config.object = '';
    config.statusobject = '';
  }

  this.clickHandler = function() {
    if (!valid) {
      return;
    }
    else if (this.config.widget) {
      Ext.Msg.alert(
        $tr('vis_object_widget'),
        $tr('vis_object_widget_error')
      );
    }
    else if (!$V.toBoolean(this.config.readonly)) {
      var o = $V.objectStore.getObject(this.config.object);
      var s = $V.objectStore.getObject(this.config.statusobject);

      if (!s || !o) {
        return;
      }
      else if (this.config.fixedvalue) {
        Ext.Ajax.request({
          url: Scada.reqUrl('objects', 'setvalue'),
          params: Scada.encodeData({
            address: o.address,
            datatype: o.rawdatatype,
            value: this.config.fixedvalue,
            type: 'text'
          })
        });
      }
      else if (this.config.mainbool) {
        Ext.Ajax.request({
          url: Scada.reqUrl('objects', 'setvalue'),
          params: Scada.encodeData({
            address: o.address,
            datatype: 1,
            value: (s.value ? 'false' : 'true')
          })
        });
      }
      else {
        o.statobj = s;
        Scada.showSetValueForm(o);
      }
    }
  };

  var params = $V.tryDecode(config.params), hideIcon;
  $extend(config, params);

  this.getFormData = function() {
    var c = $extend({}, this.config);

    if (c.object == c.statusobject) {
      c.statusobject = 0;
    }

    if (c.icons_add) {
      c.icons_add = Ext.encode(c.icons_add);
    }

    if (!c.widget) {
      c.widget = '';
    }

    return c;
  };

  if (valid && params.showcontrol) {
    var p = o.params || {}, control = $V.getControlType(o.datatype, p), size;
    if (control) {
      if (control == 'circularslider') {
        size = p.size || 200;
      }

      config.mainbool = false;
      this.createButton();
      this.setIcon('/scada/resources/images/controls/' + control + '.png', true, size, size);
      return this;
    }
  }

  this.onLock = function() {
    if (this.config.displaymode != 'value') {
      this.attachResize(params);
    }
  };

  displaymode = this.config.displaymode;
  hideIcon = displaymode == 'value';
  isrgb = valid ? Scada.isRgb(o.datatype) : false;

  this.createButton(hideIcon);

  if (displaymode == 'icon-value') {
    this.valueLabel = this.buttonEl.btnEl.insertSibling({ tag: 'div' }, 'after');
  }
  else if (displaymode == 'value') {
    this.valueLabel = this.buttonEl.btnEl;
  }

  if (displaymode != 'value' && params.width && params.height) {
    this.buttonEl.btnEl.setSize(params.width, params.height);
  }

  if (displaymode != 'icon') {
    if (isrgb) {
      this.valueLabel.addClass('scada-item-color');
    }
    else {
      this.applyTextStyles(null, params);
    }
  }

  this.updateIcon = function(value, dt) {
    var icon, i, j, add;

    if (dt == Scada.dt.bool) {
      icon = value ? this.config.icon_on : this.config.icon_off;
    }
    else {
      icon = this.config.icon_default;

      if (this.config.icons_add) {
        for (i = 0, j = this.config.icons_add.length; i < j; i++) {
          add = this.config.icons_add[ i ];

          if (add.min <= value && value <= add.max) {
            icon = add.icon;
            break;
          }
        }
      }
    }

    this.setIcon(icon);
  };

  this.listener = function(object, type) {
    if (type == 'init' || type == 'value') {
      if (this.config.displaymode != 'value') {
        this.updateIcon(object.value, object.datatype.major);
      }

      if (this.valueLabel) {
        if (isrgb) {
          this.valueLabel.setStyle('background-color', Scada.getRgbHex(object.value));
        }
        else {
          var v = Scada.decodeValue(object.value, object.rawdatatype, object.units, object.params, object.enums);
          this.valueLabel.update(v.toString());
        }
      }
    }
  };

  this.addListener(config.statusobject);

  this.dispose = function() {
    this.removeListener();
    this.destroy();
  };

  if (!valid) {
    this.markInvalid();
    this.updateIcon(0, config.type == 0 ? Scada.dt.bool : Scada.dt.int8);

    if (this.valueLabel) {
      this.valueLabel.update('???');
    }
  }

  return this;
};

$V.getPrevNextFloor = function(curr, next) {
  var ids = $V.sortedFloorIds, index = ids.indexOf(curr), max = ids.length - 1;

  if (index < 0) {
    return index;
  }

  if (next) {
    index = index == max ? 0 : (index + 1);
  }
  else {
    index = index == 0 ? max : (index - 1);
  }

  return ids[ index ];
};

$V.floorlinkWidget = function(config, container) {
  var text, params, valid = true, link = config.object;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  params = $V.tryDecode(this.config.params, { icon: this.config.params });
  $extend(this.config, params);

  if (link == 'prev' || link == 'next') {
    link = $V.getPrevNextFloor(config.floor, link == 'next');
  }

  if (link == 'trends') {
    this.url = '/cgi-bin/scada-vis/trends.cgi';
    text = $tr('trends_title');
  }
  else if (link == 'schedulers') {
    this.url = '/cgi-bin/scada-vis/schedulers.cgi';
    text = $tr('schedulers_title');
  }
  else if (link == 'url') {
    this.url = params.url;
    text = $tr('vis_link');
  }
  else if ($V.floors[ link ]) {
    this.floorObject = $V.floors[ link ];
    text = this.floorObject.text;
  }
  else {
    valid = false;
    config.object = '';
  }

  this.clickHandler = function() {
    if (!valid) {
      return;
    }
    else if (this.url) {
      window.open(this.url);
    }
    else {
      $V.showFloor(this.floorObject);
    }
  };

  this.createButton();
  if (!valid) {
    this.markInvalid();
  }

  this.onLock = function() {
    if (this.config.icon) {
      this.attachResize(params);
    }
  };

  var icon = params.icon, title = this.config.name || $tr('vis_goto', text);

  if (icon) {
    this.config.icon = icon;

    if (params.width && params.height) {
      this.buttonEl.btnEl.setSize(params.width, params.height);
    }

    if (params.icon_active && this.floorObject && this.floorObject.floorid == $V.floorId) {
      icon = params.icon_active;
    }
    this.setIcon(icon);
  }
  else {
    this.applyTextStyles(title, params, 'x-scada-text-label');
  }

  this.getFormData = function() {
    var c = $clone(this.config);
    c.floorid = c.object;

    if (!c.displaymode) {
      c.displaymode = c.icon ? 'icon' : 'value';
    }

    return c;
  };

  return this;
};

$V.getModalConfig = function(config, params) {
  config.autoopen = !!params.autoopen;
  config.icon = params.icon;

  config.win_width = params.win_width ? params.win_width.toNumber() : 160;
  config.win_height = params.win_height ? params.win_height.toNumber() : 120;

  config.width = params.width ? params.width.toNumber() : 0;
  config.height = params.height ? params.height.toNumber() : 0;
};

$V.cameraWidget = function(config, container) {
  var params = $V.tryDecode(config.params, null);

  if (!params) {
    return;
  }

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;

  config.src = params.src;
  $V.getModalConfig(config, params);

  this.setImage = function() {
    var show = this.windowEl.isVisible(), src = show ? config.src : '/scada/ext/images/s.gif';
    Ext.DomHelper.overwrite(this.windowEl.body, {
      tag: 'img',
      width: config.win_width,
      height: config.win_height,
      src: src
    });
  };

  this.clickHandler = function() {
    var width = 0;
    if (this.windowEl) {
      this.windowEl.setVisible(!this.windowEl.isVisible());
    }
    else {
      if (!config.autoopen) {
        width = this.buttonEl.getSize().width || 0;
      }

      this.windowEl = new Ext.Window(
        $def.window({
          modal: false,
          draggable: true,
          title: $tr('vis_camera'),
          x: this.config.locx.toNumber() + width,
          y: this.config.locy,
          width: config.win_width + 12,
          height: config.win_height + 30,
          bodyBorder: false,
          border: false
        })
      );

      this.container.add(this.windowEl);
      this.container.doLayout();

      this.windowEl.on('show', this.setImage, this, true);
      this.windowEl.on('hide', this.setImage, this, false);

      this.windowEl.show();
    }
  };

  this.createButton();
  this.setIcon(this.config.icon || 'camera.png');
  if (config.width && config.height) {
    this.buttonEl.btnEl.setSize(config.width, config.height);
  }

  this.onLock = function() {
    if (this.windowEl) {
      this.windowEl.hide();
    }

    this.attachResize(params);
  };

  if (this.config.autoopen && !$V.unlocked) {
    this.clickHandler();
  }

  return this;
};

$V.xValueFormatter = function(time) {
  var zpad = Dygraph.zeropad, d = new Date(time), ret = '', frac;

  frac = d.getHours() * 3600 + d.getMinutes() * 60 + d.getSeconds();
  if (frac) {
    ret = ' ';

    if (d.getSeconds()) {
      ret += zpad(d.getHours()) + ':' + zpad(d.getMinutes()) + ':' + zpad(d.getSeconds());
    }
    else {
      ret += zpad(d.getHours()) + ':' + zpad(d.getMinutes());
    }
  }

  return ' ' + d.getFullYear() + '.' + zpad(d.getMonth() + 1) + '.' + zpad(d.getDate()) + ret;
};

$V.graphWidget = function(config, container) {
  var params = $V.tryDecode(config.params, null), valid;

  if (!params) {
    return;
  }

  $extend(this, $V.widget);

  this.container = container;
  this.config = config;

  $V.getModalConfig(config, params);
  config.limit = params.limit ? params.limit.toNumber() : 10;

  this.object = $V.objectStore.getObject(config.object);
  valid = this.object && Scada.dtIsNumeric(this.object.datatype.major);

  this.updateGraph = function(res) {
    if (!this.windowEl || !this.windowEl.isVisible()) {
      return;
    }

    var data = [];
    try {
      res = Ext.decode(res.responseText);
      Ext.each(res.data, function(item) {
        data.unshift([
          new Date(item.logtime * 1000),
          item.datadec
        ]);
      });

      if (this.graph) {
        this.graph.updateOptions({ file: data });
      }
      else {
        this.windowEl.body.update('<div class="scada-graph-inner"></div>');
        this.graph = new Dygraph(this.windowEl.body.child('div').dom, data, {
          labels: [ '', this.name ],
          xAxisLabelWidth: 60,
          includeZero: true,
          axes: {
            x: {
              pixelsPerLabel: 65,
              valueFormatter: $V.xValueFormatter
            }
          }
        });
      }
    }
    catch (e) {}
  };

  this.clickHandler = function() {
    var width = 0;

    if (!valid) {
      return;
    }
    else if (this.windowEl) {
      this.windowEl.setVisible(!this.windowEl.isVisible());
    }
    else {
      if (!config.autoopen) {
        width = this.buttonEl.getSize().width || 0;
      }

      this.windowEl = new Ext.Window(
        $def.window({
          modal: false,
          draggable: true,
          title: this.object.name,
          x: this.config.locx.toNumber() + width,
          y: this.config.locy,
          width: this.config.win_width + 12,
          height: this.config.win_height + 30,
          bodyBorder: false,
          border: false
        })
      );

      this.container.add(this.windowEl);
      this.container.doLayout();

      this.windowEl.show();
      this.windowEl.addClass('scada-graph');
      this.reload();
    }
  };

  this.createButton();
  this.setIcon(this.config.icon);
  if (config.width && config.height) {
    this.buttonEl.btnEl.setSize(config.width, config.height);
  }

  this.name = config.name || (valid ? this.object.name : '') || $tr('vis_widget_graph');
  if (!valid) {
    this.markInvalid();
    config.object = '';
  }

  this.reload = function() {
    Ext.Ajax.request({
      url: Scada.reqUrl('vis', 'graph'),
      method: 'POST',
      params: Scada.encodeData({
        address: this.object.rawaddress,
        limit: this.config.limit
      }),
      success: Ext.createDelegate(this.updateGraph, this)
    });
  };

  this.listener = function(object, type) {
    if (type == 'value' && this.windowEl && this.windowEl.isVisible()) {
      this.reload();
    }

    if (type == 'init' && this.config.autoopen && !$V.unlocked) {
      this.clickHandler();
    }
  };

  this.onLock = function() {
    if (this.windowEl) {
      this.windowEl.hide();
    }

    this.attachResize(params);
  };

  if (valid) {
    this.addListener(this.config.object);
  }

  return this;
};

$V.labelWidget = function(config, container) {
  var params = $V.tryDecode(config.params);
  $extend(this, $V.widget);

  this.container = container;
  this.config = config;
  $extend(this.config, params);

  this.createButton();
  this.applyTextStyles(this.config.name, params, 'x-scada-widget-label');

  return this;
};

$V.imageWidget = function(config, container) {
  var source, src, params;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  params = $V.tryDecode(this.config.params);
  $extend(this.config, params);

  this.clickHandler = function() {
    if (this.config.link) {
      window.location = this.config.link;
    }
  };

  this.onLock = function() {
    this.attachResize(params, this.svg ? 'object' : 'img');
  };

  src = params.src;
  source = params.source == 'local' ? 'local' : 'remote';
  this.config[ 'src_' + source ] = src;

  if (source == 'local') {
    src = '/scada/resources/img/' + src;

    this.svg = !!src.match(/\.svgz?$/);
  }

  this.config.nobg = 1;
  this.createButton(false, true);
  this.setIcon(src, true, params.width, params.height);

  if (!params.link) {
    this.buttonEl.addClass('scada-widget-image-no-link');
  }

  this.buttonEl.addClass('scada-widget-image');

  return this;
};

$V.gaugeWidget = function(config, container) {
  var params = $V.tryDecode(config.params, null), id, dtm, decimals, bounds, units, valid;
  if (!params || !params.size) {
    return;
  }

  $extend(this, $V.widget);

  this.object = $V.objectStore.getObject(config.object);
  valid = this.object && Scada.dtIsNumeric(this.object.datatype.major);

  this.container = container;
  this.config = config;
  $extend(this.config, params);

  this.clickHandler = function() {
    if (valid && !$V.toBoolean(this.config.readonly)) {
      var o = $V.objectStore.getObject(this.config.object);
      Scada.showSetValueForm(o);
    }
  };

  this.config.nobg = 1;
  this.createButton();

  if (!valid) {
    this.markInvalid();
    config.object = '';
  }

  id = Ext.id();
  if (valid) {
    dtm = this.object.datatype.major;
    decimals = (dtm == Scada.dt.float16 || dtm == Scada.dt.float32) ? 2 : 0;
    bounds = Scada.dtGetBounds(this.object.rawdatatype, this.object.params);
    units = this.config.name || this.object.name || this.object.address;
  }
  else {
    decimals = 0;
    bounds = [ 0, 1 ];
    units = this.config.name || '???';
  }

  this.buttonEl.btnEl.update('<canvas id="' + id + '"></canvas>');

  this.gauge = new Gauge({
    minValue: bounds[ 0 ],
    maxValue: bounds[ 1 ],
    decimals: decimals,
    width: params.size,
    height: params.size,
    units: units,
    renderTo: id
  });

  this.gauge.draw();
  this.gauge.setValue(valid ? this.object.value : 0);

  this.listener = function(object, type) {
    if (type == 'value' && this.gauge) {
      this.gauge.setValue(object.value);
    }
  };

  if (valid) {
    this.addListener(this.config.object);
  }

  return this;
};

$V.iframeWidget = function(config, container) {
  var src, frame, params;

  $extend(this, $V.widget);
  this.container = container;
  this.config = config;

  params = $V.tryDecode(this.config.params);
  $extend(this.config, params);

  this.onLock = function() {
    var btn = this.buttonEl.btnEl, frame, rs, self = this;
    btn.update('<div class="scada-widget-iframe-inner"></div>');

    frame = this.buttonEl.btnEl.child('div');
    frame.setStyle('width', params.width + 'px');
    frame.setStyle('height', params.height + 'px');

    rs = new Ext.Resizable(btn, { wrap: true });
    rs.window = true;
    rs.on('resize', function(el, w, h) {
      frame.setStyle('width', w + 'px');
      frame.setStyle('height', h + 'px');

      $V.onResize(self, w, h);
    });

    self.params = params;
    self.resizeEl = frame;
    self.rs = rs;
  };

  if (params.source == 'trends') {
    src = '/cgi-bin/scada-vis/trends.cgi';
  }
  else if (params.source == 'schedulers') {
    src = '/cgi-bin/scada-vis/schedulers.cgi';
  }
  else {
    src = params.url;
  }

  this.config.nobg = 1;
  this.createButton(false, true);
  this.buttonEl.btnEl.update('<iframe frameborder="0" border="0">');
  frame = this.buttonEl.btnEl.child('iframe');
  frame.set({
    src: src,
    width: params.width,
    height: params.height
  });

  this.buttonEl.addClass('scada-widget-iframe');
  return this;
};


$V.floors = {};

$V.onSaveFloor = function() {
  $V.EditorControl.setText($tr('vis_unlock'));
  $V.EditorControl.enable();
  $V.EditorCancel.disable();

  $V.showFloor($V.floor, true);
};

$V.onShowFloor = function() {
  $V.Tree.enable();
  $V.Content.enable();
  $V.Editor.enable();
  $V.EditorControl.enable();
  $V.EditorCancel.disable();

  $V.hideFloatingPanel();
};

$V.setPlanSize = function(panel, width, height) {
  panel.el.setStyle('width', (width || 1024) + 'px');
  panel.el.setStyle('height', (height || 768) + 'px');
};

$V.repositionPlans = function(width, height) {
  if (!$V.center) {
    return;
  }

  var pWidth = width || $V.planSize.width || 1024,
      pHeight = height || $V.planSize.height || 768,
      lWidth = $V.layoutCfg ? $V.layoutSize.width || 1024 : 0,
      lHeight = $V.layoutCfg ? $V.layoutSize.height || 768 : 0,
      wSize = $V.wrap.getSize(),
      mWidth = Math.max(pWidth, lWidth, wSize.width),
      mHeight = Math.max(pHeight, lHeight, wSize.height),
      wDelta = Math.round((mWidth - pWidth) / 2),
      hDelta = Math.round((mHeight - pHeight) / 2),
      top = $V.centerWidth ? 0 : Math.max(hDelta, 0),
      left = Math.max(wDelta, 0);

  $V.Content.setPosition(left, top);
  $V.ContentBg.setPosition(left, top);
  $V.ContentBgAdd.setPosition(left, top);
  $V.ContentGrid.setPosition(left, top);

  if (!$V.layoutCfg) {
    return;
  }

  wDelta = Math.round((mWidth - lWidth) / 2);
  hDelta = Math.round((mHeight - lHeight) / 2);

  top = $V.centerWidth ? 0 : Math.max(hDelta, 0);
  left = Math.max(wDelta, 0);

  $V.Layout.setPosition(left, top);
  $V.LayoutBg.setPosition(left, top);
  $V.LayoutBgAdd.setPosition(left, top);
};

$V.doShowFloor = function(result) {
  var data = Ext.decode(result.responseText);

  $V.onShowFloor();

  $V.floorController.load(data);
  $V.planSize = data.size || {};
  $V.layoutSize = data.layoutsize || { width: 10, height: 10 };

  $V.setPlanSize($V.Content, $V.planSize.width, $V.planSize.height);
  $V.setPlanSize($V.ContentBg, $V.planSize.width, $V.planSize.height);
  $V.setPlanSize($V.ContentBgAdd, $V.planSize.width, $V.planSize.height);
  $V.setPlanSize($V.ContentGrid, $V.planSize.width, $V.planSize.height);

  $V.setPlanSize($V.Layout, $V.layoutSize.width, $V.layoutSize.height);
  $V.setPlanSize($V.LayoutBg, $V.layoutSize.width, $V.layoutSize.height);
  $V.setPlanSize($V.LayoutBgAdd, $V.layoutSize.width, $V.layoutSize.height);

  $V.repositionPlans();
};

$V.showFloor = function(floor, forced) {
  var layout, node;

  if (floor && floor.floorid && ($V.floorId != floor.floorid || forced)) {
    $V.floorId = floor.floorid;
    $V.floor = floor;

    node = $V.Tree.getNodeById(floor.id);
    if (node) {
      node.select();
    }

    $V.layoutCfg = false;
    layout = floor.layout.toNumber();

    if (layout) {
      node = $V.Tree.root.findChild('floorid', layout, true);
      if (node) {
        $V.layoutCfg = node.attributes;
      }
    }

    $V.floorController.clean();
    $V.Tree.disable();
    $V.Content.disable();
    $V.Editor.disable();

    Ext.Ajax.request({
      url: Scada.reqUrl('vis', 'floor'),
      success: $V.doShowFloor,
      failure: $V.onShowFloor,
      params: Scada.encodeData({ id: $V.floor.floorid })
    });
  }
};

$V.sortFloor = function(a, b) {
  return parseInt(a[ 2 ], 10) || 0 - parseInt(b[ 2 ], 10) || 0;
};

$V.hideFloatingPanel = function() {
  $V.floatingPanel.hide();
};

$V.showFloatingPanel = function() {
  $V.floatingPanel.show();
};

$V.posEl = new Ext.BoxComponent({ cls: 'scada-widget-position' });

$V.resetAllForms = function() {
  Ext.iterate($V.editorForms, function(type) {
    $V.editorForms[ type ].reset();
  });

  $V.setLockButtonState(false);
  $V.resizeAR = 0;
};

$V.onFocus = function() {
  $V.unlocked = false;

  $V.EditorPanel.disable();
  $V.WidgetCommonProps.disable();
  $V.EditorControl.disable();
  $V.EditorControl.setText($tr('vis_unlock'));
  $V.EditorCancel.disable();
  $V.Layout.removeClass('scada-vis-fade');
  $V.Sidebar.expand();

  $V.resetAllForms();
  $V.prepareAllForms();
  $V.Tree.root.reload();

  $V.WidgetCommonProps.doLayout();
  $V.WidgetCommonProps.disable();

  $V.Sidebar.doLayout();

  Scada.setLocked(false);
};

$V.isChecked = function(v) {
  return v == 'on' || v == 1;
};

$V.checkForms = function(type) {
  var form = $V.editorForms[ type ], sform = $V.WidgetCommonProps.form;
  return form.isValid() && sform.isValid();
};

$V.getVisParams = function(values, extend) {
  var result = {
    size: (values.size ? values.size.toNumber() : ''),
    color: values.color,
    font: values.font,
    bold: $V.isChecked(values.bold) ? 1 : 0,
    italic: $V.isChecked(values.italic) ? 1 : 0,
    underline: $V.isChecked(values.underline) ? 1 : 0
  }, w, h;

  if (typeof values.width !== 'undefined') {
    w = values.width.toNumber();
    h = values.height.toNumber();

    result.width = w > 0 ? w : $V.WidgetSizeW.getValue().toNumber();
    result.height = h > 0 ? h : $V.WidgetSizeH.getValue().toNumber();
  }

  $extend(result, extend || {});
  return Ext.encode(result);
};

$V.onBlur = function() {
  $V.unlocked = false;
};

$V.remove = function(store, record) {
  var id = record.id;

  if ($V.objectStore.objects[ id ]) {
    delete $V.objectStore.objects[ id ];
  }

  $V.reload(store);
};

$V.clear = function(store) {
  $V.objectStore.objects = {};
  $V.reload(store);
};

$V.reload = function(store) {
  $V.objectStore.update(store.data.items);
};

$V.setPlanGuides = function() {
  var width, height;

  if (!$V.PlanWidthInput.isValid() || !$V.PlanHeightInput.isValid()) {
    return;
  }

  width = $V.PlanWidthInput.getValue();
  height = $V.PlanHeightInput.getValue();

  $V.setPlanSize($V.Content, width, height);
  $V.setPlanSize($V.ContentBg, width, height);
  $V.setPlanSize($V.ContentBgAdd, width, height);
  $V.setPlanSize($V.ContentGrid, width, height);

  $V.repositionPlans(width, height);
};

$V.getPlanSize = function() {
  if (!$V.PlanWidthInput.isValid() || !$V.PlanHeightInput.isValid()) {
    return;
  }

  return {
    width: $V.PlanWidthInput.getValue(),
    height: $V.PlanHeightInput.getValue()
  };
};

$V.updateWidgetLocation = function() {
  var el;

  if (!$V.WidgetLocX.isValid() || !$V.WidgetLocY.isValid() || !$V.activeWidget) {
    return;
  }

  el = $V.activeWidget.buttonEl.el;
  el.setStyle('left', $V.WidgetLocX.getValue() + 'px');
  el.setStyle('top', $V.WidgetLocY.getValue() + 'px');
  $V.activeWidget.updateLocation();
};

$V.setLockButtonState = function(state) {
  var btn = $V.WidgetSizeL;
  if (state !== true) {
    btn.toggle(state);
  }
  btn.setIconClass(btn.pressed ? 'icon-lock' : 'icon-unlock');
}

$V.setWidgetLocForm = function(widget) {
  var pos, x, y;

  $V.Sidebar.doLayout();
  $V.activeWidget = widget;

  $V.setLockButtonState(false);
  $V.resizeAR = 0;

  if (widget) {
    pos = widget.buttonEl.getPosition(true);

    x = pos[ 0 ];
    y = pos[ 1 ];
  }
  else {
    x = 10;
    y = 10;
  }

  $V.WidgetLocX.setValue(x);
  $V.WidgetLocY.setValue(y);

  $V.floatingPanel.setVisible(!!widget);

  $V.manualResizeSet();
};

$V.deleteActiveWidget = function() {
  if ($V.activeWidget) {
    $V.activeWidget.markDeleted();
    $V.floorController.stopEdit();
  }
};

$V.dupeActiveWidget = function() {
  var fc = $V.floorController, edit = fc.edit, el;

  if (edit && edit.type) {
    fc.dupe = true;
    $V.editorSubmits[ edit.type ].fireEvent('click');

    setTimeout(function() {
      el = fc.widgets[ fc.widgetId - 1 ];
      if (el && el.buttonEl) {
        el.buttonEl.el.dom.click();
      }
    }, 10);
  }
};

$V.keyEvent = function(e, x, y) {
  var tag = e.target.tagName.toLowerCase(), move = x || y, el;

  if ($V.activeWidget && tag != 'input') {
    if (move) {
      el = x ? $V.WidgetLocX : $V.WidgetLocY;

      if (e.shiftKey) {
        x *= 10;
        y *= 10;
      }

      el.setValue(el.getValue() + x + y);
      $V.updateWidgetLocation();
    }
    else {
      $V.deleteActiveWidget();
    }

    e.preventDefault();
    e.stopEvent();
  }

  return true;
};

$V.init = function() {
  var cls = $V.center ? 'scada-vis-panel-center' : 'scada-vis-panel';

  $V.wrap = Ext.getCmp('scada-vis-wrap');
  $V.Tree = Ext.getCmp('scada-vis-tree');

  $V.wrap.on('afterrender', function() {
    $V.wrap.el.on('click', function(event, target) {
      var p = Ext.fly(target).findParent('#scada-vis-floating-panel');
      if (!p) {
        $V.floorController.stopEdit();
      }
    });

    $V.floatingPanel = new Ext.Toolbar({
      id: 'scada-vis-floating-panel',
      hidden: true,
      cls: 'toolbar-fixup',
      renderTo: 'scada-vis-wrap',
      items: [
        { id: 'scada-vis-delete-button', xtype: 'button', text: $tr('delete'), cls: 'x-btn-warn', width: 75, handler: $V.deleteActiveWidget }, '-',
        { id: 'scada-vis-dupe-x', xtype: 'spinnerfield', minValue: -1000, maxValue: 1000, width: 60, value: 10, allowDecimals: false },
        { id: 'scada-vis-dupe-y', xtype: 'spinnerfield', minValue: -1000, maxValue: 1000, width: 60, value: 10, allowDecimals: false },
        { id: 'scada-vis-dupe-button', xtype: 'button', text: $tr('vis_button_dupe'), cls: 'x-btn-hl', width: 75, handler: $V.dupeActiveWidget }
      ]
    });

    $V.WidgetDupeX = Ext.getCmp('scada-vis-dupe-x');
    $V.WidgetDupeY = Ext.getCmp('scada-vis-dupe-y');
  });

  $V.Content = new Ext.Container({ id: 'scada-vis-content', cls: cls });
  $V.Layout = new Ext.Container({ id: 'scada-vis-layout', cls: cls });
  $V.ContentBg = new Ext.Container({ id: 'scada-vis-content-bg', cls: cls });
  $V.ContentBgAdd = new Ext.Container({ id: 'scada-vis-content-bg-add', cls: cls });
  $V.LayoutBg = new Ext.Container({ id: 'scada-vis-layout-bg', cls: cls });
  $V.LayoutBgAdd = new Ext.Container({ id: 'scada-vis-layout-bg-add', cls: cls });

  $V.ContentGrid = new Ext.Container({ id: 'scada-vis-grid', cls: 'x-hide-display' });
  $V.Sidebar = Ext.getCmp('scada-vis-sidebar');

  $V.Sidebar.on('expand', function() {
    $V.Sidebar.doLayout();
    setTimeout($V.repositionPlans, 100);
  });

  $V.Sidebar.on('collapse', function() {
    setTimeout($V.repositionPlans, 100);
  });

  $V.PlanWidthInput = Ext.getCmp('scada-vis-plan-width');
  $V.PlanHeightInput = Ext.getCmp('scada-vis-plan-height');

  $V.PlanWidthInput.on('spin', $V.setPlanGuides);
  $V.PlanWidthInput.on('keyup', $V.setPlanGuides);
  $V.PlanHeightInput.on('spin', $V.setPlanGuides);
  $V.PlanHeightInput.on('keyup', $V.setPlanGuides);

  Ext.getCmp('scada-vis-plan-menu').on('itemclick', function(item) {
    var size = item.value;

    $V.PlanWidthInput.setValue(size[ 0 ]);
    $V.PlanHeightInput.setValue(size[ 1 ]);

    $V.setPlanGuides();
  });

  $V.WidgetLocX = Ext.getCmp('scada-vis-loc-x');
  $V.WidgetLocY = Ext.getCmp('scada-vis-loc-y');

  $V.WidgetLocX.on('spin', $V.updateWidgetLocation);
  $V.WidgetLocX.on('keyup', $V.updateWidgetLocation);
  $V.WidgetLocY.on('spin', $V.updateWidgetLocation);
  $V.WidgetLocY.on('keyup', $V.updateWidgetLocation);

  $V.WidgetSize = Ext.getCmp('scada-vis-size');
  $V.WidgetSizeW = Ext.getCmp('scada-vis-size-w');
  $V.WidgetSizeH = Ext.getCmp('scada-vis-size-h');
  $V.WidgetSizeR = Ext.getCmp('scada-vis-size-reset')
  $V.WidgetSizeL = Ext.getCmp('scada-vis-size-lock-ar')

  $V.WidgetSizeW.on('spin', $V.manualResize);
  $V.WidgetSizeW.on('keyup', $V.manualResize);
  $V.WidgetSizeH.on('spin', $V.manualResize);
  $V.WidgetSizeH.on('keyup', $V.manualResize);

  $V.WidgetSizeR.on('click', $V.manualResize);
  $V.WidgetSizeL.on('click', function(btn) {
    var v, w, h;

    if (btn.pressed) {
      v = $V.WidgetSizeW.isValid() && $V.WidgetSizeH.isValid();
      w = $V.WidgetSizeW.getValue().toNumber();
      h = $V.WidgetSizeH.getValue().toNumber();

      if (v && w && h) {
        $V.resizeAR = w / h;
      }
      else {
        btn.toggle();
      }
    }
    else {
      $V.resizeAR = 0;
    }

    $V.setLockButtonState(true);
  });

  $V.PlanSizes = Ext.getCmp('scada-vis-plan-size');
  $V.WidgetCommonProps = Ext.getCmp('scada-vis-widget-common-props');

  $V.wrap.add($V.LayoutBg);
  $V.wrap.add($V.LayoutBgAdd);
  $V.wrap.add($V.ContentBg);
  $V.wrap.add($V.ContentBgAdd);
  $V.wrap.add($V.Layout);
  $V.wrap.add($V.Content);
  $V.wrap.add($V.ContentGrid);

  $V.Content.add($V.posEl);

  $V.comboBoxes = {
    object: Ext.getCmp('scada-vis-editor-object'),
    objectwidget: Ext.getCmp('scada-vis-editor-object-widget'),
    statusobject: Ext.getCmp('scada-vis-editor-statusobject'),
    floor: Ext.getCmp('scada-vis-editor-floor'),
    graphobject: Ext.getCmp('scada-vis-editor-graph-object'),
    gauge: Ext.getCmp('scada-vis-editor-gauge-object')
  };

  $V.Editor = Ext.getCmp('scada-vis-editor');
  $V.EditorPanel = Ext.getCmp('scada-vis-editor-panel');
  $V.EditorControl = Ext.getCmp('scada-vis-editor-control');
  $V.EditorCancel = Ext.getCmp('scada-vis-editor-cancel');

  $V.EditorPanel.disable();

  $V.EditorControl.on('click', function() {
    $V.EditorPanel.setDisabled($V.unlocked);
    $V.WidgetCommonProps.setDisabled($V.unlocked);
    $V.resetAllForms();

    if ($V.unlocked) {
      $V.unlocked = false;
      $V.floorController.save();
      $V.Layout.removeClass('scada-vis-fade');
    }
    else {
      $V.unlocked = true;
      $V.EditorControl.setText($tr('vis_save_and_lock'));
      $V.EditorCancel.enable();
      $V.floorController.unlock();
      $V.Sidebar.collapse();
    }

    Scada.setLocked($V.unlocked);
  });

  $V.EditorCancel.on('click', $V.onFocus);

  $V.Tree.on('click', function(node) {
    $V.showFloor(node.attributes);
  });

  $V.Tree.loader.on('load', function(tree, rootNode) {
    $V.floors = {};
    var floorData = [], widgets = [];

    function addFloor(node) {
      var attr = node.attributes;
      if (attr && attr.floorid) {
        $V.floors[ attr.floorid ] = attr;

        if (attr.type == 'widget') {
          widgets.push([ attr.floorid, attr.text, attr.sortorder ]);
        }
        else if (!attr.type) {
          floorData.push([ attr.floorid, attr.text, attr.sortorder ]);
        }
      }

      node.eachChild(addFloor);
    }

    addFloor(rootNode);

    floorData.sort($V.sortFloor);

    $V.sortedFloorIds = []
    Ext.each(floorData, function(item) {
      $V.sortedFloorIds.push(item[ 0 ]);
    });

    floorData.unshift([ 'prev', '• ' + $tr('vis_link_prev') ]);
    floorData.unshift([ 'next', '• ' + $tr('vis_link_next') ]);
    floorData.unshift([ 'url', '• ' + $tr('vis_link') ]);
    floorData.unshift([ 'schedulers', '• ' + $tr('schedulers_title') ]);
    floorData.unshift([ 'trends', '• ' + $tr('trends_title') ]);
    $V.comboBoxes.floor.store.loadData(floorData);
    $V.comboBoxes.floor.reset();

    widgets.sort($V.sortFloor);
    widgets.unshift([ '', $tr('vis_object_widget_none') ]);
    $V.comboBoxes.objectwidget.store.loadData(widgets);
    $V.comboBoxes.objectwidget.reset();

    if ($V.floor) {
      $V.floor = $V.floors[ $V.floor.floorid ];

      if ($V.floor) {
        $V.showFloor($V.floor, true);
      }
      else {
        $V.floorId = 0;
        $V.floorController.clean();
        $V.floorController.setBackground();
      }
    }
  });

  $O.store.on('load', $V.reload);
  $O.store.on('remove', $V.remove);
  $O.store.on('clear', $V.clear);

  $O.extObjStore.on('load', function() {
    var data = [ [ 0, $tr('vis_statusobject_no') ] ];

    Ext.each($O.extObjStore.getRange(), function(item) {
      data.push([ item.data.id, item.data.text ]);
    });

    $V.comboBoxes.statusobject.store.loadData(data);
    $V.comboBoxes.statusobject.reset();
  });

  $V.iconStore.on('load', function(store) {
    var data = [];
    data.push(['', $tr('vis_icon_empty')]);

    Ext.each(store.reader.jsonData.images, function(item) {
      data.push([ item.icon, item.icon ]);
    });

    $V.visIconStore.loadData(data);

    data.shift();
    $V.graphIconStore.loadData(data);
  });

  $V.iconStore.load();
  $V.imgStore.load();

  $V.editorForms = {};
  $V.editorSubmits = {};
  $V.editorResets = {};

  var prefix = 'scada-vis-editor-';
  Ext.each($V.editorTypes, function(type) {
    var p = prefix + type + '-';
    $V.editorForms[ type ] = Ext.getCmp(p + 'panel').getForm();
    $V.editorSubmits[ type ] = Ext.getCmp(p + 'add');
    $V.editorResets[ type ] = Ext.getCmp(p + 'reset');
  });

  $V.editorSubmits.object.on('click', function() {
    if (!$V.checkForms('object')) {
      return;
    }

    var values = $V.editorForms.object.getValues(),
        oid = values.object.toNumber(),
        sid = values.statusobject ? values.statusobject.toNumber() : null,
        object = $V.objectStore.getObject(sid || oid), config, icons,
        widget = values.widget.toNumber();

    try {
      icons = Ext.decode(values.icons_add);
    }
    catch (e) {}

    config = {
      floor: $V.floorId,
      object: oid,
      statusobject: sid,
      type: object.boolean ? 0 : 1,
      params: $V.getVisParams(values, {
        icon_on: values.icon_on,
        icon_off: values.icon_off,
        icon_default: values.icon_default,
        icon_touch: values.icon_touch,
        icons_add: (icons && icons.length ? icons : []),
        displaymode: values.displaymode,
        showcontrol: values.showcontrol ? 1 : 0,
        fixedvalue: widget ? '' : values.fixedvalue,
        update: widget ? false : (typeof values.update != 'undefined'),
        pincode: values.pincode,
        widget: widget,
        backdrop: $V.isChecked(values.backdrop) ? 1 : 0
      }),
      name: values.name,
      readonly: values.readonly ? 1 : 0,
      notouch: values.notouch ? 1 : 0,
      nobg: values.nobg ? 1 : 0,
      sortorder: values.sortorder.toNumber(),
      cls: values.cls,
      insert: true
    };

    if ($V.checkObjectForm(values, oid)) {
      $V.floorController.register(config, $V.Content, true);
      Ext.getCmp('scada-vis-icons-additional-store').setValue('');
      $V.prepareForm.object();
    }
  });

  $V.editorSubmits.floorlink.on('click', function() {
    if (!$V.checkForms('floorlink')) {
      return;
    }

    var values = $V.editorForms.floorlink.getValues(), config;
    config = {
      floor: $V.floorId,
      object: values.floorid,
      type: 2,
      params: $V.getVisParams(values, {
        url: values.url,
        displaymode: values.displaymode,
        icon: values.icon,
        icon_active: values.icon_active
      }),
      name: values.name,
      notouch: values.notouch ? 1 : 0,
      nobg: values.nobg ? 1 : 0,
      sortorder: values.sortorder.toNumber(),
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
    $V.prepareForm.floorlink();
  });

  $V.editorSubmits.camera.on('click', function() {
    if (!$V.checkForms('camera')) {
      return;
    }

    var values = $V.editorForms.camera.getValues(), config;
    config = {
      floor: $V.floorId,
      object: 0,
      type: 4,
      params: Ext.encode({
        autoopen: values.autoopen ? 1 : 0,
        width: values.width,
        win_width: values.win_width,
        height: values.height,
        win_height: values.win_height,
        src: values.src,
        icon: values.icon
      }),
      name: values.name,
      nobg: values.nobg ? 1 : 0,
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.graph.on('click', function() {
    if (!$V.checkForms('graph')) {
      return;
    }

    var values = $V.editorForms.graph.getValues(), config;
    config = {
      floor: $V.floorId,
      object: values.object.toNumber(),
      type: 5,
      params: Ext.encode({
        autoopen: values.autoopen ? 1 : 0,
        width: values.width,
        win_width: values.win_width,
        height: values.height,
        win_height: values.win_height,
        limit: values.limit.toNumber(),
        icon: values.icon
      }),
      name: values.name,
      nobg: values.nobg ? 1 : 0,
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.label.on('click', function() {
    if (!$V.checkForms('label')) {
      return;
    }

    var values = $V.editorForms.label.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 6,
      params: $V.getVisParams(values),
      name: values.name,
      nobg: 1,
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.image.on('click', function() {
    if (!$V.checkForms('image')) {
      return;
    }

    var values = $V.editorForms.image.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 7,
      params: Ext.encode({
        source: values.source,
        src: (values.source == 'local' ? values.src_local : values.src_remote),
        width: values.width,
        height: values.height,
        link: values.link
      }),
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.gauge.on('click', function() {
    if (!$V.checkForms('gauge')) {
      return;
    }

    var values = $V.editorForms.gauge.getValues(), oid = values.object.toNumber(), config;
    config = {
      floor: $V.floorId,
      object: oid,
      type: 8,
      params: Ext.encode({
        size: values.size
      }),
      readonly: values.readonly ? 1 : 0,
      name: values.name,
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  $V.editorSubmits.iframe.on('click', function() {
    if (!$V.checkForms('iframe')) {
      return;
    }

    var values = $V.editorForms.iframe.getValues(), config;

    config = {
      floor: $V.floorId,
      type: 9,
      params: Ext.encode({
        source: values.source,
        url: values.url,
        width: values.width,
        height: values.height
      }),
      notouch: values.notouch ? 1 : 0,
      name: values.name,
      cls: values.cls,
      insert: true
    };

    $V.floorController.register(config, $V.Content, true);
  });

  Ext.iterate($V.editorResets, function(type, button) {
    button.on('click', function() {
      $V.editorForms[ type ].reset();
      $V.floorController.stopEdit(type);
      $V.prepareAllForms();
      $V.manualResizeReset();
    });
  });

  $V.comboBoxes.object.on('select', $V.prepareForm.object);
  $V.comboBoxes.statusobject.on('select', $V.prepareForm.object);
  $V.comboBoxes.objectwidget.on('select', $V.prepareForm.object);

  $V.comboBoxes.floor.on('select', $V.prepareForm.floorlink);

  Ext.getCmp('scada-vis-object-display-mode').on('select', $V.prepareForm.object);
  Ext.getCmp('scada-vis-editor-object-showcontrol').on('check', $V.prepareForm.object);
  Ext.getCmp('scada-vis-link-display-mode').on('select', $V.prepareForm.floorlink);
  Ext.getCmp('scada-vis-image-source').on('select', $V.prepareForm.image);
  Ext.getCmp('scada-vis-iframe-source').on('select', $V.prepareForm.iframe);

  Scada.initImgSelect($V);
  Scada.initImgSelect($V, 'icon');

  Scada.bindImgSelect($V, 'vis-src-local');
  Ext.each($V.iconSelects, function(id) {
    Scada.bindImgSelect($V, id, 'icon');
  });

  Ext.iterate($V.comboBoxes, function(index, combo) {
    combo.on('afterrender', function(combo) {
      combo.reset();
    });
  });

  $V.TouchOrderTree = Ext.getCmp('scada-vis-touch-tree');

  new Ext.KeyNav(document.body, {
    left: function(e) {
      return $V.keyEvent(e, -1, 0);
    },
    right: function(e) {
      return $V.keyEvent(e, 1, 0);
    },
    up: function(e) {
      return $V.keyEvent(e, 0, -1);
    },
    down: function(e) {
      return $V.keyEvent(e, 0, 1);
    },
    del: function(e) {
      return $V.keyEvent(e);
    }
  });
};

Scada.Modules.push($V);
