#!/usr/bin/haserl
Content-Type: text/javascript

Scada.VisGraphics = {};
var $VG = Scada.VisGraphics;

$VG.iconGrid = new Ext.DataView({
  cls: 'scada-icon-grid',
  store: Scada.Vis.iconStore,
  tpl: new Ext.XTemplate(
    '<tpl for=".">',
      '<div class="scada-icon-wrap scada-image-deletable">',
          '<div class="scada-icon">',
            '<div class="scada-icon-delete" data-id="{icon}" data-mode="icon"></div>',
            '<img src="/scada/resources/icons/{icon}?{mtime}" title="{icon}">',
          '</div>',
        '<div class="scada-icon-title">{icon}</div>',
      '</div>',
    '</tpl>',
    '<div class="x-clear"></div>'
  )
});

$VG.imgGrid = new Ext.DataView({
  cls: 'scada-icon-grid',
  store: Scada.Vis.imgStore,
  tpl: new Ext.XTemplate(
    '<tpl for=".">',
      '<div class="scada-icon-wrap scada-image-deletable">',
          '<div class="scada-icon">',
            '<div class="scada-icon-delete" data-id="{image}" data-mode="image"></div>',
            '<img src="/scada/resources/img/{image}?{mtime}" title="{image}">',
          '</div>',
        '<div class="scada-icon-title">{image}</div>',
      '</div>',
    '</tpl>',
    '<div class="x-clear"></div>'
  )
});

$VG.fontStore = new Ext.data.JsonStore({
  fields: ['id', 'name', 'type'],
  root: 'fonts',
  url: Scada.reqUrl('vis', 'fonts'),
  sortInfo: { field: 'name', direction: 'ASC' }
});

$VG.fontStoreReload = function() {
  $VG.fontStore.reload();
};

$VG.getFontName = function(value, cell, row) {
  var data = row.data;
  return data.name + ' (' + data.type + ')';
};

$VG.fontGrid = new Ext.grid.GridPanel($def.grid({
  id: 'scada-font-grid',
  store: $VG.fontStore,
  border: false,
  cls: 'x-scada-ns-grid',
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: $tr('visgraphics_font_name'), width: .95, renderer: $VG.getFontName },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$VG.deleteBtn = {
  text: $tr('visgraphics_delete_selected'),
  iconCls: 'icon-clear',
  handler: function() {
    var el = $VG.MainPanel.getActiveTab().el, els, mode, files = [];
    els = el.query('.scada-icon-selected .scada-icon-delete');

    if (els.length) {
      Ext.each(els, function(el) {
        files.push(el.getAttribute('data-id'));
        mode = el.getAttribute('data-mode');
      });

      $VG.onDelete({
        files: files,
        mode: mode
      });
    }
  }
};

$VG.deleteBtn.id = 'vis-graphics-delete-icon';
$VG.iconPanel = new Ext.Panel({
  title: $tr('visgraphics_icons'),
  autoScroll: true,
  items: $VG.iconGrid,
  bbar: [
    { text: $tr('visgraphics_add_icon'), iconCls: 'icon-add', id: 'vis-graphics-add-icon-button' },
    '-',
    $VG.deleteBtn
  ]
});

$VG.deleteBtn.id = 'vis-graphics-delete-img';
$VG.imgPanel = new Ext.Panel({
  title: $tr('visgraphics_img'),
  autoScroll: true,
  items: $VG.imgGrid,
  bbar: [
    { text: $tr('visgraphics_add_img'), iconCls: 'icon-add', id: 'vis-graphics-add-img-button' },
    '-',
    $VG.deleteBtn
  ]
});

$VG.fontPanel = new Ext.Panel({
  title: $tr('visgraphics_font'),
  layout: 'fit',
  items: $VG.fontGrid,
  bbar: [
    { text: $tr('visgraphics_font_add'), iconCls: 'icon-add', id: 'vis-graphics-add-font-button' }
  ]
});

$VG.cssPanel = new Ext.Panel({
  title: $tr('visgraphics_css_edit')
});

$VG.MainPanel = new Ext.TabPanel({
  border: false,
  region: 'center',
  margins: '5 0 0 0',
  items: [ $VG.iconPanel, $VG.imgPanel, $VG.fontPanel, $VG.cssPanel ]
});

$VG.deleteGraphics = function(files, mode) {
  Ext.Ajax.request({
    url: Scada.reqUrl('visgraphics', 'delete'),
    success: $VG.update,
    failure: $VG.update,
    params: Scada.encodeData({
      files: files,
      mode: mode
    })
  });
};

$VG.update = function() {
  $VG.addItemEvents();

  Scada.Vis.iconStore.reload();
  Scada.Vis.imgStore.reload();
  $VG.fontStoreReload();

  $VG.MainPanel.setDisabled(false);
};

$VG.uploadFW = new ExtFormWindow({
  id: 'vis-graphics-upload',
  window: { title: $tr('visgraphics_add'), width: 460, height: 210 },
  form: {
    url: Scada.reqUrl('visgraphics', 'save'),
    fileUpload: true,
    items: [
      { name: 'name', fieldLabel: $tr('visgraphics_name'), maskRe: /[\w\-]/ },
      $def.fileUpload({ name: 'file', fieldLabel: $tr('visgraphics_file') }),
      { name: 'mode', xtype: 'hidden' },
      $def.formInfoPanel('visgraphics_help')
    ]
  },
  events: { onFormSubmitComplete: $VG.update, onFormSubmitFailed: $VG.update }
});

$VG.fontUploadError = function() {
  Ext.Msg.alert($tr('error'), $tr('visgraphics_font_error'));
};

$VG.fontUploadFW = new ExtFormWindow({
  id: 'vis-font-upload',
  window: { title: $tr('visgraphics_font_add'), width: 460, height: 160 },
  form: {
    url: Scada.reqUrl('visgraphics', 'font-save'),
    fileUpload: true,
    items: [
      $def.fileUpload({ name: 'file', fieldLabel: $tr('visgraphics_file') }),
      $def.formInfoPanel('visgraphics_font_help')
    ]
  },
  events: { onFormSubmitComplete: $VG.fontStoreReload, onFormSubmitFailed: $VG.fontUploadError }
});

$VG.onDeleteConfirm = function(res) {
  $VG.clearSelection();
  if (res == 'yes') {
    $VG.MainPanel.setDisabled(true);
    Ext.Ajax.request({
      url: Scada.reqUrl('visgraphics', 'delete'),
      success: $VG.update,
      failure: $VG.update,
      params: Scada.encodeData(this)
    });
  }
};

$VG.onDelete = function(data) {
  Ext.Msg.confirm(
    $tr('visgraphics_delete_title'),
    $tr('visgraphics_delete_confirm'),
    $VG.onDeleteConfirm,
    data
  );
};

$VG.onDeleteClick = function() {
  $VG.clearSelection();
  $VG.onDelete({
    files: this.getAttribute('data-id'),
    mode: this.getAttribute('data-mode')
  });
};

$VG.attachImageHover = function(target) {
  var tmp = new Image(), src = target.src;

  tmp.onload = function() {
    if (this.width > 70 || this.height > 70) {
      Ext.QuickTips.register({
        target: target,
        cls: 'x-html-tip',
        text: '<img src="' + this.src + '">'
      });
    }
  }

  tmp.src = src;
};

$VG.setDeleteButtonState = function() {
  var fn = $VG.MainPanel.el.query('.scada-icon-selected').length > 1 ? 'addClass' : 'removeClass';

  Ext.getCmp('vis-graphics-delete-icon')[ fn ]('x-btn-hl');
  Ext.getCmp('vis-graphics-delete-img')[ fn ]('x-btn-hl');
};

$VG.clearSelection = function() {
  var els;

  if ($VG.MainPanel.el) {
    els = $VG.MainPanel.el.query('.scada-icon-selected');
    Ext.get(els).removeClass('scada-icon-selected');

    $VG.setDeleteButtonState();
  }
};

$VG.onElOver = function() {
  var img = this.query('img')[0], tip = $VG.previewEl;
  this.addClass('scada-icon-hover');

  if (img.naturalWidth > 70 || img.naturalHeight > 70) {
    tip.update('<img src="' + img.src + '">');
    tip.show();
    tip.alignTo(this, 'tr?');
  }
};

$VG.onElOut = function() {
  this.removeClass('scada-icon-hover');
  $VG.previewEl.hide()
};

$VG.onElClick = function() {
  this.toggleClass('scada-icon-selected');
  $VG.setDeleteButtonState();
};

$VG.addItemEvents = function() {
  var items = Ext.query('.scada-image-deletable');
  $VG.clearSelection();

  Ext.each(items, function(item) {
    var el = Ext.get(item).removeAllListeners(), delBtn = el.query('.scada-icon-delete')[0];

    el.on('mouseover', $VG.onElOver);
    el.on('mouseout', $VG.onElOut);
    el.on('click', $VG.onElClick);

    Ext.get(delBtn).on('click', $VG.onDeleteClick);
  });
};

$VG.onFontConfirm= function(res) {
  if (res == 'yes') {
    Ext.Ajax.request({
      url: Scada.reqUrl('visgraphics', 'font-delete'),
      success: $VG.fontStoreReload,
      failure: $VG.fontStoreReload,
      params: Scada.encodeData({
        id: this.id
      })
    });
  }
};

$VG.container = new Ext.Container({
  id: 'VisGraphics',
  title: $tr('visgraphics_title'),
  layout: 'border',
  items: $VG.MainPanel
});

$VG.init = function() {
  Scada.Vis.iconStore.on('load', $VG.addItemEvents);
  Scada.Vis.imgStore.on('load', $VG.addItemEvents);

  Ext.getCmp('vis-graphics-add-icon-button').on('click', function() {
    $VG.uploadFW.show();
    $VG.uploadFW.formPanel.form.findField('mode').setValue('icon');
  });

  Ext.getCmp('vis-graphics-add-img-button').on('click', function() {
    $VG.uploadFW.show();
    $VG.uploadFW.formPanel.form.findField('mode').setValue('image');
  });

  Ext.getCmp('vis-graphics-add-font-button').on('click', function() {
    $VG.fontUploadFW.show();
  });

  $VG.MainPanel.on('beforetabchange', function(prev, curr) {
    return curr != $VG.cssPanel;
  });

  $VG.MainPanel.on('afterrender', function() {
    el = $VG.MainPanel.el.query('li')[ 3 ];

    Ext.get(el).on('click', function() {
      window.open('/cgi-bin/scada/editor.cgi?id=css');
    });
  });

  $VG.MainPanel.on('tabchange', $VG.addItemEvents);
  $VG.MainPanel.setActiveTab(0);

  $VG.previewEl = Ext.DomHelper.append(Ext.getBody(), { tag: 'div', cls: 'scada-preview' }, true);

  $VG.fontGrid.on('cellclick', function(grid, r, c) {
    var id = grid.getStore().getAt(r).get('id')
      , field = grid.getColumnModel().getDataIndex(c);

    if (field == 'delete') {
      Ext.Msg.confirm(
        $tr('visgraphics_font_title'),
        $tr('visgraphics_font_confirm'),
        $VG.onFontConfirm,
        { id: id }
      );
    }
  });

  $VG.fontStore.on('load', function() {
    var fonts = [ 'Arial', 'Tahoma', 'Times New Roman', 'Verdana' ], data = [];

    Ext.each($VG.fontStore.getRange(), function(item) {
      var font = item.data.name;
      if (fonts.indexOf(font) < 0) {
        fonts.push(font);
      }
    });

    fonts.sort();

    Ext.each(fonts, function(font) {
      data.push([ font ]);
    });

    Scada.Vis.fontStore.loadData(data);
  });

  $VG.fontStore.load();
};

$VG.onFocus = $VG.update;
Scada.Modules.push($VG);
