#!/usr/bin/haserl
Content-Type: text/javascript

Scada.Utilities = { index: undefined };
var $U = Scada.Utilities;

$U.languageStore = new Ext.data.JsonStore({
  fields: ['code', 'name'],
  root: 'languages',
  url: Scada.reqUrl('general', 'languages')
});

$U.neighbourStore = new Ext.data.JsonStore({
  fields: ['ip', 'name'],
  root: 'neighbours',
  url: Scada.reqUrl('general', 'neighbours')
});

$U.buttons = [];

$U.addBtn = function(id, text, iconCls, cls) {
  $U.buttons.push({
    id: id,
    text: text,
    iconCls: iconCls,
    cls: cls || '',
    xtype: 'button', scale: 'large', iconAlign: 'bottom', minWidth: 120
  });
};

$U.addBtn('utilities-btn-esf-import', $tr('utilities_esf_import'), 'scada-utilities-btn-esf-import');
$U.addBtn('utilities-btn-neighbour-import', $tr('utilities_neighbour_import'), 'scada-utilities-btn-esf-import', 'hidden');
$U.addBtn('utilities-btn-reset', $tr('utilities_reset'), 'scada-utilities-btn-objects-reset');
$U.addBtn('utilities-btn-factory-reset', $tr('utilities_factory_reset'), 'scada-utilities-btn-objects-reset');
$U.addBtn('utilities-btn-datetime', $tr('utilities_time_settings'), 'scada-utilities-btn-datetime');
$U.addBtn('utilities-btn-update', $tr('utilities_update'), 'scada-utilities-btn-update');
$U.addBtn('utilities-btn-backup', $tr('utilities_backup'), 'scada-utilities-btn-backup');
$U.addBtn('utilities-btn-restore', $tr('utilities_restore'), 'scada-utilities-btn-restore');
$U.addBtn('utilities-btn-config', $tr('utilities_config'), 'scada-utilities-btn-config');
$U.addBtn('utilities-btn-config-vis', $tr('utilities_config_vis'), 'scada-utilities-btn-config');

$U.buttons.push({
  id: 'utilities-btn-system',
  text: $tr('utilities_system'),
  iconCls: 'scada-utilities-btn-config',
  handler: function() { window.open('/cgi-bin/flashsys/index.cgi') },
  xtype: 'splitbutton', scale: 'large', iconAlign: 'bottom', minWidth: 120, menu: {
    items: [
      { text: $tr('utilities_config_knx'), iconCls: 'icon-list-view', handler: function() { window.open('/cgi-bin/flashsys/index.cgi#network-knx') } },
      { text: $tr('utilities_config_login'), iconCls: 'icon-list-view', handler: function() { window.open('/cgi-bin/flashsys/index.cgi#system-login') } },
      { text: $tr('utilities_config_remote'), iconCls: 'icon-list-view', handler: function() { window.open('/cgi-bin/flashsys/index.cgi#services-remote') } }
    ]
  }
});

$U.pageReload = function() {
  window.location.reload();
};

$U.resetCB = function() {
  Scada.Objects.update(true);
};

$U.resetFormWindow = new ExtFormWindow({
  id: 'utilities-reset',
  window: {
    title: $tr('utilities_reset'),
    width: 240,
    height: 270
  },
  form: {
    url: Scada.reqUrl('general', 'reset'),
    labelWidth: 180,
    items: [
      { name: 'objects', fieldLabel: $tr('utilities_reset_objects'), xtype: 'checkbox', width: 25 },
      { name: 'objectlogs', fieldLabel: $tr('utilities_reset_objectlogs'), xtype: 'checkbox', width: 25 },
      { name: 'highpriologs', fieldLabel: $tr('utilities_reset_highpriologs'), xtype: 'checkbox', width: 25 },
      { name: 'alerts', fieldLabel: $tr('utilities_reset_alerts'), xtype: 'checkbox', width: 25 },
      { name: 'logs', fieldLabel: $tr('utilities_reset_logs'), xtype: 'checkbox', width: 25 },
      { name: 'errors', fieldLabel: $tr('utilities_reset_errors'), xtype: 'checkbox', width: 25 },
      { name: 'storage', fieldLabel: $tr('utilities_reset_storage'), xtype: 'checkbox', width: 25 }
    ]
  },
  events: { onFormSubmitComplete: $U.resetCB, onFormSubmitFailed: $U.resetCB }
});

$U.importCB = function(form, action) {
  var r = action.result, i, d;

  if (r.imported && r.discarded) {
    i = r.imported.join(', ') || $tr('utilities_import_none');
    d = r.discarded.join(', ') || $tr('utilities_import_none');

    Ext.MessageBox.alert($tr('utilities_import_result_title'), $tr('utilities_import_result', i, d));
  }
  else if (r.error) {
    Ext.MessageBox.alert($tr('utilities_import_result_title'), $tr('utilities_import_error', r.error));
  }

  Scada.Objects.update(true);
};

$U.importFormWindow = new ExtFormWindow({
  id: 'utilities-esf-import',
  window: {
    title: $tr('utilities_esf_import'),
    width: 460,
    height: 180
  },
  form: {
    url: Scada.reqUrl('objects', 'import'),
    fileUpload: true,
    items: [
      $def.fileUpload({ fieldLabel: $tr('utilities_import') }),
      $def.formInfoPanel('utilities_import_help')
    ]
  },
  events: { onFormSubmitComplete: $U.importCB, onFormSubmitFailed: $U.importCB }
});

$U.importNeighbourFormWindow = new ExtFormWindow({
  id: 'utilities-neighbours-import',
  window: {
    title: $tr('utilities_neighbour_import'),
    width: 460,
    height: 180
  },
  form: {
    url: Scada.reqUrl('objects', 'import-neighbours'),
    items: [
      $def.select({
        fieldLabel: $tr('utilities_import_neighbour'), name: 'ip', hiddenName: 'ip', allowBlank: false,
        store: $U.neighbourStore, valueField: 'ip', displayField: 'name', mode: 'local'
      }),
      { name: 'password', fieldLabel: $tr('utilities_import_password'), allowBlank: false }
    ]
  },
  events: { onFormSubmitComplete: $U.importCB, onFormSubmitFailed: $U.importCB }
});

$U.timezones = new Ext.data.ArrayStore({
  fields: ['value'],
  idIndex: 0,
  data: <? lua /lib/genohm-scada/web/general/timezones.lua ?>
});

$U.useCurrentDateTime = function() {
  var d = new Date(), form = $U.datetimeFW.form;
  form.findField('date').setValue(d);
  form.findField('timeh').setValue(d.getHours());
  form.findField('timem').setValue(d.getMinutes());
  form.findField('times').setValue(d.getSeconds());
};

$U.datetimeFW = new ExtFormWindow({
  id: 'utilities-datetime',
  window: {
    title: $tr('utilities_time_settings'),
    width: 460,
    height: 210
  },
  form: {
    url: Scada.reqUrl('general', 'datetime-save'),
    loadUrl: Scada.reqUrl('general', 'datetime-form'),
    items: [
      { name: 'current', xtype: 'displayfield', fieldLabel: $tr('utilities_dt_current') },
      {
        xtype: 'compositefield',
        fieldLabel: $tr('utilities_time'),
        items: [
          { xtype: 'spinnerfield', name: 'timeh', zeroPad: true, padLength: 2, minValue: 0, maxValue: 23, width: 40 },
          { xtype: 'spinnerfield', name: 'timem', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, width: 40 },
          { xtype: 'spinnerfield', name: 'times', zeroPad: true, padLength: 2, minValue: 0, maxValue: 59, width: 40 },
          { xtype: 'button', text: $tr('utilities_use_current'), handler: $U.useCurrentDateTime }
        ]
      },
      { name: 'date', xtype: 'datefield', fieldLabel: $tr('utilities_date'), format: 'd.m.Y', showToday: false },
      $def.select({
        fieldLabel: $tr('utilities_timezone'), name: 'timezone', hiddenName: 'timezone',
        store: $U.timezones, valueField: 'value', displayField: 'value', mode: 'local'
      })
    ]
  }
});

$U.configFW = new ExtFormWindow({
  id: 'utilities-config',
  window: {
    title: $tr('utilities_config'),
    width: 510,
    height: 480
  },
  form: {
    url: Scada.reqUrl('general', 'config-save'),
    loadUrl: Scada.reqUrl('general', 'config-form'),
    labelWidth: 200,
    items: [
      $def.select({
        name: 'language', hiddenName: 'language', fieldLabel: $tr('utilities_language'),
        mode: 'local', store: $U.languageStore, valueField: 'code', displayField: 'name'
      }),
      { name: 'perpage', fieldLabel: $tr('utilities_perpage'), xtype: 'spinnerfield', minValue: 10, maxValue: 50 },
      { name: 'autorange', fieldLabel: $tr('utilities_autorange'), invalidText: $tr('objects_error_address'), allowBlank: false, vtype: 'knxga' },
      $def.select({
        name: 'autoadd', hiddenName: 'autoadd', fieldLabel: $tr('utilities_autoadd'),
        store: [['1',  $tr('utilities_autoadd_enable')], ['0',  $tr('utilities_autoadd_disable')]]
      }),
      { name: 'logobjects', fieldLabel: $tr('utilities_logobjects'), xtype: 'spinnerfield', minValue: 100, maxValue: 10000 },
      $def.select({
        name: 'logpolicy', hiddenName: 'logpolicy', fieldLabel: $tr('utilities_logpolicy'),
        store: [['drop',  $tr('utilities_logpolicy_drop')], ['accept',  $tr('utilities_logpolicy_accept')]]
      }),
      { name: 'logalerts', fieldLabel: $tr('utilities_logalerts'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      { name: 'loglogs', fieldLabel: $tr('utilities_loglogs'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      { name: 'logerrors', fieldLabel: $tr('utilities_logerrors'), xtype: 'spinnerfield', minValue: 50, maxValue: 5000 },
      { name: 'blockly', xtype: 'checkbox', fieldLabel: $tr('utilities_blockly') },
      { name: 'tabsize', xtype: 'spinnerfield', fieldLabel: $tr('utilities_tabsize'), minValue: 2, maxValue: 8, width: 50 },
      $def.formInfoPanel('utilities_config_help'),
      { name: 'mode', xtype: 'hidden', value: 'general' }
    ]
  },
  events: {
    onFormSubmitComplete: $U.pageReload
  }
});

$U.configVisFW = new ExtFormWindow({
  id: 'utilities-config-vis',
  window: {
    title: $tr('utilities_config_vis'),
    width: 510,
    height: 430
  },
  form: {
    url: Scada.reqUrl('general', 'config-save'),
    loadUrl: Scada.reqUrl('general', 'config-form'),
    labelWidth: 200,
    items: [
      $def.select({
        name: 'fullscreen', hiddenName: 'fullscreen', fieldLabel: $tr('utilities_sidebar'),
        store: [['0',  $tr('utilities_sidebar_0')], ['2',  $tr('utilities_sidebar_2')], ['1',  $tr('utilities_sidebar_1')]]
      }),
      $def.select({
        name: 'viscenter', hiddenName: 'viscenter', fieldLabel: $tr('utilities_viscenter'),
        store: [
          ['0',  $tr('utilities_viscenter_disable')],
          ['1',  $tr('utilities_viscenter_enable')],
          ['2',  $tr('utilities_viscenter_enable_auto')],
          ['3',  $tr('utilities_viscenter_horizontal')]
        ]
      }),
      $def.select({
        name: 'visanim', hiddenName: 'visanim', fieldLabel: $tr('utilities_visanim'),
        store: [
          ['', $tr('utilities_visanim_none')],
          ['flipX', 'Flip X'],
          ['flipY', 'Flip Y'],
          ['shrink', 'Shrink'],
          ['expand', 'Expand'],
          ['slideUp', 'Slide Up'],
          ['slideDown', 'Slide Down'],
          ['slideLeft', 'Slide Left'],
          ['slideRight', 'Slide Right'],
          ['slideUpBig', 'Slide Up Big'],
          ['slideDownBig', 'Slide Down Big'],
          ['slideLeftBig', 'Slide Left Big'],
          ['slideRightBig', 'Slide Right Big']
        ]
      }),
      { name: 'visautogrow', xtype: 'checkbox', fieldLabel: $tr('utilities_visautogrow') },
      { name: 'visbgcolor', xtype: 'colorfield', fieldLabel: $tr('utilities_visbgcolor'), width: 100 },
      { name: 'visbgimg', xtype: 'imgtrigger', fieldLabel: $tr('utilities_visbgimg'), id: 'utilities-visbg-select' },
      $extend(Scada.Vis.fontSelect, { width: 250, fieldLabel: $tr('utilities_visfont'), name: 'visfont' }, true),
      { name: 'visdark', xtype: 'checkbox', fieldLabel: $tr('utilities_visdark') },
      { xtype: 'textfield', fieldLabel: $tr('utilities_pincode'), name: 'pincode', width: 80, minLength: 3, maxLength: 8, allowBlank: true, maskRe: /[0-9]/ },
      { name: 'visswipe', xtype: 'checkbox', fieldLabel: $tr('utilities_visswipe') },
      { xtype: 'compositefield', fieldLabel: $tr('utilities_visdim'), items: [
          { name: 'visdim', xtype: 'spinnerfield', allowBlank: true, minValue: 0, maxValue: 60, width: 50 },
          { xtype: 'label', text: $tr('utilities_visdim_minutes') }
        ]
      },
      { xtype: 'compositefield', fieldLabel: $tr('utilities_visdim_opacity'), items: [
          { name: 'visdimopacity', xtype: 'spinnerfield', allowBlank: true, minValue: 10, maxValue: 100, width: 50 },
          { xtype: 'label', text: '%' }
        ]
      },
      { name: 'useralerts', xtype: 'checkbox', fieldLabel: $tr('utilities_useralerts') },
      { name: 'mode', xtype: 'hidden', value: 'vis' }
    ]
  }
});

$U.updateCB = function(form, action) {
  var r = action.result || {}, m = r.success ? 'ok' : ('error_' + (r.error || 'file'));

  if (r.succes) {
    this.hide();
  }

  Ext.Msg.alert(
    $tr('utilities_update'),
    $tr('utilities_update_' + m)
  );
};

$U.updateFW = new ExtFormWindow({
  id: 'utilities-update',
  window: { title: $tr('utilities_update'), width: 460, height: 180 },
  form: {
    url: Scada.reqUrl('general', 'update'),
    fileUpload: true,
    items: [
      $def.fileUpload({ id: 'utilities-update-file', fieldLabel: $tr('utilities_update_file') }),
      $def.formInfoPanel('utilities_update_help')
    ]
  },
  events: { onFormSubmitComplete: $U.updateCB, onFormSubmitFailed: $U.updateCB }
});

$U.restoreCB = function(form, action) {
  var r = action.result || {}, m = r.success ? 'ok' : 'error';

  if (r.succes) {
    this.hide();
  }

  Ext.Msg.alert(
    $tr('utilities_restore'),
    $tr('utilities_restore_' + m)
  );
};

$U.restoreFW = new ExtFormWindow({
  id: 'utilities-restore',
  window: { title: $tr('utilities_restore'), width: 460, height: 180 },
  form: {
    url: Scada.genUrl('utilities-restore'),
    fileUpload: true,
    items: [
      $def.fileUpload({ id: 'utilities-restore-file', fieldLabel: $tr('utilities_restore_file') }),
      $def.formInfoPanel('utilities_restore_help')
    ]
  },
  events: { onFormSubmitComplete: $U.restoreCB, onFormSubmitFailed: $U.restoreCB }
});

$U.container = new Ext.Container({
  id: 'Utilities',
  title: $tr('utilities_title'),
  layout: 'border',
  items: {
    region: 'center',
    layout: 'column',
    ctCls: 'x-column-menu',
    border: false,
    margins: '10 10 10 10',
    items: $U.buttons
  }
});

$U.onResetComplete = function() {
  if (Scada.currentModule.update) {
    Scada.currentModule.update(true);
  }

  if (this.resetType == 'objects') {
    Scada.Objects.update(true);
  }

  Scada.Viewport.setDisabled(false);
};

$U.onResetConfirm = function(res) {
  var t = this.resetType || 'none', data = {};

  if (t == 'objectlogs') {
    if (res == 'cancel') {
      return;
    }

    data.highpriologs = res == 'yes';
  }
  else if (res != 'yes') {
    return;
  }

  data[ this.resetType ] = true;

  Scada.Viewport.setDisabled(true);

  Ext.Ajax.request({
    url: Scada.reqUrl('general', 'reset'),
    success: $U.onResetComplete,
    failure: $U.onResetComplete,
    params: Scada.encodeData(data)
  });
};

$U.reset = function() {
  var t = this.resetType || 'none', buttons = Ext.Msg.YESNO, msg = $tr('utilities_reset_confirm');

  if (t == 'objectlogs') {
    buttons = Ext.Msg.YESNOCANCEL;
    msg = $tr('utilities_reset_confirm_objectlogs');
  }

  Ext.Msg.show({
    title: $tr('utilities_reset_button'),
    msg: msg,
    buttons: buttons,
    fn: $U.onResetConfirm,
    scope: this,
    icon: Ext.MessageBox.QUESTION
  });
};

$U.onFactoryResetComplete = function() {
  Scada.Viewport.setDisabled(false);

  Ext.Msg.alert(
    $tr('utilities_factory_reset'),
    $tr('utilities_factory_reset_alert')
  );
}

$U.onFactoryResetConfirm = function(res) {
  if (res == 'yes') {
    Scada.Viewport.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.genUrl('utilities-reset'),
      success: $U.onFactoryResetComplete,
      failure: $U.onFactoryResetComplete
    });
  }
};

$U.factoryReset = function() {
  Ext.Msg.confirm(
    $tr('utilities_factory_reset'),
    $tr('utilities_factory_reset_confirm'),
    $U.onFactoryResetConfirm
  );
};

$U.checkDT = function() {
  var sec = (new Date()).getTime() / 1000, time = Config.time || 0;
  if (Math.abs(Config.time - sec) >= 86400) {
    Ext.Msg.confirm(
      $tr('warning'),
      $tr('utilities_dt_warning'),
      function(res) {
        if (res == 'yes') {
          $U.datetimeFW.show();
        }
      }
    );
  }
};

$U.onBusStatusConfirm = function(res) {
  if (res == 'yes') {
    Scada.Viewport.setDisabled(true);

    Ext.Ajax.request({
      url: Scada.reqUrl('general', 'bus-set-ip'),
      success: $U.pageReload,
      failure: $U.pageReload
    });
  }
  else {
    $U.checkDT();
  }
};

$U.getFilterText = function(tp, ip) {

  tp = tp.match(/^[adn]$/) ? tp : 'n';
  ip = ip.match(/^[adn]$/) ? ip : 'n';

  return $tr('utilities_filter_ip') + ': ' + $tr('utilities_filter_' + ip) +
  '; ' + $tr('utilities_filter_tp') + ': ' + $tr('utilities_filter_' + tp);
};

$U.onUsageLoad = function(res) {
  var data = Ext.decode(res.responseText), mode = data[ 2 ]
    , status = data[ 3 ], cmp, filter
    , text = $tr('usage', data[ 0 ], data[ 1 ]);

  if (!$U.usageFirstLoad) {
    $U.usageFirstLoad = true;

    if (status === false) {
      Ext.Msg.confirm(
        $tr('utilities_bus_status_title'),
        $tr('utilities_bus_status_text'),
        $U.onBusStatusConfirm
      );
    }
    else {
      $U.checkDT();
    }

    filter = $U.getFilterText(data[ 4 ], data[ 5 ]);

    cmp = Ext.getCmp('objects-filter-text');
    if (cmp.rendered) {
      cmp.update(filter);
    }
    else {
      cmp.text = filter;
    }
  }

  if (mode) {
    text += $tr('utilities_bus_mode_' + mode.toString());

    if (mode == 'tp' && typeof status !== 'undefined') {
      text += ' ' + $tr('utilities_bus_status_' + status.toString());
    }
  }

  Ext.get('scada-usage').update(text);
  setTimeout($U.usageLoad, 10000);
};

$U.usageLoad = function() {
  Ext.Ajax.request({
    url: Scada.reqUrl('general', 'usage'),
    success: $U.onUsageLoad,
    failure: $U.usageLoad,
    timeout: 10000
  });
};

$U.syncComplete = function() {
  $U.mask.hide();
};

$U.sync = function() {
  $U.mask = Ext.Msg.wait($tr('utilities_wait_text'), $tr('utilities_sync'));

  Ext.Ajax.request({
    url: Scada.genUrl('utilities-sync'),
    success: $U.syncComplete,
    failure: $U.syncComplete,
  });
};

$U.init = function() {
  Ext.getCmp('utilities-btn-reset').on('click', $U.resetFormWindow.show, $U.resetFormWindow);
  Ext.getCmp('utilities-btn-factory-reset').on('click', $U.factoryReset );
  Ext.getCmp('utilities-btn-esf-import').on('click', $U.importFormWindow.show, $U.importFormWindow);
  Ext.getCmp('utilities-btn-neighbour-import').on('click', $U.importNeighbourFormWindow.show, $U.importNeighbourFormWindow);
  Ext.getCmp('utilities-btn-datetime').on('click', $U.datetimeFW.show, $U.datetimeFW);
  Ext.getCmp('utilities-btn-update').on('click', $U.updateFW.show, $U.updateFW);
  Ext.getCmp('utilities-btn-config').on('click', $U.configFW.show, $U.configFW);
  Ext.getCmp('utilities-btn-config-vis').on('click', $U.configVisFW.show, $U.configVisFW);
  Ext.getCmp('utilities-btn-restore').on('click', $U.restoreFW.show, $U.restoreFW);

  Ext.getCmp('utilities-btn-backup').on('click', function() {
    var url = Scada.genUrl('utilities-backup');
    window.open(url);
  });

  Ext.each([ 'objects', 'objectlogs', 'alerts', 'errors', 'logs' ], function(resetType) {
    var cmp = Ext.getCmp('reset-' + resetType);
    if (cmp) {
      cmp.resetType = resetType;
      cmp.setHandler($U.reset, cmp);
    }
  });

  $U.languageStore.on('load', function(store, records) {
    if (records.length < 2) {
      return;
    }

    var select = Ext.get('language-select');

    Ext.each(records, function(record, index) {
      var data = record.data, code = data.code || '';
      select.dom.options[ index ] = new Option(data.name, code, code == Config.language, code == Config.language);
    });

    select.on('change', function() {
      var language = select.getValue();
      if (language == Config.language) {
        return;
      }

      Scada.Viewport.disable();
      Ext.Ajax.request({
        url: Scada.reqUrl('general', 'config-save'),
        success: $U.pageReload,
        failure: $U.pageReload,
        params: Scada.encodeData({ language: language })
      });
    });

    Ext.get('language-select-wrap').removeClass('hidden');
  });

  $U.neighbourStore.on('load', function(store, records) {
    if (!records.length) {
      return;
    }

    var select = Ext.get('neighbour-select');

    Ext.each(records, function(record, index) {
      var data = record.data;
      select.dom.options[ index + 1 ] = new Option(data.name, data.ip);
    });

    select.on('change', function() {
      var ip = select.getValue();
      if (ip) {
        window.location = 'http://' + ip + '/';
      }
    });

    Ext.get('neighbour-select-wrap').removeClass('hidden');
    Ext.getCmp('utilities-btn-neighbour-import').removeClass('hidden');
  });

  $U.languageStore.reload();
  $U.neighbourStore.reload();

  if (Scada.warn) {
    Ext.Msg.show({
      title: $tr('password_warn_title'),
      msg: $tr('password_warn_msg'),
      buttons: Ext.Msg.OK,
      icon: Ext.MessageBox.WARNING,
      fn: $U.usageLoad
    });
  }
  else {
    $U.usageLoad();
  }

  Scada.initImgSelect($U);
  Scada.bindImgSelect($U, 'utilities-visbg-select');

  new Ext.Button({
    text: $tr('utilities_sync'),
    renderTo: 'sync-button-wrap',
    handler: $U.sync
  });
};

Scada.Modules.push($U);
