#!/usr/bin/haserl
Content-Type: text/javascript

Scada.modbus = {};
var $MB = Scada.modbus;

$MB.reqUrl = function(request) {
  return '/cgi-bin/scada/general/modbus.cgi?request=' + request;
};

$MB.mainStore = new Ext.data.JsonStore({
  url: $MB.reqUrl('device-list'),
  root: 'data',
  fields: [ 'id', 'name', 'profile', 'proto', 'settings', 'active', 'slave', 'pollinterval' ]
});

$MB.profileStore = new Ext.data.JsonStore({
  url: $MB.reqUrl('profile-list'),
  root: 'data',
  fields: [ 'name', 'description', 'manufacturer' ]
});

$MB.mappingStore = new Ext.data.JsonStore({
  url: $MB.reqUrl('mapping-list'),
  root: 'data',
  fields: [ 'id', 'device', 'name', 'active', 'bus_write', 'bus_address', 'type', 'address', 'datatype', 'writable' ]
});

$MB.mainStoreReload = function() {
  if (!$MB.profileLoaded) {
    $MB.profileStoreReload();
    $MB.profileLoaded = true;
  }
  $MB.mainStore.reload();
};

$MB.mappingStoreReload = function() {
  $MB.mappingStore.reload();
};

$MB.profileStoreReload = function() {
  $MB.profileStore.reload();
};

$MB.objectStoreLoad = function() {
  $MB.mappingStoreReload();
  $O.store.un('load', $MB.objectStoreLoad);
};

$MB.mappingFullReload = function() {
  $O.store.on('load', $MB.objectStoreLoad);
  $O.updateExt = true;
  $O.store.load();
};

$MB.setFormParams = function() {
  var form = $MB.deviceFW.form
    , values = form.getValues()
    , tcp = values.proto == 'tcp';

  form.findField('profile').setDisabled(values.id.toNumber() > 0);
  form.findField('ip').setDisabled(!tcp).setVisible(tcp);
  form.findField('port').setDisabled(!tcp).setVisible(tcp);
  form.findField('slave').setMaxValue(tcp ? 255 : 247);
};

$MB.objRenderer = function(addr, cell, item) {
  var text = '', object;

  if (addr) {
    text = Scada.decodeGroupAddress(addr);
    object = $O.store.getById(addr);

    if (object && object.data.name) {
      text += ' (' + object.data.name + ')';
    }
  }

  return text;
};

$MB.valueRenderer = function(addr) {
  var text = '', object = $O.store.getById(addr), datadec;

  if (object) {
    datadec = object.data.datadec;
    text = Ext.grid.KNXdtColumn.prototype.renderer(datadec, 0, object);
  }

  return text;
};

$MB.protoRenderer = function(proto, cell, item) {
  proto = proto.toUpperCase();

  if (proto == 'TCP') {
    proto += ' ' + item.data.settings;
  }

  return proto;
};

$MB.pollRenderer = function(value) {
  return value > 0 ? value : 'Disabled';
};

$MB.typeMapping = {
  coil: 'Coil',
  discreteinput: 'Discrete input',
  register: 'Holding register',
  inputregister: 'Input register',
};

$MB.typeRenderer = function(type, cell, item) {
  var text = $MB.typeMapping[ type ] || '', data = item.data;
  text += ': ' + data.address;

  if (data.datatype) {
    text += ' (' + data.datatype + ')';
  }

  return text;
};

$MB.addrWriteError = function(form, resp) {
  var status = resp.result.status;
  if (status) {
    Ext.MessageBox.alert('Error', 'Failed to write address: ' + status);
  }
};

$MB.addrFW = new ExtFormWindow({
  id: 'modbus-device-address',
  window: { title: 'Write new device address', width: 320, height: 180 },
  form: {
    url: $MB.reqUrl('device-address'),
    labelWidth: 100,
    items: [
      { name: 'id', fieldLabel: 'Device address', xtype: 'spinnerfield', allowBlank: false, width: 160, value: 1, minValue: 1, maxValue: 247, allowDecimals: false },
      $def.formInfoPanel('Press programming button and click save afterwards. Programming LED will turn off after successful write operation.')
    ]
  },
  events: { onFormSubmitFailed: $MB.addrWriteError }
});

$MB.mainGrid = new Ext.grid.GridPanel($def.grid({
  id: 'modbus-grid-main',
  store: $MB.mainStore,
  ctCls: 'scada-list-view-grid',
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: 'Name', width: 0.25, dataIndex: 'name' },
      { header: 'Profile', width: 0.25, dataIndex: 'profile' },
      { header: 'Connection type', width: 0.20, dataIndex: 'proto', renderer: $MB.protoRenderer  },
      { header: 'Device address', width: 0.10, dataIndex: 'slave' },
      { header: 'Poll interval', width: 0.10, dataIndex: 'pollinterval', renderer: $MB.pollRenderer },
      { header: 'Mapping', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-object-edit', dataIndex: 'mapping' },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  viewConfig: {
    forceFit: true,
    getRowClass: function(record) {
      return record.data.active == '1' ? 'objects-row-normal' : 'objects-row-error';
    }
  },
  bbar: [
    { text: 'Add device', id: 'modbus-add', iconCls: 'icon-add' },
    '-',
    { text: 'RTU settings', id: 'modbus-config', iconCls: 'icon-config' },
    '-',
    { text: 'Profiles', id: 'modbus-profile-list', iconCls: 'icon-list-view' },
    '-',
    { text: 'Write address', id: 'modbus-address', iconCls: 'icon-run' }
  ]
}));

$MB.mappingGrid = new Ext.grid.GridPanel($def.grid({
  id: 'modbus-grid-mapping',
  store: $MB.mappingStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  border: false,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: 'Name', width: 0.25, dataIndex: 'name' },
      { header: 'Linked to object', width: 0.25, dataIndex: 'bus_address', renderer: $MB.objRenderer },
      { header: 'Current value', width: 0.25, dataIndex: 'bus_address', renderer: $MB.valueRenderer },
      { header: 'Type', width: 0.20, dataIndex: 'type', renderer: $MB.typeRenderer },
      { header: '', width: 0.05, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  })
}));

$MB.mappingWindow = new Ext.Window($def.window({
  items: $MB.mappingGrid
}));

$MB.profileGrid = new Ext.grid.GridPanel($def.grid({
  id: 'modbus-grid-profile',
  store: $MB.profileStore,
  ctCls: 'scada-list-view-grid',
  loadMask: true,
  border: false,
  colModel: new Ext.grid.ColumnModel({
    columns: [
      { header: 'Profile', width: 0.20, dataIndex: 'name' },
      { header: 'Description', width: 0.40, dataIndex: 'description' },
      { header: 'Manufacturer', width: 0.30, dataIndex: 'manufacturer' },
      { header: '', width: 0.10, xtype: 'boolclscolumn', falseCls: 'scada-cell-delete', dataIndex: 'delete' }
    ]
  }),
  bbar: [
    { text: 'Add profile', id: 'modbus-profile-add', iconCls: 'icon-add' }
  ]
}));

$MB.profileWindow = new Ext.Window($def.window({
  width: 640,
  height: 400,
  title: 'Profiles',
  items: $MB.profileGrid
}));

$MB.profileSaveError = function() {
  Ext.MessageBox.alert('Error', 'Invalid profile selected');
};

$MB.profileFW = new ExtFormWindow({
  id: 'modbus-profile-form',
  window: {
    title: 'Add profile',
    width: 460,
    height: 120
  },
  form: {
    url: $MB.reqUrl('profile-save'),
    fileUpload: true,
    items: [
      $def.fileUpload({ id: 'modbus-profile-file', fieldLabel: 'File' }),
    ]
  },
  events: { onFormSubmitComplete: $MB.profileStoreReload, onFormSubmitFailed: $MB.profileSaveError }
});

$MB.container = new Ext.Container({
  id: 'modbus',
  title: 'Modbus',
  layout: 'border',
  items: $MB.mainGrid
});

$MB.profileSelect = $def.select({
  fieldLabel: 'Profile',
  name: 'profile',
  allowBlank: false,
  hiddenName: 'profile',
  valueField: 'name',
  displayField: 'name',
  mode: 'local',
  store: $MB.profileStore
});

$MB.deviceSaveError = function() {
  Ext.MessageBox.alert('Error', 'RTU (serial) is not enabled');
};

$MB.deviceFW = new ExtFormWindow({
  id: 'modbus-form',
  window: {
    title: 'Modbus device',
    width: 460,
    height: 280
  },
  form: {
    url: $MB.reqUrl('device-save'),
    loadUrl: $MB.reqUrl('device-form'),
    items: [
      {
        id: 'modbus-device-proto',
        xtype: 'radiogroup',
        fieldLabel: 'Connection type',
        value: 'rtu',
        items: [
          { boxLabel: 'RTU (RS-485)', name: 'proto', inputValue: 'rtu' },
          { boxLabel: 'TCP/IP', name: 'proto', inputValue: 'tcp' }
        ]
      },
      { name: 'name', fieldLabel: 'Name', allowBlank: false },
      $MB.profileSelect,
      { name: 'ip', fieldLabel: 'IP', allowBlank: false, maxLength: 15, width: 130 },
      { name: 'port', fieldLabel: 'Port', xtype: 'spinnerfield', minValue: 1, maxValue: 65535, width: 65, allowBlank: false, value: 502 },
      { name: 'slave', fieldLabel: 'Device address', xtype: 'spinnerfield', minValue: 0, maxValue: 247, width: 65, allowBlank: false, value: 1 },
      { name: 'pollinterval', fieldLabel: 'Poll interval (seconds)', xtype: 'spinnerfield', minValue: 1, maxValue: 1800, width: 65, allowBlank: true, value: 5 },
      { name: 'id', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $MB.mainStoreReload, onFormSubmitFailed: $MB.deviceSaveError, onFormLoadComplete: $MB.setFormParams }
});

$MB.baudrates = new Ext.data.ArrayStore({
  fields: ['value'],
  data: [
    ['1200'], ['2400'], ['4800'], ['9600'], ['19200'], ['38400'],
    ['57600'], ['115200'], ['230400'], ['460800'], ['500000']
  ]
});

$MB.baudSelect = $def.select({
  fieldLabel: 'Baud rate',
  name: 'baudrate',
  allowBlank: false,
  hiddenName: 'baudrate',
  valueField: 'value',
  displayField: 'value',
  mode: 'local',
  store: $MB.baudrates
});

$MB.parities = new Ext.data.ArrayStore({
  fields: ['value', 'display'],
  data: [
    ['0', 'None (1 stop bit)'],
    ['1', 'Odd'],
    ['2', 'Even'],
    ['3', 'None (2 stop bits)']
  ]
});

$MB.paritySelect = $def.select({
  fieldLabel: 'Parity',
  name: 'parity',
  allowBlank: false,
  hiddenName: 'parity',
  valueField: 'value',
  displayField: 'display',
  mode: 'local',
  store: $MB.parities
});

$MB.configReset = function() {
  $MB.configFW.form.setValues({
    port: '',
    baudrate: '115200',
    parity: '2',
    duplex: 'H'
  });
};

$MB.configFW = new ExtFormWindow({
  id: 'modbus-config-form',
  window: {
    title: 'Modbus RTU settings',
    width: 460,
    height: 270
  },
  form: {
    url: $MB.reqUrl('config-save'),
    loadUrl: $MB.reqUrl('config-form'),
    items: [
      { name: 'enabled', xtype: 'checkbox', fieldLabel: 'RTU (serial) enabled', inputValue: '1' },
      { name: 'port', fieldLabel: 'Port', allowBlank: true },
      $MB.baudSelect,
      $MB.paritySelect,
      {
        xtype: 'radiogroup',
        fieldLabel: 'Duplex',
        value: 'H',
        columns: 3,
        items: [
          { boxLabel: 'Half', name: 'duplex', inputValue: 'H' },
          { boxLabel: 'Full', name: 'duplex', inputValue: 'F' }
        ]
      },
      { xtype: 'button', text: 'Reset to defaults', autoWidth: true, handler: $MB.configReset },
      $def.formInfoPanel('Leave port empty for automatic detection')
    ]
  }
});

$MB.setFreeAddress = function() {
  var addr, list = [];

  $MB.mainStore.each(function(row) {
    var data = row.data;
    if (data.proto == 'rtu') {
      list[ data.slave ] = true;
    }
  });

  for (addr = 1; list[ addr ] && addr <= 247; addr++);
  addr = Math.min(addr, 247);

  $MB.addrFW.form.findField('id').setValue(addr);
}

$MB.autoFillAddress = function(btn) {
  var combo = btn.combo, store = [], value, form = $MB.mappingFW.form;
  combo.setValue('');

  Ext.iterate(form.getValues(), function(field, value) {
    if (value && field.indexOf('address') == 0) {
      if (value.toNumber() != value) {
        value = Scada.encodeGroupAddress(value);
      }

      store.push(value);
    }
  });

  value = Scada.findFreeAddress(store);
  if (value) {
    combo.setValue(Scada.decodeGroupAddress(value)).fireEvent('change', combo);
  }
};

$MB.createAddressSelect = function() {
  var config, combo, btn;

  config = $def.select({
    id: 'mobdus-object',
    name: 'bus_address',
    hiddenName: 'bus_address',
    fieldLabel: 'Link to object',
    editable: true,
    forceSelection: false,
    allowBlank: true,
    mode: 'local',
    vtype: 'knxgaName',
    valueField: 'id',
    displayField: 'text',
    store: $O.extObjStore,
    matchAny: true,
    width: 223
  });

  combo = new Ext.form.ComboBox(config);
  btn = {
    combo: combo,
    handler: $MB.autoFillAddress,
    iconCls: 'icon-add',
    tooltip: 'Create new object',
    tooltipType: 'title',
    width: 16,
    xtype: 'button'
  };

  return {
    items: [ combo, btn ],
    xtype: 'compositefield',
    width: 250
  };
};

$MB.mappingFW = new ExtFormWindow({
  id: 'modbus-mapping-form',
  window: {
    width: 460,
    height: 270
  },
  form: {
    url: $MB.reqUrl('mapping-save'),
    loadUrl: $MB.reqUrl('mapping-form'),
    items: [
      { name: 'name', fieldLabel: 'Name', allowBlank: false },
      $MB.createAddressSelect(),
      { name: 'bus_write', xtype: 'checkbox', fieldLabel: 'Write to bus', inputValue: '1' },
      { name: 'tags', xtype: 'tagbox', fieldLabel: $tr('objects_tags') },
      { name: 'comment', xtype: 'textarea', fieldLabel: 'Comments' },
      { name: 'id', xtype: 'hidden' }
    ]
  },
  events: { onFormSubmitComplete: $MB.mappingFullReload, onFormSubmitFailed: $MB.mappingStoreReload }
});

$MB.onConfirm = function(res) {
  if (res == 'yes') {
    Ext.Ajax.request({
      url: $MB.reqUrl('device-delete'),
      success: $MB.mainStoreReload,
      failure: $MB.mainStoreReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$MB.onMappingConfirm = function(res) {
  if (res == 'yes') {
    $MB.mappingGrid.loadMask.show();

    Ext.Ajax.request({
      url: $MB.reqUrl('mapping-delete'),
      success: $MB.mappingStoreReload,
      failure: $MB.mappingStoreReload,
      params: Scada.encodeData({ id: this.id })
    });
  }
};

$MB.onProfileConfirm = function(res) {
  if (res == 'yes') {
    $MB.profileGrid.loadMask.show();

    Ext.Ajax.request({
      url: $MB.reqUrl('profile-delete'),
      success: $MB.profileStoreReload,
      failure: $MB.profileStoreReload,
      params: Scada.encodeData({ profile: this.profile })
    });
  }
};

$MB.onFocus = function() {
  if (!$MB.ready) {
    $MB.mainStore.load();
    $MB.profileStore.load();

    $MB.ready = true;
  }
};

$MB.init = function() {
  Ext.getCmp('modbus-add').on('click', function() {
    $MB.deviceFW.setLoadParams({ id: 0 });
    $MB.deviceFW.show();
    $MB.setFormParams();
  });

  Ext.getCmp('modbus-config').on('click', function() {
    $MB.configFW.show();
  });

  Ext.getCmp('modbus-address').on('click', function() {
    $MB.addrFW.show();
    $MB.setFreeAddress();
  });

  Ext.getCmp('modbus-profile-list').on('click', function() {
    $MB.profileWindow.show();
  });

  Ext.getCmp('modbus-profile-add').on('click', function() {
    $MB.profileFW.show();
  });

  Ext.getCmp('modbus-device-proto').on('change', $MB.setFormParams);

  $MB.mainGrid.on('cellclick', function(grid, row, col) {
    var item = grid.getStore().getAt(row)
      , id = item.get('id').toNumber()
      , field = grid.getColumnModel().getDataIndex(col)
      , size = Scada.Viewport.getSize();

    if (field == 'delete') {
      Ext.Msg.confirm(
        $tr('Delete device'),
        $tr('Are you sure you want to delete this device?'),
        $MB.onConfirm,
        { id: id }
      );
    }
    else if (field == 'mapping') {
      $MB.mappingStore.setBaseParam('id', id);
      $MB.mappingStore.load();

      $MB.mappingWindow.setTitle('Object mapping for ' + item.data.name);
      $MB.mappingWindow.setSize(size.width - 200, size.height - 200).show().center();
    }
    else {
      $MB.deviceFW.setLoadParams({ id: id });
      $MB.deviceFW.show();
    }
  });

  $MB.mappingGrid.on('cellclick', function(grid, row, col) {
    var item = grid.getStore().getAt(row)
      , id = item.get('id').toNumber()
      , field = grid.getColumnModel().getDataIndex(col);

    if (field == 'delete') {
      Ext.Msg.confirm(
        $tr('Remove mapping'),
        $tr('Are you sure you want to remove mapping for this object?'),
        $MB.onMappingConfirm,
        { id: id }
      );
    }
    else {
      $MB.mappingFW.window.setTitle('Mapping for ' + item.data.name);

      $MB.mappingFW.setLoadParams({ id: id });
      $MB.mappingFW.show();
    }
  });

  $MB.profileGrid.on('cellclick', function(grid, row, col) {
    var item = grid.getStore().getAt(row)
      , field = grid.getColumnModel().getDataIndex(col);

    if (field == 'delete') {
      Ext.Msg.confirm(
        $tr('Delete profile'),
        $tr('Are you sure you want to delete this device profile?'),
        $MB.onProfileConfirm,
        { profile: item.get('name') }
      );
    }
  });

  $MB.reloaderTask = { run: $MB.mainStoreReload, interval: 6000 };
};

Scada.Modules.push($MB);
